#!/usr/bin/perl
#

use strict;
use warnings;

 #Use Perl modules
use Net::IP;
use Net::DNS;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use DBI;
use Getopt::Std;
use Switch;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/pop_host_stats.log";

my $debug           = 0;
my $log_level       = 5;
my $track_progress  = 0;
my $time_to_die     = 0;
my $use_scanlite    = 0;
my $notify_by_email = 1;


#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

my ( $dbh );

main( );
		
sub main {
    my ( $dbh, $sql,$sql2, $sth_sel, $sth_sel2, $sth_upd );



        open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
            *STDERR=*LOG;
            *STDOUT=*LOG;


    $dbh = conn_db();

    my %hostHash;

    my ($data, $id, $sql_loc) = "";

    my ( $report_id, $host_id, $hostname, $hostip, $os, $scantime, $plugin_id, $os_msg ) = "";

    my ( $host, $domain, $mac_address ) = ""; 

    $sql = qq{ SELECT id, hostip, hostname, os
	FROM hosts WHERE report_id='0' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    while ( ( $host_id, $hostip, $host, $os )= $sth_sel->fetchrow_array ) {

	if ( defined( $host ) && $host ne "" ) {
            logwriter( "LOOKING UP HOST $host", 5 );

	    $sql = qq{ SELECT max( report_id ) FROM nessus_results
		WHERE hostname = '$host' };
            logwriter( $sql, 5 );
	    $sth_sel2 = $dbh->prepare( $sql );
	    $sth_sel2->execute;
	    ( $report_id ) = $sth_sel2->fetchrow_array ( );

	    if ( defined($report_id) && $report_id > 0 ) {
		
		$sql = qq{ SELECT scantime, hostip FROM nessus_results
		    WHERE report_id = '$report_id' AND hostname = '$host' LIMIT 1 };
                logwriter( $sql, 5 );
	        $sth_sel2 = $dbh->prepare( $sql );
	        $sth_sel2->execute;
	        ( $scantime, $hostip ) = $sth_sel2->fetchrow_array ( );

		$sql = qq{ SELECT msg FROM nessus_results
		    WHERE report_id = '$report_id' AND hostname = '$host'
		    AND scriptid='11936' LIMIT 1 };
                logwriter( $sql, 5 );
	        $sth_sel2 = $dbh->prepare( $sql );
	        $sth_sel2->execute;
	        ( $os_msg ) = $sth_sel2->fetchrow_array ( );

		$os = extract_os( $os_msg );

		my %RISKS = ( 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0);

		if ( defined( $report_id )) {
		    $sql = qq{ SELECT count(risk) as count, risk FROM `nessus_results` 
			WHERE report_id='$report_id' AND hostname='$host'
	        	AND falsepositive<>'Y' AND scriptid <> 10180 and msg <> '' GROUP BY risk };
                    logwriter( $sql, 5 );
	    	    $sth_sel2 = $dbh->prepare( $sql );
		    $sth_sel2->execute;
		    while( my ($count, $risk )=$sth_sel2->fetchrow_array) {
		        $RISKS{"$risk"} = $count;
		    }
		}

		my $scan_state = "Passed";
		my $risk_value = $RISKS{1} + $RISKS{2};

		if ( $risk_value > 0 ) { $scan_state = "Pending Fixes"; }

		$sql = "UPDATE hosts SET hostip='$hostip', os='$os', scanstate='$scan_state', 
		    lastscandate='$scantime', report_id = '$report_id' WHERE id='$host_id';";
		safe_db_write ( $sql, 4 );

		$sql = qq{ SELECT vs_id FROM host_stats
		    WHERE report_id = '$report_id' AND host_id = '$host_id' LIMIT 1 };
                logwriter( $sql, 5 );
	        $sth_sel2 = $dbh->prepare( $sql );
	        $sth_sel2->execute;
	        my ( $vs_id ) = $sth_sel2->fetchrow_array ( );



                if ( defined($vs_id ) && $vs_id  > 0) {    #DUPE
		
		} else { #NOT A DUPE
                   #ADD ENTRY TO VULN_SUMMARY TABLE
                   $sql = "INSERT INTO host_stats ( report_id, host_id, hostip, hostname, scantime, 
	              vSerious, vHigh, vMed, vMedLow, vLowMed, vLow, vInfo ) VALUES (
                      '$report_id', '$host_id', '$hostip', '$host', '$scantime', 
                      '$RISKS{1}', '$RISKS{2}', '$RISKS{3}', '$RISKS{4}', '$RISKS{5}', '$RISKS{6}', '$RISKS{7}' );";
                   safe_db_write ( $sql, 4 );
                }

                update_incidents ( $host_id, $hostip, $report_id );


		( $report_id, $hostname, $host, $hostip, $os, $scantime, $os_msg ) = ""; 

	    }  
	}
    }
}



#called to update incident tracker, against most recent scan results
sub update_incidents {
    my ( $host_id, $hostip, $report_id ) = @_;

    my ( $sql, $sql2, $sth_sel, $sth_sel2, $scantime );

    #$nessus_plugins

    #INSERT /UPDATE EXISTING INCIDENTS
    my $now = getCurrentDateTime("datetime");

    $sql = qq{ SELECT result_id, scriptid, risk, service, msg FROM nessus_results 
        WHERE report_id='$report_id' AND hostip='$hostip' AND risk <='2' AND falsepositive<>'Y' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $result_id, $scriptid, $risk, $service, $msg )=$sth_sel->fetchrow_array) {
        $sql2 = qq{ SELECT id FROM I3_Incidents
	    WHERE host_id='$host_id' AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
        logwriter( $sql2, 5 );
        $sth_sel2 = $dbh->prepare( $sql2 );
        $sth_sel2->execute;
	my ( $incident_id ) = $sth_sel2->fetchrow_array;

	if ( defined($incident_id ) && $incident_id ne "" ) {
	    $sql2 = qq{ UPDATE I3_Incidents SET result_id='$result_id', msg='$msg' WHERE id="$incident_id" };
	} else {
	    $sql2 = qq{ INSERT INTO I3_Incidents ( host_id, result_id, scriptid, service, risk, msg, date_open ) VALUES (
	        '$host_id', '$result_id', '$scriptid', '$service', '$risk', '$msg', '$now' );};
	}
        safe_db_write ( $sql2, 4 );

    }


}




sub extract_os {
    # VER: 1.0 MODIFIED: 11/15/07 15:43
    my ( $txt_msg ) = @_;

    my $os = "";

    my @arrMSG = split /\n/, $txt_msg;

    foreach my $line (@arrMSG) {
        $line =~ s/\#.*$//;
       	chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line =~ "Remote operating system") {
	    my @temp=split(/:/,$line,2);
	    $temp[1] =~ s/\s+$//;
	    $temp[1] =~ s/^\s+//;
	    $os = $temp[1];
	} else { next; }
    }

    if ( defined( $os ) ) {
	return $os;
    } else {
	return "";
    }    

}

sub extract_hostname {
    my ( @arrMSG ) = @_;

    my $hostname = "";

    foreach my $line (@arrMSG) {
        $line =~ s/\#.*$//;
       	chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line =~ "Computer name") {
	    my @temp=split(/=/,$line,2);
	    $temp[0] =~ s/\s+$//;
	    $temp[0] =~ s/^\s+//;
	    $hostname = $temp[0];
	} else { next; }
    }

    if ( defined( $hostname ) ) {
	return $hostname;
    } else {
	return "";
    }    

}


#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
