#!/usr/bin/perl
#
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##


use strict;
use FindBin qw($RealBin);

sub usage
{
    print "Usage: install [options]\n";
    print "	--noclean		Don't remove previous installation files\n";
    print "	--verbose		Display more information during installation\n";
    print "	--remove		Remove Inprotect from system\n";
    print "	--help			Show this message\n";
}

use Getopt::Long;
my ($withHelp, $withoutClean, $verbose, $remove);
(usage, exit 1) if !GetOptions("help" => \$withHelp,
                               "noclean" => \$withoutClean,
                               "verbose" => \$verbose,
                               "remove" => \$remove);

(usage, exit 1) if ($ARGV[0]);

sub verbosePrint {
    print @_,"\n" if $verbose;
}

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

use vars qw/%CONFIG %DEFAULTS %UPATH/;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Redirect output to the log file
my $installlog = "/tmp/inprotect_install.log";

#GLOBAL VARIABLES
my $debug              = 0;
my $log_level          = 4;

my ( $dbh );

our $binName = 'Inprotect';
chdir $RealBin;

#SETUP DEFAULT VARIABLES
$DEFAULTS{'REV'}          = qx/cat REV/;
chop ( $DEFAULTS{'REV'} );
$DEFAULTS{'DEFROOT'}      = $RealBin;
$DEFAULTS{'DEFROOT'}      =~ s/\/install//;
$DEFAULTS{'DEFWEB'}       = $DEFAULTS{'DEFROOT'} . "/html";
$DEFAULTS{'DBFILE'}       = $DEFAULTS{'DEFROOT'} . "/install/sql/clean_install_inprotect_" . $DEFAULTS{'REV'} . ".sql";
$DEFAULTS{'DEFMYSQL'}     = qx/which mysql 2>\/dev\/null/;
chop ( $DEFAULTS{'DEFMYSQL'} );
$DEFAULTS{'DEFNESSUS'}    = qx/which nessus 2>\/dev\/null/;
chop ( $DEFAULTS{'DEFNESSUS'} );
$DEFAULTS{'DEFNESUPDATE'} = qx/which nessus-update-plugins 2>\/dev\/null/;
chop ( $DEFAULTS{'DEFNESUPDATE'} );
$DEFAULTS{'DEFREQUE'}     = "3";
$DEFAULTS{'DEFDBHOST'}    = "localhost";
$DEFAULTS{'DEFDATABASE'}  = "inprotect2";
$DEFAULTS{'DEFDBUSER'}    = "root";
$DEFAULTS{'DEFDBPASS'}    = "";
$DEFAULTS{'DEFNESHOST'}   = "localhost";
$DEFAULTS{'DEFNESPORT'}   = "1241";
$DEFAULTS{'DEFREQSSL'}    = "1";
$DEFAULTS{'LOCALUPDATE'}   = "1";
$DEFAULTS{'DBI'}          = qx/dd if=\/dev\/urandom count=1 2> \/dev\/null | uuencode -m - | sed -ne 2p | cut -c-56/;

if ( $DEFAULTS{'DEFNESSUS'} eq "" && -e "/opt/nessus/bin/nessus" ) {
    $DEFAULTS{'DEFNESSUS'} = "/opt/nessus/bin/nessus";
}
if ( $DEFAULTS{'DEFNESUPDATE'} eq "" && -e "/opt/nessus/sbin/nessus-update-plugins" ) {
    $DEFAULTS{'DEFNESUPDATE'}= "/opt/nessus/sbin/nessus-update-plugins";
}

Main();
exit;

sub Main {

   my $input = "";

   my $failed = checkDependencies();
   if ( $failed == "1" ) {
      print "System Does not have the required Dependancies, Installation can not continue!\n\n";
      print "Attempt auto-Install Missing Dependancies? (y/n)";
      my $input = <STDIN>;
      chomp $input;
      if ( defined($input) && $input =~ /y/i ) {
         installDependencies( );
         exit;
      } else {
         print "!! PLEASE INSTALL THE MISSING PERL MODULES BEFORE CONTINUING!!\n\n";
         print "example:\n";
         print "
cpan shell -- CPAN exploration and modules installation (v1.7602)
ReadLine support available (try 'install Bundle::CPAN')
at cpan> run the following list

install Bundle::CPAN
install DBI
install Term::ReadKey
install Config::IniFiles
install Parallel::ForkManager
install MIME::Lite
install Date::Calc
install Crypt::CBC
install MIME::Base64
install IO::Socket
install IO::Socket::SSL
install POSIX
install Socket
install Crypt::Blowfish
install Net::IP
install Net::Nslookup
install Net::Netmask
install Net::Telnet

pear upgrade pear
pear install Crypt_CBC
pear install Mail_Mime
pear install Mail_mimeDecode
\n";
         exit;
      }
      exit;
   }

   while ( $input !~ /q/i ) {
      showMenu();
      $input = <STDIN>;
      chomp $input;
      if ($input eq '1') {
         setupConfigs();
         exit;
      }elsif ($input eq '2') {
         setupConfigs();
         initDB();
         exit;
      }elsif ($input eq '3') {
         upgrade_schema();
         exit;
      }elsif ($input eq '4') {
         setupConfigs();
         initDB();
         migrate_data();
         exit;
      } else {
         print "Invalid Selection: TRY AGAIN!!!!\n\n";
      }
   }
}

sub upgrade_schema {
   my ( $sql, $sth_sel );

   &load_configs("/etc/inprotect.cfg");
   $dbh = conn_db();


   #USE SAFE QUERY SO SCRIPT DOESN'T CROAK IF USER NOT RUNNING 1.X GENERATION (REALLY 0.3X IS SUFFICENT / BUT MUST BE 1.X SCHEMA)
   $sql = qq{ SELECT settingValue FROM inprotect_settings WHERE settingName='version' LIMIT 1 };
   my @row = safe_db_query( $sql );

   if ( $DEFAULTS{'REV'} eq $row[0] ) { 
      print "DB SCHEMA IS UP TO DATE.  Current REV = " . $DEFAULTS{'REV'} . "\n\n";
      disconn_db($dbh);
      exit;
   }

   print "WELCOME TO INPROTECT UPGRADE SCHEMA\n\n";
   print "INPROTECT CURRENT REV: " . $DEFAULTS{'REV'} . "\n";
   print "         DETECTED REV: " . $row[0] . "\n\n";
   print "Continue Upgrade (y/n )";


   my $input = <STDIN>;
   chomp $input;
   if ( defined($input) && $input =~ /y/i ) {
      my ( $configfile ) = "sql/UPGRADE_PATH";
      my $noconfig=0;
      open(CONF,"<$configfile") || $noconfig++;
      my @CONFILE=<CONF>;
      close(CONF);
      if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
      foreach my $line (@CONFILE) {
         $line =~ s/\#.*$//;
         chomp($line);
         $line =~ s/\s+$//;
         $line =~ s/^\s+//;
         if ($line eq "") { next; }
         my @temp=split(/:/,$line,2);
         print "1=[" . $temp[0] . "]\t2=[" . $temp[1]. "]\n";
         if ($temp[1] ne "") { $UPATH{$temp[0]}=$temp[1]; }
      }

      my $continue = "1";
      while ( $continue eq "1" ) {
         $sql = qq{ SELECT settingValue FROM inprotect_settings WHERE settingName='version' LIMIT 1 };
         my @trow = safe_db_query( $sql );
         if ( $UPATH{$trow[0]} ne "" ) {
           my $sfile = "sql/" .$UPATH{$trow[0]};
           if ( -e $sfile ) {
              shell_ex ( "$DEFAULTS{'DEFMYSQL'} -u$CONFIG{'DATABASEUSER'} -p$CONFIG{'DATABASEPASSWORD'} $CONFIG{'DATABASENAME'} < $sfile",
                 "IMPORTING $sfile INTO DATABASE $CONFIG{'DATABASENAME'}" );
           } else {
              $continue = "0";   #PATCH SQL SCRIPT MISSING
           }
         } else {
            $continue = "0";     #SHOULD HAVE COMPLETED THE PATCH SEQUENCE
         }
      }

      $sql = qq{ SELECT settingValue FROM inprotect_settings WHERE settingName='version' LIMIT 1 };
      @row = safe_db_query( $sql );
      if ( $DEFAULTS{'REV'} == $row[0] ) { 
         print "DB SCHEMA SUCCESSFULLY UPDATED to Current REV = " . $DEFAULTS{'REV'} . "\n\n";
         disconn_db($dbh);
         exit;
      } else {
         print "DB SCHEMA UPDATE FAILED to REV = " . $DEFAULTS{'REV'} . "\n";
         print "         DETECTED REV: " . $row[0] . "\n\n";
         print "PLEASE NOTIFY INPROTECT TEAM\n\n";
         disconn_db($dbh);
         exit;

      }

      disconn_db($dbh);
      exit;
   } else {
      print "Be sure to keep your schema current as to ensure program functions as intended\n\n";
      print "Thanks, The Inprotect Team\n";
      print "\n";
      disconn_db($dbh);
      exit;
   }


   exit;

}

sub setup_first_server {

   my ($a, $cmd ) = "";

   print "Adding First Nessus Server\n";
   print "\n";

   #GET NESSUS NAME
   get_value ( "nessusname", "", "Enter SERVER NAME", 0 );
   #GET NESSUS HOST
   get_value ( "nessushost", "DEFNESHOST", "Enter SERVER IP", 0 );
   #GET NESSUS PORT
   get_value ( "nessusport", "DEFNESPORT", "Enter Nessus Port", 0 );
   #GET NESSUS USER
   get_value ( "nessususer", "", "Enter Nessus User", 0 );
   #GET NESSUS PASSWORD
   get_value ( "nessuspass", "", "Enter Nessus Password", 0 );


   print "Preping Import First Scan Node\n";
   my $addSERVER = qx/cat configs\/add_first_server.sql/;
   $addSERVER =~ s/<SERVERNAME>/$CONFIG{'nessusname'}/;
   $addSERVER =~ s/<SERVERIP>/$CONFIG{'nessushost'}/;
   $addSERVER =~ s/<NESSUSPORT>/$CONFIG{'nessusport'}/;
   $addSERVER =~ s/<NESSUSUSER>/$CONFIG{'nessususer'}/;
   $addSERVER =~ s/<NESSUSPASS>/$CONFIG{'nessuspass'}/;


   my $sfile = "/tmp/add_first_server.sql";
   open (iCFGFILE, ">$sfile" );
   print iCFGFILE ( $addSERVER );
   close (iCFGFILE); 

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} < /tmp/add_first_server.sql",
      "IMPORTING $sfile INTO DATABASE $CONFIG{'mysqldb'}" );

   shell_ex ( "rm -rf /tmp/add_first_server.sql" , "" );

   print "IMPORTED FIRST SERVER\n";
   print "\n";
   print "After Updating Plugins:  The following need performed\n";
   print "  - Login as Admin / Change Password\n";
   print "  - Setup additonal Admin/User Accounts\n";
   print "  - Setup Zones\n";
   print "  - Setup Profiles\n";
   print "  - Run Test Scans\n";
   print "  - Consider Addons to Nessus: [THC - Hydra / Nikto ]\n";
   print "\n";
   print "Tell you friends about Inprotect!\n";
   print "\n";
   print "Proceeding to run UpdatePlugins: $CONFIG{'IROOT'}/sbin/updateplugins_1.00.pl\n";
   print "\n";
   print "Please Wait ... (Updating Plugins will take about 5 Minutes\n";

   shell_ex ( "$CONFIG{'IROOT'}/sbin/updateplugins_1.00.pl" , "" );

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} < $CONFIG{'IROOT'}/install/configs/add_base_profiles.sql",
      "SETTING UP SAFE/ENABLE-ALL PROFILES / SYSTEM FULLY READY TO USE" );

   print "\n";
   print "SWEET!\n";
   print "\n";
   print "*** REMEMBER to link your website directory to\n$CONFIG{'IROOT'}/html\n\n";
   print "\n";
   print "Thanks, The Inprotect Team\n";
   print "\n";

}

sub initDB {

   print "WARNING: CLEAN INSTALL SCRIPT WILL DROP TABLES IF EXIST IN THE NEW DB\n";
   print "BE VERY CERTAIN YOU DO NO SPECIFY EXISTING DB AS NEW DB NAME\n";
   print "BACKUP YOUR EXISTING DB FIRST EXAMPLE: # mysqldump -p inprotect > inprotect_backup.sql\n";
   print "NOT RESPONSIBLE FOR ACCIDENTS / MISUSE\n";

   print "using file $DEFAULTS{'DBFILE'}\n";

   #GET USE SSL VALUE
   get_value ( "useSSL", "DEFREQSSL", "REQUIRE HTTPS (SSL) for website: valid options [0|1]", 0 );

   if ( $CONFIG{'useSSL'} == "0" || $CONFIG{'useSSL'} == "1" ) {
      print "Good SSL VALUE\n";
   } else {
      print "DEFAULTING TO ENABLED as bad ssl VALUE SUPPLIED\n";
      $CONFIG{'useSSL'} = "1";
   }

   my $txtSSL = "UPDATE inprotect_settings SET settingValue = '$CONFIG{'useSSL'}' WHERE settingName='enablessl' LIMIT 1;";

   open (iSSL, ">/tmp/iSSL.tmp" );
   print iSSL "$txtSSL\n";
   close (iSSL);

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} -e \"CREATE DATABASE IF NOT EXISTS $CONFIG{'mysqldb'}\";",
      "CREATING NEW DATABASE [ $CONFIG{'mysqldb'} ] for Inprotect $DEFAULTS{'REV'} use!" );

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} < $DEFAULTS{'DBFILE'}",
      "IMPORTING $DEFAULTS{'DBFILE'} INTO DATABASE $CONFIG{'mysqldb'}" );

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} < /tmp/iSSL.tmp",
      "Updating SSL setting to $CONFIG{'useSSL'}" );

   shell_ex ( "rm -f /tmp/iSSL.tmp", "" );

   print "finished setting up database\n\n";

   print "verifying configs\n";
   &load_configs("/etc/inprotect.cfg");


   print "Installation Complete!\n";
   print "\n";
   print "Proceed with Post Install Configuration of the following for New Installation? \n";
   print " -  Add first Nessus Server\n";
   print " -  Load Plugins\n";
   print "Continue: ( Y/N )?\n";

   my $input = <STDIN>;
   chomp $input;
   if ( defined($input) && $input =~ /y/i ) {
      setup_first_server( );
      exit;
   } else {
      print "Be sure to Add first server vi portal and launch $CONFIG{'IROOT'}/sbin/update_plugins_1.00.pl\n";
      print "to setup scan profiles to Complate post install setup configuration\n";
      print "Must be done in that order to enable ability to create a scan profile for the purpose of scanning.\n";
      print "*** REMEMBER to link your website directory to\n\t$CONFIG{'IROOT'}\n";
      print "\n";
      print "Thanks, The Inprotect Team\n";
      print "\n";
      exit;
   }

}

sub migrate_data {

   my $continue = 0;

   my $merge_file = "/tmp/merge_data.sql";

   print "Congradulations on your decison to upgrade to the $DEFAULTS{'REV'} generation DB schema\n";

    print "Collecting DB connection information for previous install\n";
   #GET DBHOST
   #get_value ( "pmysqlhost", "DEFDBHOST", "Enter Database [HOSTNAME/IP] hosting mysql", 0 );
   #GET DBNAME

   while ( $continue == "0") {
      get_value ( "pmysqldb", "DEFDATABASE", "Enter Database name", 0 );
      if ( $CONFIG{'pmysqldb'} ne "" && $CONFIG{'pmysqldb'} != $CONFIG{'mysqldb'} ) {
         $continue = 1;
      } else {
          if ( $CONFIG{'pmysqldb'} eq ""  ) {
             print "Existing DB name can not be blank for upgrade\n";
          } elsif ( $CONFIG{'pmysqldb'} eq $CONFIG{'mysqldb'} ) {
             print "You cannot use same dbname for (existing and new install)\n";
          }
      }
   }

   #GET DBUSER
   #get_value ( "pmysqluser", "DEFDBUSER", "Enter Database user with permission to create database", 0 );
   #GET DBPASSWORD
   #get_value ( "pmysqlpass", "DEFDBPASS", "Enter Database user password", 0 );



   print "WARNING: CLEAN INSTALL SCRIPT WILL DROP TABLES IF EXIST IN THE NEW DB\n";
   print "BE VERY CERTAIN YOU DO NO SPECIFY EXISTING DB AS NEW DB NAME\n";
   print "BACKUP YOUR EXISTING DB FIRST EXAMPLE: # mysqldump -p inprotect > inprotect_backup.sql\n";
   print "NOT RESPONSIBLE FOR ACCIDENTS / MISUSE\n";

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'}  -e \"TRUNCATE TABLE users\" $CONFIG{'mysqldb'};",
      "TRUNCATING NEW DB USERS TABLE SO MERGE FROM OLD DB WILL BE A SUCCESS" );

   shell_ex ( "mysqldump -c -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} --no-create-info --tables nessus_category nessus_family nessus_plugins nessus_preferences nessus_preferences_defaults nessus_servers nessus_server_zones nessus_settings nessus_settings_category nessus_settings_family nessus_settings_plugins nessus_settings_preferences nessus_settings_users nessus_zones nessus_zone_ranges plugins > $merge_file",
      "DUMPING TABLES TO DIRECTLY MIGRATE ROWS TO NEW DATABASE" );

   shell_ex ( "$CONFIG{'pathmysql'} -u$CONFIG{'mysqluser'} -p$CONFIG{'mysqlpass'} $CONFIG{'mysqldb'} < $merge_file",
      "IMPORTING DATA THAT WILL DIRECTLY MIGRATE TO DB" );

   print "START DATA MIGRATION NOW?  ( Warning:  This will take some time to run )\n";
   my $input = <STDIN>;
   chomp $input;
   if ( defined($input) && $input =~ /y/i ) {
      print "Launching Migration Script\n";
      shell_ex ( "$CONFIG{'IROOT'}/install/migrate_data_to_1.0_official.pl" , "" );
   } else {
      print "Run data migration script manually from install dir\n";
   }
}

sub setupConfigs {

   my ( $a, $cmd, $input, $sfile ) = "";

   if ( $DEFAULTS{'DEFROOT'} ne "/opt/Inprotect" ) {
      print "Is the following location for Inprotect Installation acceptable ( ".$DEFAULTS{'DEFROOT'}." ) [yes/no] ? ";
      $input = <STDIN>;
      chomp $input;
      if ( defined($input) && $input eq "yes" ) {
        $CONFIG{'IROOT'} = $DEFAULTS{'DEFROOT'};
      } else {
         print "\nPlease Move the Inprotect Structure to the desired installation location such as \"/opt/Inprotect\" and restart installation\n\n";
         print "The Inprotect Team\n\n";
         exit;
      }
   } else {
      print "Nice!: A Perferred installation path accepted: ". $DEFAULTS{'DEFROOT'} ."\n";
      $CONFIG{'IROOT'} = $DEFAULTS{'DEFROOT'};
   }

   #GET DBHOST
   get_value ( "mysqlhost", "DEFDBHOST", "Enter Database [HOSTNAME/IP] hosting mysql", 0 );
   #GET DBNAME
   get_value ( "mysqldb", "DEFDATABASE", "Enter Database name", 0 );
   #GET DBUSER
   get_value ( "mysqluser", "DEFDBUSER", "Enter Database user with permission to create database", 0 );
   #GET DBPASSWORD
   get_value ( "mysqlpass", "DEFDBPASS", "Enter Database user password", 0 );

   #GET MYSQLPATH
   get_value ( "pathmysql", "DEFMYSQL", "Enter Mysql Path", 0 );
   #GET NESSUSPATH
   get_value ( "pathnessus", "DEFNESSUS", "Enter Nessus Path", 0 );
   #GET NESSUS-UPDATE-PLUGINSPATH
   get_value ( "pathnessusupdate", "DEFNESUPDATE", "Enter Nessus-Update-Plugins Path", 0 );

   #FORCE LOCAL NESSUS SERVER TO UPDATE PLUGINS ELSE IMPORT PLUGINS REMOTE SERVER
   get_value ( "localupdate", "LOCALUPDATE", "Enable nessus-update-plugins (requires valid feed on localhost): valid options [0|1],", 0 );

   if ( $CONFIG{'localupdate'} == "0" || $CONFIG{'localupdate'} == "1" ) {
      print "Good LocalUpdate VALUE\n";
   } else {
      print "DEFAULTING TO ENABLED as bad UPDATE VALUE SUPPLIED\n";
      $CONFIG{'localupdate'} = 1;
   }

   my $inprotectCFG = qx/cat configs\/inprotect.cfg/;
   $inprotectCFG =~ s/<NESSUSPATH>/$CONFIG{'pathnessus'}/;
   $inprotectCFG =~ s/<NESSUSUPDATEPLUGINSPATH>/$CONFIG{'pathnessusupdate'}/;
   $inprotectCFG =~ s/<MYSQLPATH>/$CONFIG{'pathmysql'}/;
   $inprotectCFG =~ s/<DBHOST>/$CONFIG{'mysqlhost'}/;
   $inprotectCFG =~ s/<DBNAME>/$CONFIG{'mysqldb'}/;
   $inprotectCFG =~ s/<DBUSER>/$CONFIG{'mysqluser'}/;
   $inprotectCFG =~ s/<DBPASS>/$CONFIG{'mysqlpass'}/;
   $inprotectCFG =~ s/<UPDATEPLUGINS>/$CONFIG{'localupdate'}/;
   $inprotectCFG =~ s/<DBI>/$DEFAULTS{'DBI'}/;
   $inprotectCFG =~ s/<ROOTDIR>/$CONFIG{'IROOT'}/g;

   $sfile = $CONFIG{'IROOT'}."/etc/inprotect.cfg";
   open (iCFGFILE, ">$sfile" );
   print iCFGFILE ( $inprotectCFG );
   close (iCFGFILE);

   my $configPHP = qx/cat configs\/config.php/;
   $configPHP =~ s/<DBHOST>/$CONFIG{'mysqlhost'}/;
   $configPHP =~ s/<DBNAME>/$CONFIG{'mysqldb'}/;
   $configPHP =~ s/<DBUSER>/$CONFIG{'mysqluser'}/;
   $configPHP =~ s/<DBPASS>/$CONFIG{'mysqlpass'}/;
   $configPHP =~ s/<REV>/$CONFIG{'localupdate'}/;
   $configPHP =~ s/<DBI>/$DEFAULTS{'DBI'}/;
   $configPHP =~ s/<ROOTDIR>/$CONFIG{'IROOT'}/g;

   $sfile = "$CONFIG{'IROOT'}/html/config.php";
   print "sfile=$sfile\n";
   open (pCFGFILE, ">$sfile" );
   print pCFGFILE "$configPHP";
   close (pCFGFILE);

   my $cronTAB = qx/cat configs\/crontab/;
   $cronTAB =~ s/<ROOTDIR>/$CONFIG{'IROOT'}/g;

   $sfile = "/tmp/crontab.tmp";
   print "sfile=$sfile\n";
   open (pCFGFILE, ">$sfile" );
   print pCFGFILE "$cronTAB";
   close (pCFGFILE);

   print "Backing up Crontab to /tmp/cron.bak\n";
   shell_ex ( "crontab -l > /tmp/cron.bak", "" );
   shell_ex ( "crontab -l | grep -F -v -f /tmp/crontab.tmp | grep -v ^\# > /tmp/cron.tmp", "" );
   shell_ex ( "cat /tmp/crontab.tmp >> /tmp/cron.tmp", "" );
   shell_ex ( "cat /tmp/cron.tmp", "" );
   print "Installing Crontab\n";
   shell_ex ( "crontab /tmp/cron.tmp", "" );
   shell_ex ( "rm -rf /tmp/cron.tmp", "" );
   shell_ex ( "rm -rf /tmp/crontab.tmp", "" );


   shell_ex ( "rm -f /etc/inprotect.cfg" , "UPDATING LINE FROM /etc/inprotect.cfg to path [ $CONFIG{'IROOT'}/etc/inprotect.cfg ]" );
   shell_ex ( "ln -s $CONFIG{'IROOT'}/etc/inprotect.cfg /etc/inprotect.cfg" , "Linking /etc/inprotect.cfg" );

   print "setting permissions on Web Directory and configs\n";
   shell_ex ( "chmod 644 $CONFIG{'IROOT'}/etc/inprotect.cfg" , "" );
   shell_ex ( "chmod -R 755 $CONFIG{'IROOT'}/html/*.php" , "" );
   shell_ex ( "chmod 644 $CONFIG{'IROOT'}/html/config.php" , "" );

}

sub showMenu {
   print "System meets the prerequisites, continuing...\n\n";
   print "Welcome to Inprotect $DEFAULTS{'REV'} [ Central Node Installation]\n";
   print "======================================================\n\n";
   print "**Warning: installation must be run as root user!!!!\n";
   print "Please Choose from the following options:\n\n";
   print "1 - Configure Inprotect\n";
   print "2 - install Database components and Configure Inprotect\n";
   print "3 - Upgrade DB Schema to Lates Rev\n";
   print "4 - Migrage existing data to new Inprotect Installation\n";
   print "Q - Quit without installation\n";
   print "To install scanner nodes load nessus, add first users, and configure a feed.\n";
   print "Make your selection [1-4]\n";
}

sub shell_ex {
   my($cmd, $text )=@_;

   my $a = "";
   if ( $debug ) { print "cmd=$cmd\n"; }
   if ( $text ) { print "$text\n"; }
   $a = qx/$cmd/;

}

sub get_value {
   my($cfgname, $defname, $text, $nullOK )=@_;

   my $continue = 1;
   my $valid_def = 0;
   my $input = "";

   if ( defined($defname) && $DEFAULTS{$defname} ne "" ) { 
      $text .= "or use default ( $DEFAULTS{$defname} )";
      $valid_def = 1;
   }

   while ( $continue == "1" ) { 
      print "$text: ";
      $input = <STDIN>;
      chomp $input;
      if ( $input ne "" ) {
         $continue = 0;
      } elsif ( $input eq "" && $valid_def == "1" ) { 
         $input = $DEFAULTS{$defname};
         $continue = 0;
      } elsif ( $nullOK == "1" ) {
         $continue = 0;
      } else {
         print "Missing required Argument!\n\n";
      }
   }
   $CONFIG{$cfgname} = $input;
   return 1;
}

sub checkDependencies {

    my @dependencies = qw/DBI CPAN::DistnameInfo Term::ReadKey Parallel::ForkManager Config::IniFiles MIME::Lite Date::Calc Crypt::CBC MIME::Base64 LWP IO::Socket Net::SSLeay IO::Socket::SSL POSIX Socket Crypt::Blowfish Net::IP Net::Netmask Net::DNS Net::Nslookup Net::Telnet/;

    my $failed = 0;
    print "\nChecking Prerequisities:\n\n";

    #CHECK PER DEPENDANCIES
    print "Checking Perl requirements\n";
    foreach (@dependencies) {
        my $test = "OK";
        eval "use $_";
        $test = "Missing" if ($@);
        if ( $test eq "Missing" ) { $failed = 1; }
        print $_, ' 'x(35-length($_)), $test, "\n";
    }

    #CHECKING PEAR DEPENDANCIES
    print "\nChecking Pear requirements\n";
    my $cmd = "pear list | grep Crypt_CBC";
    my $CBC = qx/$cmd/;
    if ( $CBC =~ /Crypt_CBC/ ) {
        print "Pear Crypt_CBC", ' 'x(35-length("Pear Crypt_CBC")), "OK", "\n";
    } else {
        print "Pear Crypt_CBC", ' 'x(35-length("Pear Crypt_CBC")), "Missng", "\n";
        $failed = 1;
    }

    my $cmd = "pear list | grep Mail_Mime";
    my $CBC = qx/$cmd/;
    if ( $CBC =~ /Mail_Mime/ ) {
        print "Pear Mail_Mime", ' 'x(35-length("Pear Mail_Mime")), "OK", "\n";
    } else {
        print "Pear Mail_Mime", ' 'x(35-length("Pear Mail_Mime")), "Missng", "\n";
        $failed = 1;
    }

    my $cmd = "pear list | grep Mail_mimeDecode";
    my $CBC = qx/$cmd/;
    if ( $CBC =~ /Mail_mimeDecode/ ) {
        print "Pear Mail_mimeDecode", ' 'x(35-length("Pear Mail_mimeDecode")), "OK", "\n";
    } else {
        print "Pear Mail_mimeDecode", ' 'x(35-length("Pear Mail_mimeDecode")), "Missng", "\n";
        $failed = 1;
    }

    print "\n";

    return $failed;
}

sub installDependencies {

    my @dependencies = qw/DBI CPAN::DistnameInfo Term::ReadKey Config::IniFiles Parallel::ForkManager MIME::Lite Date::Calc Crypt::CBC MIME::Base64 LWP IO::Socket Net::SSLeay IO::Socket::SSL POSIX Socket Crypt::Blowfish Net::IP Net::Netmask Net::DNS Net::Nslookup Net::Telnet/;

    my $failed = 0;
    print "\nChecking/Install Prerequisities:\n";
    foreach (@dependencies) {
        my $test = "OK";
        eval "use $_";
        $test = "Missing" if ($@);
        if ( $test eq "Missing" ) { 
           shell_ex ( "perl -MCPAN -e 'install $_'", "INSTALL PERL MODULE: $_" );
           #print "execute install\n";
           $failed = 1; 
        }
        print $_, ' 'x(35-length($_)), $test, "\n";
    }
    print "\n";

    return $failed;
}

sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
