#!/usr/bin/perl -w
#
###############################################################################
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.				      #
###############################################################################
##
##    PRGM AUTHOR: Kenneth Kline \ Clutch-INC
##   PROGRAM NAME: updateReportStats TOOL
##   PROGRAM DATE: 04/23/2007
##  PROGM VERSION: 0.1
##  REVISION HIST:


use strict;
use warnings;

use Data::Dumper::Simple;
use Net::IP;
use Net::Nslookup;
use Net::Netmask;
use MIME::Lite;
#use MIME::Lite::HTML;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use DBI;
use Getopt::Std;
use Switch;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
use Term::ReadKey;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

#CONFIG PATH TO THE CURRENT VERSION OF INPROTECT 1.X OR HIGHER
&load_configs("/etc/inprotect.cfg");

# CONFIG OLD DB VARIABLES
my $platform = "mysql";
# get them from the user
print "Enter old DB name (inprotect): ";
my $database = ReadLine(0);
chomp $database;
if($database eq "") { $database = "inprotect"; }
#my $database = "i225";
print "Enter old DB host (localhost): ";
#my $host = "localhost";
my $host = ReadLine(0);
chomp($host);
if($host eq "") { $host = "localhost"; }
print "Enter old DB port (3306): ";
#my $port = "3306";
my $port = ReadLine(0);
chomp $port;
if($port eq "") { $port = "3306"; }
print "Enter old DB user (inprotect): ";
#my $user = "inprotect";
my $user = ReadLine(0);
chomp $user;
if($user eq "") { $user = "inprotect";}
print "Enter old DB password: ";
ReadMode('noecho');
#my $pw = "inprotect";
my $pw = ReadLine(0);
chomp $pw;
if($pw eq "") { die("Password required"); }
ReadMode(1);
print "\n";



#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/next_generation.log";

print "Enable Debugging output? [0|1] (only do this is you are having problems): ";
#my $debug           = 0;
my $debug = ReadLine(0);
chomp $debug;
my $log_level       = 5;
my $track_progress  = 0;
my $time_to_die     = 0;
my $use_scanlite    = 0;
my $notify_by_email = 1;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
#use constant MAX_FORKS => 20;


#Declare Public Variables to use
my $outdir = "$CONFIG{'ROOTDIR'}/tmp";

my @nessus_plugins;


# CONNECT TO ORGINAL DB
#DATA SOURCE NAME
my $dsn = "dbi:$platform:$database:$host:$port";
my $db_orig  = DBI->connect( $dsn, $user, $pw );

$dsn = "$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}:$CONFIG{'DATABASEHOST'}";
my $dbh = conn_db( $dsn, "$CONFIG{'DATABASEUSER'}", "$CONFIG{'DATABASEPASSWORD'}" );


#Writing some sample code to build/poplulate a stats table

main( );


sub main {
    my ( $work, $outfile, $targets, $qlan, $qtime );


    open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
    *STDERR=*LOG;
    *STDOUT=*LOG;

    my $starttime = time;
    print "STARTING - " . localtime($starttime) . "\n";

    #GENERATE RANDOM SEED
    srand(time() ^($$ + ($$ <<15)));


    begin_work( );

    disconn_db($dbh);
    disconn_db($db_orig);

    my $endtime = time;
    print "COMPLETE - " . localtime($endtime) . "\n";
    print "Elapsed time = " . ($endtime - $starttime) . "\n";
    #end of main
    exit;

}

sub begin_work {
    my ( $sth_sel, $sth_sel2 );

    #IF 0.80 OR HIGHER MOVE INPROTECT_SETTINGS FOR EXISTING ENTRIES   

#    print "Merging inprotect_settings table...";
#    merge_inprotect_settings( );
#    print "complete\n";
#    print "Merging user table...";
#    merge_users();
#    print "complete\n";
#    print "Setting up job_schedule tables...";
#    merge_job_schedule();
#    print "complete\n";
#    print "Setting up reports tables...";
#    merge_reports();
#    print "complete\n";
#    print "Copying over exceptions...";
#    copy_exceptions();
#    print "complete\n";
    print "Copying over nessus data...";
    copy_nessusdata();
    print "complete\n";

}

sub copy_nessusdata {
    my ( $sql, $sth_sel );
    my $oldDB = $database;
    my $newDB = $CONFIG{'DATABASENAME'};
    my $db;

    # drop the new tables so we can just copy them over directly
    my @tableArr = (
'nessus_category',
'nessus_family',
'nessus_notes',
'nessus_plugins',
'nessus_preferences',
'nessus_preferences_defaults',
'nessus_server_zones',
'nessus_servers',
'nessus_settings',
'nessus_settings_category',
'nessus_settings_family',
'nessus_settings_plugins',
'nessus_settings_preferences',
'nessus_settings_users',
'nessus_user_zones',
'nessus_zone_ranges',
'nessus_zones'
);

    foreach $db (@tableArr) {
       $sql = qq{ DROP TABLE IF EXISTS $newDB.$db  };
       safe_db_write( $sql, 4 );
       $sql = qq{ CREATE TABLE IF NOT EXISTS $newDB.$db SELECT * FROM $oldDB.$db  };
       safe_db_write( $sql, 4 );
    }
}

sub copy_exceptions {
    my ( $sql, $sth_sel );
    my $oldDB = $database;
    my $newDB = $CONFIG{'DATABASENAME'};
    my $db;

    # drop the new tables so we can just copy them over directly
    my @tableArr = ('exceptions');

#    $sql = qq{ truncate users  };
    foreach $db (@tableArr) {
       $sql = qq{ DROP TABLE IF EXISTS $newDB.$db  };
       safe_db_write( $sql, 4 );
       $sql = qq{ CREATE TABLE IF NOT EXISTS $newDB.$db SELECT * FROM $oldDB.$db  };
       safe_db_write( $sql, 4 );
    }
}

sub merge_inprotect_settings {
    #VERIFIED

    my ( $sth_sel );
    my $sql = qq{ SELECT settingName, settingValue, settingSection FROM inprotect_settings
	WHERE settingName != 'version' };
    logwriter( $sql, 5 );
    $sth_sel = $db_orig->prepare( $sql );
    $sth_sel->execute;
    while( my ($name, $value, $section ) = $sth_sel->fetchrow_array ){
	#ATTEMPT SAFE WRITE IF ERROR WILL CONTINUE
	$sql = qq{ UPDATE inprotect_settings SET settingValue='$value' WHERE settingname='$name'
	    AND settingSection='$section' LIMIT 1 };
	safe_db_write( $sql, 4 );
    }

}

sub merge_users {
    #VERIFIED
    my ( $sql, $sth_sel );
    my $oldDB = $database;
    my $newDB = $CONFIG{'DATABASENAME'};
    my $db;

    # drop the new tables so we can just copy them over directly
    my @tableArr = ('users','user_roles','groups','group_roles','roles');

#    $sql = qq{ truncate users  };
    foreach $db (@tableArr) {
       $sql = qq{ DROP TABLE IF EXISTS $newDB.$db  };
       safe_db_write( $sql, 4 );
       $sql = qq{ CREATE TABLE IF NOT EXISTS $newDB.$db SELECT * FROM $oldDB.$db  };
       safe_db_write( $sql, 4 );
    }

    # now add back the new fields into users
    $sql = qq{ ALTER TABLE users ADD pass_lastchanged datetime not NULL };
    safe_db_write( $sql, 4);
    $sql = qq{ ALTER TABLE users ADD lockout_reset datetime };
    safe_db_write( $sql, 4);

    #$sql = qq{ SELECT pn_uname, pn_email, pn_pass, expire, locked, created, lastaccessed, owniponly, 
    #   sc_now, sc_once, sc_daily, sc_weekly, sc_monthly FROM users };
    #logwriter( $sql, 5 );
    #$sth_sel = $db_orig->prepare( $sql );
    #$sth_sel->execute;
    #while( my ( $pn_uname, $pn_email, $pn_pass, $expire, $locked, $created, $lastaccessed, $owniponly,
    #  $sc_now, $sc_once, $sc_daily, $sc_weekly, $sc_monthly ) = $sth_sel->fetchrow_array ){
#	#ATTEMPT SAFE WRITE IF ERROR WILL CONTINUE
#	$sql = qq{ INSERT INTO users ( pn_uname, pn_email, pn_pass, expire, locked, created, lastaccessed, owniponly,
#           sc_now, sc_once, sc_daily, sc_weekly, sc_monthly ) VALUES 
#	   ( '$pn_uname', '$pn_email', '$pn_pass', '$expire', '$locked', '$created', '$lastaccessed', '$owniponly',
#           '$sc_now', '$sc_once', '$sc_daily', '$sc_weekly', '$sc_monthly' ); };
#	safe_db_write( $sql, 4 );
#    }
}

sub merge_job_schedule {

    #ONLY MOVE JOB SCHEDULES THAT ARE DAILY, WEEKLY, MONTHLY RUN ONCE JOBS SHOULD ONLY BE INSERTED IN THE JOB TABLE

    my ( $sql, $sql2, $sth_sel, $sth_sel2 );
    $sql = qq{ SELECT id, name, username, schedule_type, day_of_week, day_of_month, time, next_scan, status, email, sid, timeout
       FROM nessus_schedule };

    logwriter( $sql, 5 );
    $sth_sel = $db_orig->prepare( $sql );
    $sth_sel->execute;
    while( my ( $id, $name, $username, $schedule_type, $day_of_week, $day_of_month, $time, $next_scan, $status, 
       $email, $sid, $timeout ) = $sth_sel->fetchrow_array ){
        my $target_list = "";
        my $enabled = "0";

        $sql2 = qq{ SELECT hostip FROM nessus_hosts WHERE sched_id='$id' };
        logwriter( $sql, 5 );
        $sth_sel2 = $db_orig->prepare( $sql2 );
        $sth_sel2->execute;
        while( my ( $hostip ) = $sth_sel2->fetchrow_array ){
           $target_list .= "$hostip\n";
        }

        if ( $schedule_type ne "D" && $schedule_type ne "W" && $schedule_type ne "M" ) { $schedule_type = "D"; $enabled="0"; }
	#ATTEMPT SAFE WRITE IF ERROR WILL CONTINUE
	$sql2 = qq{ INSERT INTO I3_job_schedule ( id, name, username, schedule_type, day_of_week, day_of_month,
           time, email, meth_TARGET, meth_CRED, meth_VSET, meth_TIMEOUT, next_CHECK, enabled ) VALUES (
           '$id', '$name', '$username', '$schedule_type', '$day_of_week', '$day_of_month', 
           '$time', '$email', '$target_list', Null, '$sid', '$timeout', '$next_scan', '$enabled' ); };
	safe_db_write( $sql2, 4 );
    }
}

sub merge_reports {
    my ( $sql, $sql2, $sql3, $sqlold, $sth_sel, $sth_sel2, $sth_sel3, $sth_selold );

    my ( $report_id, $subnet, $start_dttm, $submit_dttm );


    my $errCount = 0;
    my @errMsg;
    my $ctable = {};	#STORE NETBLOCKS FOR ORG LOOKUP

    $sql = qq{ SELECT id, CIDR FROM I3_subnets WHERE 1 };
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while( my ( $subID, $CIDR )=$sth_sel->fetchrow_array){
       my $net_table = new2  Net::Netmask( $CIDR );
       $net_table->storeNetblock( $ctable );
    }
    logwriter( "LOADED ALL Netblocks", 5 );

    $sql = qq{ SELECT distinct(scantime), sched_id FROM nessus_results WHERE 1 };
    logwriter( $sql, 5 );
    $sth_sel = $db_orig->prepare( $sql );
    $sth_sel->execute;

    #PROCESS EACH UNIQUE SCANTIME/SCHED_ID
    while ( my ( $scantime, $sched_id ) = $sth_sel->fetchrow_array ) {
       my %hostHash;  #HOSTHASH RESET ON EACH REPORT
       my $report_key = "";

       print "Processing sched_id=$sched_id\tscantime=$scantime\n";

       #LOOKUP OTHER REPETITIVE VALUES COMMON TO ALL RESULTS OF THE SCAN
       $sql2 = qq{ SELECT t1.username, t1.sid, t1.domain, t2.name, t2.email, t2.timeout FROM nessus_results t1
          LEFT JOIN nessus_schedule t2 on t1.sched_id=t2.id
          WHERE t1.scantime="$scantime" AND t1.sched_id="$sched_id" LIMIT 1 };
       logwriter( $sql2, 5 );
       $sth_sel2 = $db_orig->prepare( $sql2 );
       $sth_sel2->execute;
       my ( $username, $sid, $domain, $job_title, $notify, $timeout ) = $sth_sel2->fetchrow_array( );
       $sth_sel2->finish;
       logwriter( "username=$username, sid=$sid, domain=$domain, job_name=$job_title", 4 );

       #CREATE JOB ENTRY (FUDGE LESS IMPORTANT VALUES)
       $sql2 = qq{ INSERT INTO I3_jobs ( name, username, job_TYPE, meth_SCHED, meth_VSET, meth_TIMEOUT, scan_END,
           scan_PRIORITY, status, notify, authorized, author_uname) VALUES ( '$job_title', '$username', 'M', '$sched_id',
           '$sid', '$timeout', '$scantime', '3', 'C', '$notify', '1', 'ACL'); };
       safe_db_write ( $sql2, 4 );

       $sql2 = qq{ SELECT id FROM I3_jobs WHERE name='$job_title' AND scan_END='$scantime' ORDER BY scan_END DESC LIMIT 1 };
       logwriter( $sql2, 5 );
       $sth_sel2=$dbh->prepare( $sql2 );
       $sth_sel2->execute;
       my $job_id = $sth_sel2->fetchrow_array( );

       #CREATE REPORT ENTRY  -VERY IMPORTANT TO GET A REPORT_ID AND REPORT_KEY
       #Build a report_key value to secure reports.
       my @arr = split(/\./, rand() );
       if ( $arr[1] && is_number($arr[1]) ) {
          $report_key = $arr[1];
       } else {
          logwriter( "Failed Report Key generation", 3 );
       }

       $sql2 = qq{ INSERT INTO nessus_reports ( username, name, sid, scantime, scantype, report_key ) VALUES (
          '$username', '$job_title', '$sid', '$scantime', 'M', '$report_key' ) };
       safe_db_write ( $sql2, 4 );

       $sql2 = qq{ SELECT report_id FROM nessus_reports WHERE scantime='$scantime' AND report_key='$report_key'
        ORDER BY scantime DESC LIMIT 1 };
       logwriter( $sql2, 5 );
       $sth_sel2=$dbh->prepare( $sql2 );
       $sth_sel2->execute;

       #SUPER IMPORTANT LOOKUP REPORT_ID AFTER REPORT ENTRY IS CREATED OTHERWISE REPORT LIKELY NOT CREATED FOR WHATEVER REASON
       #FORIEGN KEY TO JOINING TABLES TOGETHER NESSUS_REPORT_STATS, NESSUS_RESULTS, I3_JOBS, HOSTS, HOST_STATS FOR INSTANTLY PULLING
       #THE MOST RECENT SCAN FOR EACH OF THE FOLLOWING SUBNET, SYSTEM, HOST
       my $report_id = $sth_sel2->fetchrow_array( );
       $sth_sel2->finish;
       if ( ! $report_id ) {
          logwriter( "failed to lookup report after insert for scan $scantime", 2 );
          return 0;
       } else {
          $sql = qq{ UPDATE I3_jobs SET report_id='$report_id', scan_END='$scantime' 
             WHERE id='$job_id' LIMIT 1};
          safe_db_write ( $sql, 4 );            #use insert/update routine
       }

       #PROCESS DISTINCT HOSTS IN THE REPORT
       $sqlold = qq{ SELECT distinct(host) FROM nessus_results WHERE scantime="$scantime" AND sched_id="$sched_id" };
       logwriter( $sql2, 5 );
       $sth_selold = $db_orig->prepare( $sqlold );
       $sth_selold->execute;
       while ( my ( $host ) = $sth_selold->fetchrow_array ) {
          #ensure values are reset 
          my ( $hostip, $hostname, $domain, $mac_address) = "";

          #DETERMINE IF IT IS AN IP (CRITICAL STEP AS SCANLITE RETURNS EITHER HOSTIP/HOSTNAME) AS HOST CAN FUBAR US
          #AS HOSTNAME/HOSTIP COULD BE WRITTEN TO THE WRONG FIELD
          my $isIP = FALSE;
          if ( $host =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)/ ){
             if($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255) {
                $hostip=$host;
                $isIP = TRUE;
             }
          }

          if ( $isIP == FALSE ) {	    #MUST BE A NAME ATTEMPT TO RESOLVE ELSE FAILED MISERABLY
             $hostname=$host;	    #LETS AT LEAST SET NAME IN CASE ALL ELSE FAILS
             my $tmp_hostip = resolve_name2ip( $host );
             if ( defined( $tmp_hostip) && $tmp_hostip ne "" ) { $hostip = $tmp_hostip; }
          }

          if ( defined( $hostip ) && $hostip ne "" ) {
             my $match = findNetblock($hostip, $ctable);
             if ( $match ) {
                $sql2 = qq{ SELECT id, site_code, ORG FROM I3_subnets WHERE CIDR = "$match" LIMIT 1 };
                my $sth_sel3 = $dbh->prepare( $sql2 );
                $sth_sel3->execute;
                my ( $subID, $site_code, $ORG) = $sth_sel3->fetchrow_array( );

                $hostHash{$host}{'org'} = "$ORG";	    #FOUND ORG
                $hostHash{$host}{'site'} = "$site_code";    #FOUND SITE
                logwriter( "FOUND ORG MATCH FOR hostip=$hostip ORG=$ORG site=$site_code", 5 );
             }
             #ATTEMPT TO CONSULT VARIOUS DNS SERVERS IN CASE SCANLITE RETURNTED IP
             my $tmp_hostname = resolve_host( $hostip );
             if ( defined( $tmp_hostname ) && $tmp_hostname ne "" ) { $hostname = $tmp_hostname; }
          }

          $hostHash{$host}{'ip'} = $hostip; 
          if( defined( $hostname ) && $hostname ne "" ) {
             $hostHash{$host}{'fqdn'} = $hostname;
             $hostHash{$host}{'dns'} = "1";    #INDICATE RESOLVED BY NAME FAILED
             logwriter( "nessus_scan: successfully looked up name [$host]", 5 );
          } else {
             $hostHash{$host}{'dns'} = "-1";    #INDICATE RESOLVED BY NAME FAILED
          }

          $sql3 = qq{ SELECT service, scriptid, risk, msg, falsepositive FROM nessus_results
             WHERE scantime="$scantime" AND sched_id="$sched_id" AND host="$host" };
          logwriter( $sql3, 5 );
          $sth_sel3 = $db_orig->prepare( $sql3 );
          $sth_sel3->execute;
          while ( my ( $service, $scriptid, $risk, $msg, $falsepositive ) = $sth_sel3->fetchrow_array ) {
             my ( $port, $proto ) = "";
             my $app = $service;

             if(defined($service) && $service ne "") {
                if(defined($proto) && $proto ne "") {
                    $service = "$service ($port/$proto)";
                } else {
                    $app = "general";
                    $proto = $service;
                    $port = "";
                    $service = "general/$service";
                }
             }
             if ( $scriptid eq "11936" ) {
                my $os = extract_os( $msg );
                $hostHash{$host}{'os'} = $os;
             }

             if ( $scriptid eq "10150" ) {
                $mac_address = "unknown";
                my $txt_msg = $msg;
                logwriter( "nessus_scan: plugin 10150 data: [[$txt_msg]]", 5 );
                my @arrMSG = split /\n/, $txt_msg;
                foreach my $line (@arrMSG) {
                    $line =~ s/\#.*$//;
                    chomp($line);
                    $line =~ s/\s+$//;
                    $line =~ s/^\s+//;
                    logwriter( "nessus_scan: LINE=[$line]", 5 ); 
                    if ($line =~ /computer\sname/i ) {
                        my @temp=split(/=/,$line,2);
                        $temp[0] =~ s/\s+$//;
                        $temp[0] =~ s/^\s+//;
                        $hostname = lc( $temp[0] );
                        logwriter( "nessus_scan: hostname=[$hostname]", 5 );
                    } elsif ($line =~ /workgroup/i ) {
                        my @temp=split(/=/,$line,2);
                        $temp[0] =~ s/\s+$//;
                        $temp[0] =~ s/^\s+//;
                        $domain = lc( $temp[0] );
                        logwriter( "nessus_scan: domain=[$domain]", 5 );
                    } elsif ($line =~ /^([0-9a-fA-F][0-9a-fA-F]:){5}([0-9a-fA-F][0-9a-fA-F])$/ ) {
                        $mac_address = uc( $line );
                    }

                    next;
                }
                if ( $hostHash{$host}{'dns'} eq "-1" ) { #ONLY UPDATE NAME FROM 10150 WHEN DNS FAILS
                    if ( defined( $hostname ) && $hostname ne "" && defined( $domain ) && $domain ne "" ) {
                        $hostHash{$host}{'fqdn'} = $hostname;
                        $hostHash{$host}{'domain'} = $domain;
                        $hostHash{$host}{'dns'} = '1'; 
                        logwriter( "nessus_scan: success plugin 10150 to look up name [$hostname]", 5 );  
                    }
                }
                $hostHash{$host}{'mac'} = $mac_address;   #ALWAYS GET MAC
            }

             #LOOK MAC/HOSTNAME/OS/ETC BY FALLBACK TO NESSUS PLUGIN

             $sql2 = qq{ INSERT INTO nessus_results ( report_id, scantime, hostip, hostname, service,
                app, port, protocol, scriptid, risk, msg, falsepositive ) VALUES ( 
               '$report_id', '$scantime', '$hostip', '$hostname', '$service',
               '$app', '$port', '$proto', '$scriptid', '$risk',  '$msg', '$falsepositive' ); };
             safe_db_write ( $sql2, 4 );

          } #END LOOP EACH HOST RESULT
       } #END HOST LOOP

       #UPDATE HOST RECORD
       foreach my $host ( sort keys %hostHash ) {
	  my $hostip	= $hostHash{$host}{'ip'};
          my $hostname	= $hostHash{$host}{'fqdn'};
          my $mac_address = $hostHash{$host}{'mac'};
          my $os		= " ";
	  my $ip_org	= " ";
	  my $ip_site	= " ";
          if ( defined ( $hostname ) ) {
             if ( $hostHash{$host}{'os'}   ) {     $os = $hostHash{$host}{'os'};     }
	     if ( $hostHash{$host}{'org'}  ) { $ip_org = $hostHash{$host}{'org'};    }
	     if ( $hostHash{$host}{'site'} ) { $ip_site = $hostHash{$host}{'site'};  }
             logwriter( "mac=$mac_address\tname=$hostname\tip=$hostip\treport=$report_id\tos=$os\torg=$ip_org", 4 );
             update_host_record ( $mac_address, $hostname, $hostip, $os, $ip_org, $ip_site, $report_id );
          }
       }
       logwriter( "Completed SQL Import", 4 );
       update_stats ( $job_id, $job_title, $report_id, $scantime );
    } # END REPORT LOOP
}

#called by load_scanlite_results to populate stats for report
sub update_stats {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $job_id, $job_title, $report_id, $scantime ) = @_;
    my ( $sth_sel, $sql );

    my %SCAN = ();
    my %RISKS = ( 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0);
    my $hostcnt = 0;
    my $runtime = 0;
    my $trend = 0;

    if ( ! $report_id ) {
        logwriter( "UPDATE STATS: failed to lookup report for scan $scantime", 2 );
        return;
    }

    $sql = qq{ SELECT msg from nessus_results
        WHERE report_id = '$report_id' AND scriptid='19506' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    $SCAN{'Scanner IP'} = "";
    $SCAN{'Nessus version'} = "";
    $SCAN{'Type of plugin feed'} = "";
    $SCAN{'Plugin feed version'} = "";

    while ( my ( $txt_msg ) = $sth_sel->fetchrow_array ) {
        # Exclude null nessus_value records
        if ( $txt_msg && $txt_msg ne "" ) {
            my @arrMSG = split /\n/, $txt_msg;

            foreach my $line (@arrMSG) {
                $line =~ s/\#.*$//;
                chomp($line);
                $line =~ s/\s+$//;
                $line =~ s/^\s+//;
                if ($line eq "") { next; }
                my @temp=split(/:/,$line,2);
                $temp[0] =~ s/\s+$//;
                $temp[0] =~ s/^\s+//;
                $temp[1] =~ s/\s+$//;
                $temp[1] =~ s/^\s+//;
                if ($temp[0] ne "") { $SCAN{$temp[0]}=$temp[1]; }
                #print "0=$temp[0] 1=$temp[1]\n";
            }
        }
    }
    my $server_ip = "" . $SCAN{'Scanner IP'};
    my $server_nver = "" . $SCAN{'Nessus version'};
    my $server_feedtype= "" . $SCAN{'Type of plugin feed'};
    my $server_feedversion = "" . $SCAN{'Plugin feed version'};

    if ( defined ( $server_ip ) && $server_ip ne "" ) {
        $sql = qq{ UPDATE nessus_reports SET server_ip='$server_ip', 
            server_nversion='$server_nver',        server_feedtype='$server_feedtype',
            server_feedversion='$server_feedversion' WHERE report_id='$report_id' };
        safe_db_write( $sql, 4 );

        #UPDATE INFO IN THE NESSUS_SERVERS TABLE TOO IF FEED DATE CHANGED
        $sql = qq{ UPDATE nessus_servers SET server_nversion='$server_nver',
            server_feedtype='$server_feedtype', server_feedversion='$server_feedversion' 
            WHERE hostname='$server_ip' AND server_feedversion <= '$server_feedversion' };
        safe_db_write( $sql, 4 );

    }

    $sql = qq{ SELECT scan_START, scan_END FROM I3_jobs        WHERE id = '$job_id' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    my ( $start_dttm, $end_dttm ) = $sth_sel->fetchrow_array;

    $sql = qq{ SELECT count(risk) as count, risk FROM `nessus_results` WHERE report_id='$report_id'
        AND falsepositive<>'Y' AND scriptid <> 10180 GROUP BY risk };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ($count, $risk )=$sth_sel->fetchrow_array) {
        $RISKS{"$risk"} = $count;
    }

    $sql = qq{ SELECT count( distinct hostIP) as count FROM nessus_results 
        WHERE report_id='$report_id' AND falsepositive<>'Y' AND scriptid <> 10180 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    ( $hostcnt ) = $sth_sel->fetchrow_array; 
    $sth_sel->finish;

    $runtime = datediff( $start_dttm, $end_dttm, "M" );

    my $update_time = getCurrentDateTime("datetime");

    $sql = "INSERT INTO nessus_report_stats ( report_id, name, iHostCnt, dtLastScanned, iScantime,
        vSerious, vHigh, vMed, vMedLow, vLowMed, vLow, vInfo, trend, dtLastUpdated ) VALUES (
        '$report_id', '$job_title', '$hostcnt', '$scantime', '$runtime', '$RISKS{1}', '$RISKS{2}', 
        '$RISKS{3}', '$RISKS{4}', '$RISKS{5}', '$RISKS{6}', '$RISKS{7}', '$trend', '$update_time' ); ";

    safe_db_write ( $sql, 4 );

}

#called to update host tracker, most recent scan info for hosts
sub update_host_record {
    # VER: 1.1 MODIFIED: 5/22/08 07:31
    my ( $mac_address, $hostname, $hostip, $os, $ip_org, $ip_site, $report_id ) = @_;

    $os=~ s/^\s+//;  #MAY HAVE PASSED " " SO IT WOULD NOT BE NULL

    my ( $sql, $sql_select, $sth_sel, $scantime );
    my $scan_state = "Passed";  #SET INITAL STATE TO PASS
    my %RISKS = ( 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0);

    if ( defined( $report_id )) {
        $sql = qq{ SELECT count(risk) as count, risk, scantime FROM nessus_results 
            WHERE report_id='$report_id' AND hostip='$hostip' AND hostname='$hostname'
            AND falsepositive<>'Y' AND scriptid <> 10180 and msg <> '' GROUP BY risk };

        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;
        while( my ($count, $risk, $tmp_scantime )=$sth_sel->fetchrow_array) {
            $RISKS{"$risk"} = $count;
            $scantime=$tmp_scantime;
        }
    }

    if ( $ip_org eq "" ) { $ip_org = "unknown"; }
    my $risk_value = $RISKS{1} + $RISKS{2};
    if ( $risk_value > 0 ) { $scan_state = "Pending Fixes"; }

    $sql = qq{ SELECT id FROM hosts WHERE hostname = '$hostname' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    my ( $host_id ) = $sth_sel->fetchrow_array;

    if ( defined( $host_id ) && $host_id > 0 ) {
        my $os_sql = " ";  #ONLY UPDATE OS FIELD IF FOUND ( SOME PROFILES MAY NOT TEST )
	if ( defined( $os ) && $os ne "" ) { $os_sql = " os='$os',"; }
        $sql = "UPDATE hosts SET hostname='$hostname', hostip='$hostip',$os_sql
           scanstate='$scan_state', site_code='$ip_site', ORG='$ip_org', lastscandate='$scantime', report_id='$report_id' WHERE id='$host_id';";
	safe_db_write ( $sql, 4 );
    } else {
        $sql = "INSERT INTO hosts ( hostip, hostname, status, os, site_code, ORG, scanstate, report_id, lastscandate ) VALUES (
            '$hostip', '$hostname', 'Production', '$os', '$ip_site', '$ip_org', '$scan_state', '$report_id', '$scantime' );";
        safe_db_write ( $sql, 4 );
    }

    #ENSURE HOST_ID IS LOOKED UP AFTER INSERT OR QUIT 
    #LOOKUP HOST_ID AGAIN FOR UPDATE OF MACS / INCIDENT TRACKER CODE
    $sql = qq{ SELECT id FROM hosts WHERE hostname = '$hostname' LIMIT 1 };
    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    ( $host_id ) = $sth_sel->fetchrow_array;
    if ( defined( $host_id ) && $host_id > 0 ) {
        #UPDATE ONE-TO-MANY MAC/IP ADD
        if ( defined( $mac_address ) && $mac_address ne "unknown" && $mac_address ne "" ) { 
            $sql = qq{ SELECT id FROM host_macs WHERE mac_address = '$mac_address' LIMIT 1 };
            logwriter( $sql, 5 );
            $sth_sel = $dbh->prepare( $sql );
            $sth_sel->execute;
            my ( $mac_id ) = $sth_sel->fetchrow_array;
            if ( defined( $mac_id ) && $mac_id > 0 ) {
               $sql = "UPDATE host_macs SET host_id='$host_id', hostip='$hostip' WHERE id='$mac_id'";
               safe_db_write ( $sql, 4 );
            } else {
                $sql = "INSERT INTO host_macs ( host_id, hostip, mac_address, createdate ) VALUES (
                    '$host_id', '$hostip', '$mac_address', '$scantime' );";
                safe_db_write ( $sql, 4 );
            }
        }

        #AN EFFORT TO TRACK PREVIOUSLY IDENTIFIED CRITICALS / HIGHS AGAINST THIS SCAN
        #WILL CODE IN A VALIDATION TO CHECK PLUGINS TESTED AGAINST KNOWN RISKS
        #USE OF CREDENTIALS WILL AFFECT ACCURACY (OF TRACKER)
        #update_incident( id_in_hosts_table, ip_at_time_of_scan, report_id this scan );
        update_incidents ( $host_id, $hostname, $hostip, $report_id );
    } else {
	$host_id = 0;
    }

    #ADD ENTRY TO VULN_SUMMARY TABLE
    $sql = "INSERT INTO host_stats ( report_id, host_id, hostip, hostname, scantime, 
	vSerious, vHigh, vMed, vMedLow, vLowMed, vLow, vInfo ) VALUES (
        '$report_id', '$host_id', '$hostip', '$hostname', '$scantime', 
	'$RISKS{1}', '$RISKS{2}', '$RISKS{3}', '$RISKS{4}', '$RISKS{5}', '$RISKS{6}', '$RISKS{7}' );";

    safe_db_write ( $sql, 4 );

}

#called to update incident tracker, against most recent scan results
sub update_incidents {
    my ( $host_id, $hostname, $hostip, $report_id ) = @_;

    my ( $sql, $sql2, $sth_sel, $sth_sel2, $scantime );

    #$nessus_plugins

    #INSERT /UPDATE EXISTING INCIDENTS
    my $now = getCurrentDateTime("datetime");

    $sql = qq{ SELECT result_id, scriptid, risk, service, msg FROM nessus_results 
        WHERE report_id='$report_id' AND hostname='$hostname' AND hostip='$hostip' 
	AND risk <='2' AND falsepositive<>'Y' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $result_id, $scriptid, $risk, $service, $msg )=$sth_sel->fetchrow_array) {
        $sql2 = qq{ SELECT id FROM I3_Incidents
	    WHERE host_id='$host_id' AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
        logwriter( $sql2, 5 );
        $sth_sel2 = $dbh->prepare( $sql2 );
        $sth_sel2->execute;
	my ( $incident_id ) = $sth_sel2->fetchrow_array;

	if ( defined($incident_id ) && $incident_id ne "" ) {
	    $sql2 = qq{ UPDATE I3_Incidents SET result_id='$result_id', risk='$risk', msg='$msg', status='open' WHERE id="$incident_id" };
	} else {
	    $sql2 = qq{ INSERT INTO I3_Incidents ( host_id, result_id, scriptid, service, risk, msg, date_open ) VALUES (
	        '$host_id', '$result_id', '$scriptid', '$service', '$risk', '$msg', '$now' );};
	}
        safe_db_write ( $sql2, 4 );

    }


    $sql = qq{ SELECT id, scriptid, service FROM I3_Incidents
            WHERE host_id='$host_id' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $incident_id, $scriptid, $service )=$sth_sel->fetchrow_array) {
	#FAIL SAFE DO MARK ANY PLUGINS NOT TESTED AS RESOLVED
	if ( grep { $_ eq $scriptid } @nessus_plugins ) {
	    $sql2 = qq{ SELECT result_id FROM nessus_results
        	WHERE report_id='$report_id' AND hostip='$hostip' 
		    AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
            logwriter( $sql2, 5 );
	    $sth_sel2 = $dbh->prepare( $sql2 );
            $sth_sel2->execute;
	    my ( $result_id ) = $sth_sel2->fetchrow_array;

	    if ( defined($result_id ) && $result_id ne "" ) {
		#CURRENTLY NOT RESOLVED
	    } else {
		#CURRENTLY CREDENTIALS VS NO /CREDENTIALS WILL CAUSE AN INVALID CLEANUP STATE TO BE SET
	        $sql2 = qq{ UPDATE I3_Incidents SET status='resolved', date_resolved='$now' WHERE id='$incident_id' };
	    }
            safe_db_write ( $sql2, 4 );

	} else {
	    logwriter( "PLUGIN $scriptid apparrently was not tested", 5 );
	}

    }

}

#needed for host updated
sub extract_os {
    # VER: 1.0 MODIFIED: 11/15/07 15:43
    my ( $txt_msg ) = @_;
    my $os = "";
    my @arrMSG = split /\n/, $txt_msg;

    foreach my $line (@arrMSG) {
        $line =~ s/\#.*$//;
               chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line =~ "Remote operating system") {
            my @temp=split(/:/,$line,2);
            $temp[1] =~ s/\s+$//;
            $temp[1] =~ s/^\s+//;
            $os = $temp[1];
        } else { next; }
    }

    if ( defined( $os ) ) {
        logwriter( $os, 5 );
        return $os;
    } else {
        return "";
    }

}

#needed for host updated / load results
sub resolve_host {
    # VER: 2.0 MODIFIED: 5/06/08 15:30
    my ( $hostip ) = @_;

    if ( ! defined ( $hostip) || $hostip eq "" ) { return ""; }

    # ATTEMPT GET HOST BY ADDRESS WILL CHECK FILES/DNS PRIMARY
    my $iaddr = inet_aton( $hostip ); # or whatever address
    my $namer  = gethostbyaddr($iaddr, AF_INET);

    if ( defined($namer ) ) {
        my $thost = lc ( $namer );
        #logwriter( $thost, 5 );
        return $thost;
    } else {

	if ( $CONFIG{'nameservers'} ne "" ) {
	    my @nameservers = split /,/, $CONFIG{'nameservers'};
	    foreach my $nameserver ( @nameservers ) {
		$nameserver =~ s/\s+//g;
		my $namer = nslookup(host => "$hostip", type => "PTR", server => "$nameserver" );
		if ( defined($namer ) && $namer ne "" ) {
	            my $thost = lc ( $namer );
	    	return $thost;
	        }
	    }
	} 
    }
    logwriter( "REVERSE IP [$hostip] TO NAME FAILED\n", 3 );
    return "";

}

#ENSURE IP'S ARE IN THE BUILDLIST Otherwise Hostnames are returned instead IP's
#THEY GET STUFFED IN THE HOSTIP Field as a result
sub resolve_name2ip {
    # VER: 2.0 MODIFIED: 5/06/08 15:30
    my ( $hostname ) = @_;
    if ( ! defined ( $hostname ) || $hostname eq "" ) { return ""; }

    # ATTEMPT GET HOST BY ADDRESS WILL CHECK FILES/DNS PRIMARY
    my $packed_ip = gethostbyname( $hostname );

    if ( defined( $packed_ip ) ) {
        my $c_ip = inet_ntoa($packed_ip);
        return $c_ip;
    } else {
        #TRY OTHER NAMES SERVERS
	if ( $CONFIG{'nameservers'} ne "" ) {
	    my @nameservers = split /,/, $CONFIG{'nameservers'};

	    foreach my $nameserver ( @nameservers ) {
		$nameserver =~ s/\s+//g;
		my $namer = nslookup(host => "$hostname", server => "$nameserver" );
		if ( defined($namer ) && $namer ne "" ) {
	            my $thost = lc ( $namer );
	    	    return $thost;
	        }
	    }
	} 
    }
    logwriter( "RESOLVE [$hostname] TO IP FAILED\n", 3 );
    return "";
}

#needed for report stats
sub datediff {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $start_date, $end_date, $unit ) = @_;
    my ( %start, %end );

    if ( ! defined( $start_date ) || ! defined( $end_date ) ) {
        return  -1;
    }

    $start_date =~ s/\///g;
    $start_date =~ s/\s//g;
    $start_date =~ s/-//g;
    $start_date =~ s/://g;

    $start{YEAR} = substr($start_date, 0,4 );
    $start{MO} = substr($start_date, 4,2 );
    $start{D} = substr($start_date, 6,2 );
    $start{H} = substr($start_date, 8,2 );
    $start{M} = substr($start_date, 10,2 );
    $start{S} = substr($start_date, 12,2 );

    $end_date =~ s/\///g;
    $end_date =~ s/\s//g;
    $end_date =~ s/-//g;
    $end_date =~ s/://g;

    $end{YEAR} = substr($end_date, 0,4 );
    $end{MO} = substr($end_date, 4,2 );
    $end{D} = substr($end_date, 6,2 );
    $end{H} = substr($end_date, 8,2 );
    $end{M} = substr($end_date, 10,2 );
    $end{S} = substr($end_date, 12,2 );

    my ($Dd,$Dh,$Dm,$Ds) = Delta_DHMS($start{YEAR},$start{MO},$start{D}, $start{H},$start{M},$start{S},
        $end{YEAR},$end{MO},$end{D}, $end{H},$end{M},$end{S});

    my $diff = 0;

    if ( $Dd ) { $diff += ( $Dd * 216000 ); }
    if ( $Dh ) { $diff += ( $Dh * 3600 ); }
    if ( $Dm ) { $diff += ( $Dm * 60 ); }
    if ( $Ds ) { $diff += $Ds }

    if ( $unit eq "D" ) {
        $diff = ( $diff / 216000 );
    } elsif ( $unit eq "H" ) {
        $diff = ( $diff / 3600 );
    } elsif ( $unit eq "M" ) {
        $diff = ( $diff / 60 );
    } else {
        # Already seconds do not convert; 
    }

    $diff = sprintf("%.2f", $diff);

    return $diff;
}

#reusable routine to get subnet id from CIDR
sub getSubnetID {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $subnet ) = @_;
    my ($sql, $sth_sel, $sub_id);

    ### CHECK THE DATABASE FOR A MATCHING SUBNET
    $sql = qq{ SELECT id FROM I3_subnets WHERE Name='$subnet' LIMIT 1 };

    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    ### Retrieve the returned rows of data
    while ( my ( $id ) = $sth_sel->fetchrow_array(  ) ) {
        $sth_sel->finish;
        return $id;
    }
}

#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}

