#!/usr/bin/perl
#
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##
##    PRGM AUTHOR: kkline
##   PROGRAM NAME: maintainence
##   PROGRAM DATE: 08/01/2008
##  PROGM VERSION: 0.5
##  REVISION HIST:
##       08/01/2008 - FIRST VERSION TO MOVE BACKEND MAINTAINENCE INTO SINGLE SCRIPT
##
use strict; 
 

use DBI; 
# PERL MODULES WE WILL BE USING
use DBI;
use DBD::mysql;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use Getopt::Std;
use Switch;

my $debug	    = 0;
my $log_level	    = 4;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/maintainence.log";

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

my ( $dbh, $sql, $sql2, $sth, $sth_upd, $sth_sel, $sth_sel2, $stime );	    


my %options=();
getopts("cdij:k:l:no:qr:st:u:v:w:h?",\%options);

main( );
exit;

sub main {

    if( $options{d} || $debug eq "1" ) {         		    #ENABLE DEBUGGING
	use warnings;
        print "Debugging mode\n";
        $debug = 1;
	$log_level = 5;
    }

    open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
        *STDERR=*LOG;
        *STDOUT=*LOG;

    $dbh = conn_db();
    #USE Front End Variables
    load_db_configs ( );

    #UPDATE INCIDENT TRACKER WITH EXCEPTIONS STATUS
    update_exceptions( );

    #ASSIGN ALL ISVMS A RISK VALUE OF HIGH IF LOWER ( MEDIUM/LOW/INFO )
    update_isvm_custom_risk_value ( );

    #UPDATE RISK WITH CUSTOM RISK VALUE
    update_isvm_stats_values ( );
    exit;

    #UPDATE SCAN STATE IS DONE VIA nessusCron for all hosts with a FQDN.

    #CHECK INACTIVE HOST PER THE inprotect_settings maxhostAge settings
    #FLAG THEM INACTIVE IF HAVE NOT BEEN SCANNED in days > maxHostAge ( deleted / filtered from view )
    update_inactive ( );

    #CHECK OUT OF COMPLIANCE HOST PER THE inprotect_settings maxscanAge settings
    #FLAG THEM OUT OF COMPLIANCE IF HAVE NOT BEEN SCANNED in days > maxscanAge
    update_scan_status ( );

    #REASSOCIATE VULNS FOR HOSTNAME WHERE HOSTNAME.DOMAIN.X NOW EXISTS
    #Typically a machine was scanned prior to joining the domain and the hostname alone
    #May have existed in the hosts table
    remove_dups ( );

    #IMPORT CHECKS FOR AUDIT FILES TO ALLOW USE OF AUDIT and default to restricted access.
    load_audit_checks ( );

    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");

}


sub update_isvm_stats_values {
   my ( $sql, $sth_sel, $sth_sel2, $sth_sel3 );

   $sql = qq{ SELECT id FROM I3_ISVM WHERE archived = '0' };

   logwriter( $sql, 5 );

   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   while ( my ( $iid )= $sth_sel->fetchrow_array ) {

      $sql = qq{ SELECT count(nessus_id) FROM I3_ISVM_plugins WHERE iid='$iid' };
      logwriter( $sql, 5 );
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );
      my ( $pcount  ) = $sth_sel2->fetchrow_array( );

      $sql = qq{ SELECT t1.id FROM plugins t1 LEFT JOIN I3_ISVM_plugins t2 on t1.id = t2.nessus_id
            WHERE t2.iid = '$iid' };
      logwriter( $sql, 5 );
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );

      my $vcount = 0;
      while ( my ( $plugid )= $sth_sel2->fetchrow_array ) {
          $sql = qq{ SELECT count(id) FROM I3_Incidents WHERE scriptid='$plugid' AND status !='resolved' };
          logwriter( $sql, 5 );
          $sth_sel3 = $dbh->prepare( $sql );
	  $sth_sel3->execute(  );
	  my ( $tmpCount  ) = $sth_sel3->fetchrow_array( );
          $vcount += $tmpCount; 
      }
      $sql = qq{ UPDATE I3_ISVM SET iCntPlugs='$pcount', iVulnHosts='$vcount' WHERE id='$iid' LIMIT 1  };
      safe_db_write( $sql, 4);
   }

}

sub update_isvm_custom_risk_value {
   my ( $sql, $sth_sel );

   $sql = qq{ SELECT t2.id FROM I3_ISVM_plugins t1
      LEFT JOIN nessus_plugins t2 on t1.nessus_id = t2.id 
      WHERE t2.risk>'2' AND custom_risk IS NULL };

   logwriter( $sql, 5 );

   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   while ( my ( $plugin_id )= $sth_sel->fetchrow_array ) {
      #YOU CHOSE TO UPGRADE RISK TO CRITICAL/HIGH
      $sql = qq{ UPDATE nessus_plugins SET custom_risk='2' WHERE id='$plugin_id' };
      safe_db_write( $sql, 4);

      #UPGRADE EXISTING RESULTS TO A HIGH
      $sql = qq{ UPDATE nessus_results SET risk='2' WHERE scriptid='$plugin_id' };
      safe_db_write( $sql, 4);

      #CHECK TO ADD EXISTING DATA TO INCIDENTS TO TRACKER
      import_incidents( $plugin_id );

      check_incident_closed ( $plugin_id );

   }
}

sub check_incident_closed {
    my ( $plugin_id ) = @_;

    my ( $sql, $sth_sel, $sth_sel2, $sth_sel3 );

    my $now = getCurrentDateTime("datetime");

    $sql = qq{ SELECT t1.id, t2.id, t2.hostip, t2.hostname, t1.date_open
FROM I3_Incidents t1
LEFT JOIN hosts t2 ON t2.id = t1.host_id
WHERE t1.scriptid='$plugin_id' AND t1.status = 'open'
GROUP BY host_id };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    while ( my ( $incident_id, $host_id, $hostip, $host, $open  )= $sth_sel->fetchrow_array ) {
	#Microsoft Windows XPConfidence Level : 99Method : MSRPC The remote host is running Microsoft Wi

	if ( $host eq "" ) { next; }

	$open =~ s/-//g;
	$open =~ s/\s//g;
	$open =~ s/://g;

	print "REPROCESSING id=$incident_id\tIP=$hostip\tNAME=$host\topen=$open\n";

	$sql = qq{ 
SELECT t1.report_id, t2.scantime, t2.sid
FROM hosts t1
LEFT JOIN nessus_reports t2 ON t1.report_id = t2.report_id
WHERE t1.id='$host_id' AND t2.scantime >= '$open'
};
        logwriter( $sql, 5 );
        $sth_sel2 = $dbh->prepare( $sql );
	$sth_sel2->execute(  );

        while ( my ( $report_id, $scantime, $sid  )= $sth_sel2->fetchrow_array ) {
	    print "report_id=$report_id\tscantime=$scantime\tsid=$sid\n";
	    my @nessus_plugins = get_plugins( $sid );

	    $sql = qq{ SELECT scriptid FROM nessus_results WHERE report_id='$report_id' AND
               hostname='$host' and scriptid='$plugin_id' };
	    logwriter( $sql, 5 );
            $sth_sel3 = $dbh->prepare( $sql );
	    $sth_sel3->execute(  );
            my ( $scriptid )= $sth_sel3->fetchrow_array (  );

            print "scriptid=$scriptid\t";

            if ( $scriptid ) {
               print "Not Patched\n";
	    } else {
                if ( $nessus_plugins[$scriptid] ) {
		    print "plugin tested\n";

	            $sql2 = qq{ UPDATE I3_Incidents SET status='resolved', date_resolved='$now' WHERE id='$incident_id' };
	            safe_db_write ( $sql2, 4 );

	        } else {
		    print "plugin not tested\n";

	        }

	    }
	}
	print "\n";

    }
}


#called to update incident tracker, against most recent scan results
sub import_incidents {
    my ( $plugin_id ) = @_;

    my ( $sql, $sql2, $sth_sel, $sth_sel2, $scantime );

    #$nessus_plugins

    #INSERT /UPDATE EXISTING INCIDENTS

    $sql = qq{ SELECT t1.result_id, t1.scantime, t1.scriptid, t1.risk, t1.service, t1.msg, t1.hostip, t2.id 
        FROM nessus_results t1
        LEFT JOIN hosts t2 on t1.hostname=t2.hostname
        WHERE scriptid='$plugin_id' AND t2.id > 0 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $result_id, $date_open, $scriptid, $risk, $service, $msg, $hostip, $host_id )=$sth_sel->fetchrow_array) {
        $sql2 = qq{ SELECT id FROM I3_Incidents
	    WHERE host_id='$host_id' AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
        logwriter( $sql2, 5 );
        $sth_sel2 = $dbh->prepare( $sql2 );
        $sth_sel2->execute;
	my ( $incident_id ) = $sth_sel2->fetchrow_array;

	if ( defined($incident_id ) && $incident_id ne "" ) {
	    $sql2 = qq{ UPDATE I3_Incidents SET date_open='$date_open' WHERE id="$incident_id" };
	} else {
	    $sql2 = qq{ INSERT INTO I3_Incidents ( host_id, result_id, scriptid, service, risk, msg, date_open ) VALUES (
	        '$host_id', '$result_id', '$scriptid', '$service', '$risk', '$msg', '$date_open' );};
	}
        safe_db_write ( $sql2, 4 );
    }

}


sub update_exceptions {
   my ( $sql, $sth_sel, $sth_sel2 );

   $sql = qq{ SELECT eid, status, plugin, hostip, hostname, port FROM exceptions };
   logwriter( $sql, 5 );

   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   while ( my ( $eid, $estatus, $eplugin, $ehostip, $ehostname, $eport )= $sth_sel->fetchrow_array ) {
      my $sql_filter="";

      if ( ! ( $ehostip eq "ANY" || $ehostname eq "ANY" ) ) { $sql_filter = " AND t2.hostname='$ehostname'";   } 
      if ( ! ( !defined($eport) || $eport == "" || $eport == "0" ) ) { $sql_filter .= " AND t1.service LIKE '%($eport/%'"; }

      $sql = qq{ SELECT t1.id FROM I3_Incidents t1
         LEFT JOIN hosts t2 on t1.host_id = t2.id 
         WHERE t1.scriptid='$eplugin' AND t1.status='open' $sql_filter };
      logwriter( $sql, 5 );

      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );

      while ( my ( $match_id ) =  $sth_sel2->fetchrow_array ) {
         my $sql_update="";
         if ( $estatus eq "Approved" ) {
            $sql_update = "exception_id='$eid', exception_approved='1'";
         } else {
            $sql_update = "exception_id=NULL, exception_approved='0'";
         }
         $sql = qq{ UPDATE I3_Incidents SET $sql_update WHERE id='$match_id'; };
         safe_db_write ( $sql, 4);

      }
   }

}

sub load_audit_checks {

  my @checks = ("unixAuditDir", "winAuditDir", "winFileAudits" );

    foreach my $check ( @checks )  {
	$sql = qq{ SELECT settingValue FROM inprotect_settings WHERE settingName='$check' LIMIT 1 };
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );
        my ( $check_dir ) = $sth_sel->fetchrow_array(  );
	$sth_sel->finish;

	my @files = <$check_dir/*.audit>; 
	foreach my $file (@files) {
	    print $file . "\n";
	    process_audit_file( $file, $check_dir, $check );
	} 
    }
}

sub update_inactive {

    my $curr_time = time;

    logwriter( "CALLED UPDATE INACTIVE", 5 );

    #DEFAULT TO INVALID STATE TO MAKE SURE WE DON'T ACCIDENTLY PRUNE ALL RECORDS
    #SHOULD THE WEBFORM HAVE A NON NUMERIC VALUE OR LESS THAT 90
    my $prune_age = "-1";

    if ( is_number( $CONFIG{'maxHostAge'} ) && $CONFIG{'maxHostAge'} >= 60 ) {
       $prune_age = $CONFIG{'maxHostAge'};
       logwriter( "PRUNE_AGE=$prune_age", 5 );
    }

    if ( $prune_age > 0 ) {
       my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(($curr_time - ( 86400 * $prune_age)));
       $year+=1900;
       $mon++;

       my $prune_date = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
       logwriter( "PRUNING ENTRIES PRIOR TO DATE $prune_date", 4 );

       $sql = qq{ UPDATE hosts SET inactive='1' WHERE lastscandate <= '$prune_date' AND inactive=0 };
       safe_db_write( $sql, 4);

       $sql = qq{ SELECT t1.id, t2.hostname FROM I3_Incidents t1
          LEFT JOIN hosts t2 ON t1.host_id=t2.id
          WHERE t1.status!='resolved' AND t2.inactive='1' ORDER BY RISK };
       logwriter( $sql, 5 );
       $sth_sel = $dbh->prepare( $sql );
       $sth_sel->execute(  );

       while ( my ( $incident_id, $hostname )= $sth_sel->fetchrow_array ) {
          my $note = "Vulnerability Closed by inactive Host Script\n\nSystem [ $hostname ] has not been audited on the network for $prune_age days";
          $sql2 = qq{ UPDATE I3_Incidents SET notes='$note', date_resolved=now(), status='resolved' WHERE id='$incident_id'; };
          safe_db_write( $sql2, 4);

       }
       $sth_sel->finish;
    }
}

sub update_scan_status {

    my $curr_time = time;

    logwriter( "CALLED UPDATE SCAN STATUS", 5 );

    #DEFAULT TO INVALID STATE TO MAKE SURE WE DON'T ACCIDENTLY PRUNE ALL RECORDS
    #SHOULD THE WEBFORM HAVE A NON NUMERIC VALUE OR LESS THAT 90
    my $max_age = "-1";

    if ( is_number( $CONFIG{'maxScanAge'} ) && $CONFIG{'maxScanAge'} >= 7 ) {
       $max_age = $CONFIG{'maxScanAge'};
       logwriter( "MAX_AGE=$max_age", 5 );
    }

    if ( $max_age > 0 ) {
       my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(($curr_time - ( 86400 * $max_age)));
       $year+=1900;
       $mon++;

       my $maxage_date = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
       logwriter( "MARKING HOSTS PENDING SCAN WITH NO RECENT SCAN AS OF DATE $maxage_date", 4 );

       $sql = qq{ UPDATE hosts SET scanstate='Pending Scan' WHERE lastscandate <= '$maxage_date' AND inactive=0 };
       safe_db_write( $sql, 4);
    }
}

sub remove_invalid_incidents {

    $sql = qq{ SELECT t1.id, t3.hostname
FROM I3_Incidents t1
LEFT JOIN hosts t2 ON t1.host_id = t2.id
LEFT JOIN nessus_results t3 ON t1.result_id = t3.result_id
AND t2.hostname = t3.hostname
WHERE t3.hostname is NOT Null AND t3.hostip = '' };

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ($id, $hostname )=$sth_sel->fetchrow_array) {
	if ( $hostname  ne "" ) {
	    print "DELETE FROM I3_Incidents WHERE id='$id' LIMIT 1;\n";
	}
	
    }
    $sth_sel->finish;
}

sub remove_dups {

   print "starting..\n";

    $sql = qq{ SELECT id, hostname FROM hosts };

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ($host_id, $hostname )=$sth_sel->fetchrow_array) {

	if ( $hostname =~ /\./ ) {

	    print "valid fqdn [$hostname]\n";


	} else {

	    $sql = qq{ SELECT count(hostname) FROM hosts WHERE hostname LIKE '%$hostname%' };
	    print "query=[$sql]\n";
	    $sth_sel2 = $dbh->prepare( $sql );
	    $sth_sel2->execute;
	    my ( $count ) = $sth_sel2->fetchrow_array ( );

	    if ( $hostname ne "" && $count > 1 ) {		
	        print "REMOVING THIS DUPLICATE HOST: [ $hostname ]\n";

	        $sql = qq{ DELETE FROM hosts WHERE id='$host_id' };		
	        safe_db_write ( $sql, 4 );

			
    	        $sql = qq{ SELECT id FROM hosts WHERE hostname LIKE '%$hostname%' AND id !='$host_id' LIMIT 1 };
		$sth_sel2 = $dbh->prepare( $sql );
	        $sth_sel2->execute;
	        my ( $replace_id  ) = $sth_sel2->fetchrow_array ( );
	
       	        if ( $replace_id ) {
	
      	            $sql = qq{ UPDATE host_macs SET host_id='$replace_id' WHERE host_id='$host_id'};
		    safe_db_write ( $sql, 4 );
			
    	  	    $sql = qq{ UPDATE host_stats SET host_id='$replace_id' WHERE host_id='$host_id'};
		    safe_db_write ( $sql, 4 );
			
      		    $sql = qq{ UPDATE I3_Incidents SET host_id='$replace_id' WHERE host_id='$host_id'};
		    safe_db_write ( $sql, 4 );
       		}	
	    }
            $sth_sel2->finish;
	}
    }

    $sth_sel->finish;
}

sub process_audit_file {
    my ( $audit_file, $check_dir, $check ) = @_;

    my ( $sql, $sth_sel );

    open(DAT, $audit_file) || die("Could not open file!");
	my @raw_data=<DAT>;
    close(DAT);

    $audit_file =~ s/$check_dir\///g;

    $sql = qq{ SELECT id FROM nessus_audits WHERE name='$audit_file' LIMIT 1 };
    logwriter( $sql, 5 );
    ### LOOKUP NESSUS CHECK BY NAME
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    my $match_id = $sth_sel->fetchrow_array(  );

    if ( ! $match_id ) {
	$sql = qq{ INSERT INTO nessus_audits ( name, description, check_type, type, owner
	    ) VALUES (  '$audit_file', "Imported by Backend Script", "$check", "P", "Admin" ); };
	safe_db_write ( $sql, 4);

    }

    my $now = getCurrentDateTime("datetime");

    my $isRecord = FALSE;

    my ( $temp_name, $temp_value ) = "";

    foreach my $line (@raw_data) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
	
	my ( $name, $value )=split(/:/,$line,2);
        $name =~ s/\s+$//;
        $name =~ s/^\s+//;

	if ( defined( $value ) && $value ne "" ) {
	    $value =~ s/\"//g;
	    $value =~ s/\\/\\\\/g;
	    $value =~ s/\n//g;
            $value =~ s/\s+$//;
            $value =~ s/^\s+//;
	}
	
	if ( $name =~ /<item>/ || $name =~ /<custom_item>/  ) {
	    $isRecord = TRUE;
	} elsif ( $name =~ /<\/item>/ ) {
	    $isRecord = FALSE;

    	    print "checking setting $temp_value\n";

	    $sql = qq{ SELECT id FROM nessus_checks WHERE name='$temp_name' LIMIT 1 };
            logwriter( $sql, 5 );
            ### LOOKUP NESSUS CHECK BY NAME
            $sth_sel = $dbh->prepare( $sql );
       	    $sth_sel->execute(  );
            my $match_id = $sth_sel->fetchrow_array(  );

	    if ( ! $match_id ) {
	        my $sql2 = qq{ INSERT INTO nessus_checks ( enabled, name, created
		    ) VALUES (  'Y', '$temp_name', '$now' ); };
		safe_db_write ( $sql2, 4);
		### LOOKUP NESSUS CHECK BY NAME AFTER INSERT
		$sth_sel = $dbh->prepare( $sql );
		$sth_sel->execute(  );
        	$match_id = $sth_sel->fetchrow_array(  );
	    }
	    
	    if ( ! $match_id ) {
		logwriter( "FAILED LOOKUP check name \"$temp_name\" AFTER AN INSERT", 1 );
		exit;
	    }

	    $sql = qq{ SELECT id FROM nessus_checks_audits WHERE cid='$match_id' AND auditfile='$audit_file' LIMIT 1 };
            logwriter( $sql, 5 );
            $sth_sel = $dbh->prepare( $sql );
       	    $sth_sel->execute(  );
            ### Retrieve the returned rows of data
            my $entry_id = $sth_sel->fetchrow_array(  );

	    if ( ! $entry_id ) {
	        $sql = qq{ INSERT INTO nessus_checks_audits ( cid, enabled, created, value, auditfile
		    ) VALUES (  '$match_id', 'Y', '$now', '$temp_value','$audit_file' ); };
		safe_db_write ( $sql, 4);

	    }

	    $temp_name = "";
	    $temp_value = "";

	} elsif ( $isRecord && ( $name =~ /^name$/ || $name =~ /description/ ) ) {
	    $temp_name =  $value;
	} elsif ( $isRecord && ( $name =~ /^value$/ || $name =~ /value_data/ ) ) {
	    $temp_value= $value;
	}
    }

    $sth_sel->finish;
}


# returns an array of the plugins enabled for this scan id
sub get_plugins {
    my ($sid ) = @_;
    my (@plugins);

    my ($sth_sel, $sth_sel2, $sql, $nessus_id, $nessus_value);

    #LOOKUP AUTOENABLE
    $sql = qq{ SELECT autoenable FROM nessus_settings WHERE id=$sid };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    my ($autoenable)=$sth_sel->fetchrow_array;

    #SCANNER SET
    $sql = qq{ SELECT t1.id, t1.enabled from nessus_settings_plugins t1
        LEFT JOIN plugins t2 ON t1.id=t2.id
        where t2.category ='scanner' and t1.sid=$sid order by id };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
        if ($nessus_value eq "N") {
        } else {
            push(@plugins,"$nessus_id");
        }
    }

    if ($autoenable eq "C") {
        $sql = qq{ SELECT t1.cid, t1.status FROM nessus_settings_category t1
            LEFT JOIN nessus_category t2 ON t2.id=t1.cid
         WHERE t2.name<>"scanner" AND t1.sid=$sid ORDER BY t2.id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while ( my ($catid, $catstatus)=$sth_sel->fetchrow_array ) {
            $sql = qq{ SELECT t1.id, t1.enabled FROM nessus_settings_plugins t1 
                LEFT JOIN nessus_plugins t2 ON t1.id=t2.id
               WHERE t1.sid=$sid AND t2.category =$catid ORDER BY t1.id };
            $sth_sel2=$dbh->prepare( $sql );
            $sth_sel2->execute;

            while (($nessus_id, $nessus_value)=$sth_sel2->fetchrow_array ) {
                if ($nessus_value eq "N") {
                } else {
                    push(@plugins,$nessus_id);
                }
            } #end loop
        } #end if
    } elsif ($autoenable eq "F") {

        $sql = qq{ SELECT t1.fid, t1.status FROM nessus_settings_family t1
            LEFT JOIN nessus_family t2 ON t2.id=t1.fid
         WHERE t2.name<>"Port scanners" AND t1.sid=$sid ORDER BY t2.id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while ( my ($famid, $famstatus)=$sth_sel->fetchrow_array ) {

            $sql = qq{ SELECT t1.id, t1.enabled FROM nessus_settings_plugins t1
                LEFT JOIN nessus_plugins t2 ON t1.id=t2.id
               WHERE t2.family =$famid AND t1.sid=$sid ORDER BY id };
            $sth_sel2=$dbh->prepare( $sql );
            $sth_sel2->execute;

            while (($nessus_id, $nessus_value)=$sth_sel2->fetchrow_array ) {
               if ($nessus_value eq "N") {
               } else {
                  push(@plugins,$nessus_id);
               }
            } #end loop
         } #end if

    } else {
        $sql = qq{ SELECT t1.id, t1.enabled from nessus_settings_plugins t1
            LEFT JOIN plugins t2 on t1.id=t2.id
            WHERE t2.category <>'scanner' AND t1.sid=$sid ORDER BY id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
            if ($nessus_value eq "N") {
            } else {
                push(@plugins,$nessus_id);
            }
        }
    }
    $sth_sel->finish;
    return(@plugins);
}

#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

sub read_file {
    # VER: 1.0 MODIFIED: 6/27/08 11:36
    my ( $myfile ) = @_;
    my ( $string ) = "";

    open FILE, "$myfile" or die "Couldn't open file: $!"; 
    while (<FILE>){
	$string .= $_;
    }
    close FILE;

    return $string;

}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
