#!/usr/bin/perl -w
#
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##
##   PROGRAM NAME: OS FingerPrint TOOL
##   PROGRAM DATE: 01/13/2007
##  PROGM VERSION: 0.1
##  REVISION HIST:
##        01/13/2007 - FIRST VERSION OF OS FINGERPRINT IDENTIFIER
##        03/18/2008 - Reworking Code to Better Inventory Network

use strict;
use warnings;

#Use Perl modules;
#use Parallel::ForkManager;
use DBI;
use Switch;
use Nmap::Parser;
use Getopt::Std;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/fingerprint.log";

#BEGIN { push( @INC, "$CONFIG{'ROOTDIR'}/usr/share/Inprotect/perl" ); };

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

#GLOBAL VARIABLES
my $debug              = 0;
my $log_level          = 4;

#my $serverid = "$CONFIG{'SERVERID'}";
my $serverid = "";

#Declare Public Variables to use
my $outfile = "";
my $datadir = "$CONFIG{'ROOTDIR'}/usr/data/nmap";
my $outdir = "$CONFIG{'ROOTDIR'}/tmp";


my ($dbh, $sth_sel, $sth_upd, $sth_upd1, $sel_id, $SiteID, $subTemp);
my ( $iCntScanned, $iCntExists ) = 0;


# READ ARGUMENTS FROM COMMAND-LINE
my %options=();
getopts("dhl:n:t:?",\%options);

print localtime(time)." Begin Examine NMap Scan Results\n\n";
main( );
print localtime(time)."End Examine NMap Scan Results\n\n";
exit;

sub main {
    my ( $work, $outfile, $subnet_name, $qlan, $qtime );

    if( $options{d} ) {                #ENABLE DEBUGGING
        print "Debugging mode\n";
        $debug = 1;
    }

    if( $options{t} && $options{n} ) {        #NORMAL RUN DOES WORK
        print "Cron mode\n";

        open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
            *STDERR=*LOG;
            *STDOUT=*LOG;

        my $search = "";
        if ( $options{n} ne "" ) { 
            $search = $options{n}; 
        } else {
            return;
        }
        $dbh = conn_db();

        if ( $options{t} =~ /zone/i ) {
            process( "zone", $search );
        } elsif ( $options{t} =~ /site/i ) {
            process_site ( $search );
        } elsif ( $options{t} =~ /subnet/i ) {
            process_subnet ( $search );
        }


        disconn_db($dbh);
        #end of main

    } elsif( $options{l} ) {        #LIST ZONE/SITE/SUBNET

        my $search = "";
        if ( defined( $options{n} ) && $options{n} ne "" ) { $search = $options{n}; }

        if ( $options{l} ne "" ) {
            # CONNECT TO DATABASE
            $dbh = conn_db();
            if ( $options{l} =~ /zone/i ) {
                list_zones( $search );
            } elsif ( $options{l} =~ /site/i ) {
                list_sites( $search );
            } elsif ( $options{l} =~ /subnet/i ) {
                list_subnets( $search );
            }

            disconn_db($dbh);

        } else {
            print "[$$]\tInvalid argument $options{t} for nessusCron.pl\n\n";
            exit;
        }
    } else {        #DISPLAY USAGE
        print "USAGE: osFingerPrint.pl\n\n";
        print "\tosFingerPrint.pl -l [subnet|site|zone]\t - Get a ranges to audit.\n";
        print "\tosFingerPrint.pl -t zone -n 1\t - Process LAN Zone 1\n";
        print "\tosFingerPrint.pl -t site -n \"Crystal\"\t - Process Crystal City Site Ranges\n";
        print "\tosFingerPrint.pl --help \t - Get This Help\n";
        exit;
    }

    exit;        #PROGRAM NORMAL END
}

sub list_zones {
    my ( $search ) = @_;

    my ( $sql, $sth_sel, $sql_where );   

    if ( $search ne "" ) {
        $sql_where = "name like '%$search%'";
    } else {
        $sql_where = "1";
    }

    $sql = qq{ SELECT id, name, description FROM `nessus_zones` WHERE $sql_where ORDER BY id };
    logwriter ( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    printf("%-6s%-6s%-25s%-100s\n","", "ID", "NAME", "DESCRIPTION");
    while ( my ( $Lid, $Lname, $Ldesc ) = $sth_sel->fetchrow_array(  ) ) {
        if ( !defined( $Ldesc ) || $Ldesc eq "" ) { $Ldesc = " "; }
        printf("%-6s%-6s%-25s%-100s\n","", $Lid, $Lname, $Ldesc);
    }
}

sub list_sites {
    my ( $search ) = @_;

    my ( $sql, $sth_sel, $sql_where );   

    if ( $search ne "" ) {
        $sql_where = "name like '%$search%' OR site_code like '%$search%' ";
    } else {
        $sql_where = "1";
    }

    $sql = qq{ SELECT id, name, site_code, description FROM I3_sites WHERE $sql_where ORDER BY id };
    logwriter ( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    printf("%-6s%-6s%-35s%-25s%-100s\n","", "ID", "NAME", "SITECODE", "DESCRIPTION");
    while ( my ( $Lid, $Lname, $Lsitecode, $Ldesc ) = $sth_sel->fetchrow_array(  ) ) {
        if ( !defined( $Ldesc ) || $Ldesc eq "" ) { $Ldesc = " "; }
        printf("%-6s%-6s%-35s%-25s%-100s\n","", $Lid, $Lname, $Lsitecode, $Ldesc);

    }
}

sub list_subnets {
    my ( $search ) = @_;

    my ( $sql, $sth_sel, $sql_where );   

    if ( $search ne "" ) {
        $sql_where = "CIDR like '%$search%' OR site_code like '%$search%'";
    } else {
        $sql_where = "1";
    }

    $sql = qq{ SELECT id, CIDR, site_code, description FROM I3_subnets WHERE $sql_where ORDER BY id };
    logwriter ( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    printf("%-6s%-6s%-35s%-25s%-100s\n","", "ID", "NAME", "SITECODE", "DESCRIPTION");
    while ( my ( $Lid, $Lname, $Lsitecode, $Ldesc ) = $sth_sel->fetchrow_array(  ) ) {
        if ( !defined( $Ldesc ) || $Ldesc eq "" ) { $Ldesc = " "; } else { chomp( $Ldesc ); }
        printf("%-6s%-6s%-35s%-25s%-100s\n","", $Lid, $Lname, $Lsitecode, $Ldesc);

    }
}

sub process_site ( ){
    my ( $SiteID ) = @_;

    if ( !$SiteID ) {
        print "Missing SiteID\n";
    }

    my ( $sql, $sth_sel ); 

    $sql = qq{ SELECT id, name, site_code, description FROM I3_sites WHERE id='$SiteID' LIMIT 1 };
    logwriter ( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    my ( $Lid, $Lname, $Lsitecode, $Ldesc ) = $sth_sel->fetchrow_array(  );

    if ( $Lid ) {
        print "\t$Lid\t$Lname\t$Ldesc\n";

        ### Select Network Subnets to Process
        $sql = qq{ SELECT CIDR FROM I3_subnets
            WHERE site_code = "$Lsitecode" AND status != "available" AND 
                serial_flag="N" AND auditable="Y" ORDER BY dtLastNMAP };
        logwriter ( $sql, 5 );
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );

        ### Retrieve the returned rows of data
        while ( my ( $subnet, $tmpSubCount ) = $sth_sel->fetchrow_array(  ) ) {
            prep_job ( $subnet );
        }

    } else {
        print "Invalid LanID [$SiteID], try running osFingerprint.pl --list\n\n";
    }

    $sth_sel->finish;

}

sub process_subnet {
    my ( $SubID ) = @_;

    my ( $sql, $sth_sel ); 

    $sql = qq{ SELECT id, CIDR, site_code, description FROM I3_subnets WHERE id="$SubID" LIMIT 1 };
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    my ( $Lid, $Lname, $Lsitecode, $Ldesc ) = $sth_sel->fetchrow_array(  );

    if ( $Lid ) {
        print "\t$Lid\t$Lname\t$Ldesc\n";

        ### Select Network Subnets to Process
        $sql = qq{ SELECT CIDR FROM I3_subnets
            WHERE id="$Lid" AND status != "available" AND 
                serial_flag="N" AND auditable="Y" ORDER BY dtLastNMAP };
        logwriter ( $sql, 5 );
        $sth_sel->execute(  );

        ### Retrieve the returned rows of data
        while ( my ( $subnet, $tmpSubCount ) = $sth_sel->fetchrow_array(  ) ) {
            prep_job ( $subnet );
        }

    } else {
        print "Invalid LanID [$SiteID], try running osFingerprint.pl --list\n\n";
    }

    $sth_sel->finish;
}

sub prep_job {
    my ( $subnet ) = @_;

    # Temporary file for Results Processing
    $subTemp = $subnet;
    print "subnet=$subTemp\n";
    $subTemp =~ s/\//_/;
    my $outfile = "${outdir}/NM_Results_$subTemp.xml";
    my $datafile = "${datadir}/NM_Results_$subTemp.xml";

    unlink $outfile if -e $outfile;            # REMOVE ANY PRE-EXISTING Results File
    print "\nProcessing ".$subnet."...";
    run_nmap ( $subnet, $outfile, 0);
    load_fingerprint_results( $SiteID, $subnet, $outfile, $dbh);

    #unlink $outfile if -e $outfile;
    #Must reside on the same filesystem and destroys destination file if exists.
    rename $outfile, $datafile;

}

sub run_nmap {
    my ( $subnet, $outfile, $mode) = @_;

    #        my $cmd = "nmap -O -P0";
    #        DEVELOP VARIOUS $OPTIONS FOR NMAP TO IMPROVE SPEED AND DETECTION.

    my $options = "";

 
    if ( !$mode ) { $mode = 0 }
    if ( $mode == 0 ) {

        $options = "-sS -sV -O -F -T5 --max_rtt_timeout 10000 --min_rtt_timeout 100 "
            . "--initial_rtt_timeout 1000 --max_parallelism 50 --min_parallelism 10 -oX ".$outfile;

    } elsif ( $mode == 1 ) {    
        $options = "-sX -PB -p 1-1024 -O -T Insane -oX ".$outfile;
    } else {
        $options = "-sA -F -O -T 5 --max-hostgroup=10 -oX ".$outfile;
    }

    my $cmd = "/usr/bin/nmap ".$options." ".$subnet;

    open(PROC,"$cmd |") or die "failed to fork :$!\n";
    my ($version, $port, $type, $status, $description,$nhost, $trash, $os);

    while(<PROC>){

        if (/Interesting/) {
            if (/(\d+\.\d+\.\d+\.\d+)/) {
                $nhost=$1;
            }
        } elsif (m!(\d+)/(\w+)\s+(\w+)\s+(\w+)\s+(.*)!g) {
            $port=$1;
            $type=$2;
            $status=$3;
            $description=$4;
            $version=$5;
        } elsif(/Running|guess/) {
            ($trash, $os) = split (/:/, $_);
            #            $os=$1;
        }
    }
    close PROC;
}

sub load_fingerprint_results {
    my ( $SiteID, $subnet, $file_xml, $dbhf ) = @_;


    # DECLARE PRIVATE VARIABLES
    my ($sth_sel, $sth_upd, $sth_inst);
    my ($value, $sql, $sel_id, $subnet_id, $cntHosts, $iAccuracy );
    my ($ip_address, $mac_address, $hostname, $os, $count ) = "";
    my ( $start_time, $finish_time ) = "";

    my $now = getMysqlTime();        #GET CURRENT TIME FORMATED FOR MYSQL

    ### CHECK THE DATABASE FOR A MATCHING SUBNET
    $subnet_id = getSubnetID( $subnet, $dbhf);


    my %results = ParseResultsFile( $file_xml );

    #UPDATE DATABASE

    foreach my $family ( sort keys %results ) {
        #FIND EXISTING RECORD - AM I ALLOWED TO UPDATE ALL FIELDS?
        #print "$family: \n\n ";

        $sel_id="";

        if ( $family =~/session/ ) {

            $sql = "";

            $start_time = $results{$family}{scan_start};
            $finish_time = $results{$family}{scan_stop};

            $iCntExists = cntSubnetHosts( $subnet, $dbhf );
            if ( $iCntExists == 0 ) { $iCntExists = $iCntScanned }

            if ($subnet_id ) {        # UPDATE RECORD

                $sql = "";
                $sql = "UPDATE I3_subnets SET ";
                $sql .= "dtLastNMAP='$finish_time', ";
                #$sql .= "cntHosts='$iCntExists', ";
                #$sql .= "cntScan='$iCntScanned', ";
                #$sql .= "scan_args='$results{$family}{scan_args}' ";
                $sql .= "WHERE id ='$subnet_id';";

            } else {                # INSERT RECORD

                #$sql = "INSERT INTO network_subnets VALUES ( '', ";
                #$sql .= "'1', '${LanID}', '$iCntExists', '$iCntScanned', ";
                #$sql .= "'$results{$family}{scan_args}', '$start_time', ";
                #$sql .= "'$finish_time', '1');";
            }
            if ( $debug || $log_level ge "1" )  {
                print "$sql\n";
            } else {
                $sth_upd = $dbhf->prepare( qq{ $sql } );
                $sth_upd->execute(  );
            }
        } else {

            my $hostip='$results{$family}{hostip}';
            ### Prepare a SQL statement for execution
            $sth_sel = $dbhf->prepare( "SELECT id FROM I3_nmap WHERE hostip='$family' LIMIT 1" );

            ### Execute the statement in the database
            $sth_sel->execute(  );

            ### Retrieve the returned rows of data
            while ( my $id = $sth_sel->fetchrow_array(  ) ) {
                #print "Found $family with id: $id\n";
                $sel_id = $id;
            }

            $sql = "";

            if ($sel_id ) {        # UPDATE RECORD

                $sql = "UPDATE I3_nmap SET subnet_id='$subnet_id', dtLastUpdated = '$now'";

                if ( $results{$family}{mac_address} ) { $sql .= ", mac_address='$results{$family}{mac_address}'"; }
                if ( $results{$family}{mac_vendor} ) { $sql .= ", mac_vendor='$results{$family}{mac_vendor}'"; }
                if ( $results{$family}{hostname} ) { $sql .= ", hostname='$results{$family}{hostname}'"; }
                if ( $results{$family}{osClass_type} ) { $sql .= ", osClass_type='$results{$family}{osClass_type}'"; }    
                if ( $results{$family}{osClass_vendor} ) { $sql .=", osClass_vendor='$results{$family}{osClass_vendor}'"; }
                if ( $results{$family}{osClass_family} ) { $sql .=", osClass_family='$results{$family}{osClass_family}'"; }
                if ( $results{$family}{osClass_osgen} ) { $sql .=", osClass_osgen='$results{$family}{osClass_osgen}'"; }
                if ( $results{$family}{osClass_accuracy} ) { $sql .=", osClass_accuracy='$results{$family}{osClass_accuracy}'"; }
                if ( $results{$family}{osMatch_name} ) { $sql .=", osMatch_name='$results{$family}{osMatch_name}'"; }
                if ( $results{$family}{osMatch_accuracy} ) { $sql .=", osMatch_accuracy='$results{$family}{osMatch_accuracy}'"; }
                if ( $results{$family}{open_tcp} ) { $sql .=", open_tcp='$results{$family}{open_tcp}'"; }
                if ( $results{$family}{open_tcp_list} ) { $sql .=", open_tcp_list='$results{$family}{open_tcp_list}'"; }
                if ( $results{$family}{open_udp} ) { $sql .=", open_udp='$results{$family}{open_udp}'"; }
                if ( $results{$family}{open_udp_list} ) { $sql .=", open_udp_list='$results{$family}{open_udp_list}'"; }
                $sql .= " WHERE id ='$sel_id';";
            } else {                # INSERT RECORD

                $sql = "INSERT INTO I3_nmap VALUES ( ";
                $sql .= "'', '$subnet_id', '$results{$family}{mac_address}', '$results{$family}{mac_vendor}',";
                $sql .= "'$results{$family}{hostip}', '$results{$family}{hostname}', '$results{$family}{osClass_type}',";
                $sql .= "'$results{$family}{osClass_vendor}', '$results{$family}{osClass_family}',";
                $sql .= "'$results{$family}{osClass_osgen}', '$results{$family}{osClass_accuracy}',";
                $sql .= "'$results{$family}{osMatch_name}', '$results{$family}{osMatch_accuracy}',";
                $sql .= "'1', '', '', '$results{$family}{open_tcp}', '$results{$family}{open_tcp_list}',";
                $sql .= "'$results{$family}{open_udp}', '$results{$family}{open_udp_list}','');";

            }
            if ( $debug || $log_level ge "1" )  {
                print "$sql\n";
            } else {
                if ( $results{$family}{osClass_accuracy} ) { 
                    $iAccuracy = $results{$family}{osClass_accuracy};
                } else {
                    $iAccuracy = 0;
                }
                if ( $iAccuracy > 50 ) {
                    $sth_upd = $dbhf->prepare(qq{ $sql } );
                    $sth_upd->execute(  );
                    #print "$sql\n";
                }
            }
        }

    } #END DATABASE UPDATE

}


sub ParseResultsFile {

    my ( $file_xml ) = @_;

    # Private Variables
    my ($value, $iAccuracy );
    my ($ip_address, $mac_address, $hostname, $os, $count ) = "";

    # Create Instance of Nmap::Parser
    my $np = new Nmap::Parser;
    $np->parsefile($file_xml);

    my $session    = $np->get_session();

    my %results = ();

    $results{session}{scan_args} = $session->scan_args;
    $results{session}{scan_start} = convertTime( $session->start_str );
    $results{session}{scan_stop} = convertTime( $session->time_str );

    $iCntScanned = 0;
    foreach my $host ( $np->all_hosts() ) {

        $ip_address    = $host->ipv4_addr;

        #INITIALIZE ALL VALUES TO NULL

        $results{$ip_address}{mac_address} = "";
        $results{$ip_address}{mac_vendor} = "";
        $results{$ip_address}{hostip} = "";
        $results{$ip_address}{hostname} = "";
        $results{$ip_address}{osClass_type} = "";
        $results{$ip_address}{osClass_vendor} = "";
        $results{$ip_address}{osClass_family} = "";
        $results{$ip_address}{osClass_osgen} = "";
        $results{$ip_address}{osClass_accuracy} = "";
        $results{$ip_address}{osMatch_name} = "";
        $results{$ip_address}{osMatch_accuracy} = "";
        $results{$ip_address}{open_tcp} = "";
        $results{$ip_address}{open_tcp_list} = "";
        $results{$ip_address}{open_udp} = "";
        $results{$ip_address}{open_udp_list} = "";

        $mac_address   = $host->mac_addr;
        if ($mac_address ) {
            $mac_address =~ s/://g;
        }
        $hostname      = $host->hostname;

        if ( $hostname eq "0" ) {
           $hostname="";
        }

        $results{$ip_address}{hostip} = $ip_address;
        if ( $mac_address ) { $results{$ip_address}{mac_address} = $mac_address; }
        if ( $host->mac_vendor ) { $results{$ip_address}{mac_vendor} = $host->mac_vendor; }
        if ( $hostname ) { $results{$ip_address}{hostname} = $hostname; }
        if ( $host->tcp_port_count ) { $results{$ip_address}{open_tcp} = $host->tcp_port_count; }
        if ( $host->udp_port_count ) { $results{$ip_address}{open_udp} = $host->udp_port_count; }

        for my $tcp_open ( $host->tcp_ports('open')) {
            $results{$ip_address}{open_tcp_list} .= $tcp_open.":";
        }

        for my $udp_open ( $host->udp_ports('open')) {
            $results{$ip_address}{open_udp_list} .= $udp_open.":";
        }

        $os = $host->os_sig;
        $count = $os->name_count();

        #CODE ATTEMPT TO VALIDATE HOSTS BY MOST ACCURATE OSCLASS_GENERATION
        my $verify_generation = 0;
        my $os_gen = "";

        if ( $count ) {

            if ( $count > 0 ) {
                if ( $os->vendor(0) =~ /microsoft/i && $os->osfamily(0) =~ /windows/i ) {
                    #print "validate generation $ip_address\n";
                    $verify_generation = 1;
                    for ( my $i=0; $i <= $count; $i++ ) {
                        $os_gen = $os->osgen($i);
                        if ( $os_gen ) {
                            $os_gen =~ s/\/.NET//;
                            last;
                        }
                    }
                }
            }


            for ( my $i=0; $i < $count; $i++ ) {

                if ( $i == 0 ) {
                    if ( $os->type($i) ) { $results{$ip_address}{osClass_type} = $os->type($i) }
                    if ( $os->vendor($i) ) { $results{$ip_address}{osClass_vendor} = $os->vendor($i) }
                    if ( $os->osfamily($i) ) { $results{$ip_address}{osClass_family} = $os->osfamily($i) }
                    if ( $os->osgen($i) ) { $results{$ip_address}{osClass_osgen} = $os->osgen($i) }
                    if ( $os->class_accuracy($i) ) { $results{$ip_address}{osClass_accuracy} = $os->class_accuracy($i) }
                    if ( $os->name($i) ) { $results{$ip_address}{osMatch_name} = $os->name($i) }
                    if ( $os->name_accuracy($i) ) { $results{$ip_address}{osMatch_accuracy} = $os->name_accuracy($i) }
                }

                if ( $verify_generation == 1 ) {
                    if (  $os->name($i) =~ /$os_gen/i ) {
                        #print "updating  ".$hostname." from ".$results{$ip_address}{osMatch_name}." to ".$os->name($i) ."\n";
                        $results{$ip_address}{osClass_type} = $os->type($i);
                        $results{$ip_address}{osMatch_name} = $os->name($i);
                        $verify_generation = 0;

                    } else {
                        #print "No Match Found for $os_gen against ".$os->name($i) ."\n";
                    }
                } else {
                    # TAKE NMAP WORD FOR IT! USE THE MOST ACCURATE RECORD 0
                    last;
                    print "we gave up\n";
                }
            }


            #ONE LAST ATTEMPT TO CLEAN UP NAMES
            $results{$ip_address}{osMatch_name} =~ s/\'//g;
        }

        if ( $results{$ip_address}{osClass_accuracy} ) { 
                $iAccuracy = $results{$ip_address}{osClass_accuracy};
        } else {
            $iAccuracy = 0;
        }
        if ( $iAccuracy > 50 ) {
            $iCntScanned++;
        }
    }

    if ($debug) {
        print "Debug code\n";
        # print the whole thing  somewhat sorted

        my $value;
        foreach my $family ( sort keys %results ) {
            print "$family: \n\n ";
            for $value ( sort keys %{ $results{$family} } ) {
                if ( $results{$family}{$value} ) {
                    print "$value = " . $results{$family}{$value} . " \n ";
                }
            }
            print "\n\n";
        }
    }

    return %results;

}


sub cntSubnetHosts {
    my ( $subnet, $dbhf ) = @_;

    my ( $sql, $sth_sel );

    my @class = split(/\.+/, $subnet);
    my $class_c = "$class[0].$class[1].$class[2]";

    #print "subnet=$subnet  class_c=$class_c\n";

    $sql = qq{ SELECT hostip FROM I3_nmap WHERE hostip LIKE '%$class_c%' };
    logwriter ( $sql, 5 );

    $sth_sel =$dbhf->prepare( $sql );
    $sth_sel->execute;

    my @ip;
    my $iHosts = 0;

    while ( my $strIPAddress =$sth_sel->fetchrow_array ) {

        #print "$strIPAddress\n";
        # Exclude null nessus_value records
        @ip = split(/\.+/, $strIPAddress);

        #print "$ip[3]\n";
        if ( $class[3] =~ /0/ && $ip[3] <= 127 ) {
            $iHosts++;
        } elsif ( $class[3] =~ /129/ && $ip[3] >= 129 ) {
            $iHosts++;
        }
    }

    return $iHosts;

}

sub getSubnetID {

    my ( $subnet, $dbhf ) = @_;

    my ($sql, $sth_sel, $sub_id);

    ### CHECK THE DATABASE FOR A MATCHING SUBNET
    $sql = qq{ SELECT id FROM I3_subnets WHERE CIDR='$subnet' LIMIT 1 };
    logwriter ( $sql, 5 );
    $sth_sel = $dbhf->prepare( $sql );
    $sth_sel->execute(  );

    ### Retrieve the returned rows of data
    while ( my ( $id ) = $sth_sel->fetchrow_array(  ) ) {
        return $id;
    }

}


sub getMysqlTime {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    return sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);
}


sub convertTime {
    my ( $strTime ) = @_;

    my @arrTime = split /\s+/, $strTime;

    #EXAMPLE: Sat Jan 13 06:17:41 2007
    my $month ="";

    for ( $arrTime[1] ) {

            if ( $arrTime[1] =~ /Jan/i ) {
                $month = "01";
           } elsif ( $arrTime[1] =~ /Feb/i ) {
                $month = "02";
           } elsif ( $arrTime[1] =~ /Mar/i ) {
                $month = "03";
           } elsif ( $arrTime[1] =~ /Apr/i ) {
                $month = "04";
           } elsif ( $arrTime[1] =~ /May/i ) {
                $month = "05";
           } elsif ( $arrTime[1] =~ /Jun/i ) {
                $month = "06";
           } elsif ( $arrTime[1] =~ /Jul/i ) {
                $month = "07";
           } elsif ( $arrTime[1] =~ /Aug/i ) {
                $month = "08";
           } elsif ( $arrTime[1] =~ /Sep/i ) {
                $month = "09";
           } elsif ( $arrTime[1] =~ /Oct/i ) {
                $month = "10";
           } elsif ( $arrTime[1] =~ /Nov/i ) {
                $month = "11";
           } elsif ( $arrTime[1] =~ /Dec/i ) {
                $month = "12";
        } else {
        }
    }
    my $mysqltime = "$arrTime[4]-$month-$arrTime[2] $arrTime[3]";
    return $mysqltime;
}


#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw{Su Mo Tu We Th Fr Sa};
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

sub read_file {
    # VER: 1.0 MODIFIED: 6/27/08 11:36
    my ( $myfile ) = @_;
    my ( $string ) = "";

    open FILE, "$myfile" or die "Couldn't open file: $!"; 
    while (<FILE>){
        $string .= $_;
    }
    close FILE;

    return $string;

}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
