#!/usr/bin/perl
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################

use strict;
use warnings;

use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $portscanlog = "$CONFIG{'ROOTDIR'}/var/log/port_scan.log";
open(LOG,">>$portscanlog") or die "Failed to create $portscanlog: $!\n";
*STDERR=*LOG;
*STDOUT=*LOG;

use DBI;
use Parallel::ForkManager;
use MIME::Lite;
use Switch;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

use constant MAX_FORKS => 5;
use constant TRUE => 1;
use constant FALSE => 0;

my $debug              = 1;
my $log_level          = 4;

my ( $dbh );

## Call main subroutine
main();

close LOG;

exit(0);

# Main subroutine
sub main {
   my ($dbh, $sql, $sth_sel, $sth_upd, $sth_upd1);
   my ($options, $id, $username, $host, $email, $ports, $portstart, $portend, $common );
   my ($pm, $outfile, $timeout, $scanq);
   my $outputdir = "$CONFIG{'ROOTDIR'}/var/lib/nmap";

   # Open a connection to the database
   $dbh = conn_db();
   load_db_configs ( );

   $sql = qq{ SELECT count(id) FROM portscan WHERE scancomplete = "R" };
   logwriter ( $sql, 5);
   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute;
   $scanq=$sth_sel->fetchrow_array;

   $scanq=$CONFIG{'MAXPORTSCANS'}-$scanq;

   if ($scanq<=0) { exit(0);}

   # Open a ForkManager handle limiting the number of children
   # to MAX_FORKS
   $pm = new Parallel::ForkManager($scanq);

   # Timeout for individual nmap scans set to 6hrs
   $timeout = 7800;

   $sql = qq{ SELECT id, username, scanip, email, ports, portstart, portend, common, options
      FROM portscan WHERE scancomplete = 'N' LIMIT 0, $scanq };
   logwriter ( $sql, 5);
   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute;

   while ( my ($id, $username, $host, $email, $ports, $portstart, $portend, $common, 
      $options ) = $sth_sel->fetchrow_array ) {

      $pm->start and next; # do the fork

      my $dbhf;

      $dbhf = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
         PrintError => 0,
         RaiseError => 1,
         AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");

      my $sql = qq{ UPDATE portscan SET scancomplete="R" WHERE id = '$id' };
      safe_db_write ( $dbhf,  $sql, 4 );

      if ( $debug ) {
         print localtime(time)." port_scan: ";
         print join("|", $username, $host, $email, $ports, $portstart, $portend, $common ),"\n";
      }

      $outfile = "${outputdir}/nmap_${id}.out";
      unlink $outfile if -e $outfile;

      send_email($CONFIG{mailfrom}, $email, $username, "Port Scan Started", "Dear $username,\n\nPort scan has started");

      if ( run_nmap($host, $ports, $portstart, $portend, $common, $outfile,$timeout, $dbhf, $id, $options) ) {

         # Send email only on a successful nmap scan
         send_email($CONFIG{mailfrom}, $email, $username, "Port scan has finished", "Dear $username,\n\nPort scan has finished sucsessufully. You can access the scan results here:\n$CONFIG{inprotect_url}/portscanres.php?disp=view&pid=$id");

         # Mark the scan as completed
         # Commented by Ajay to not update the table during test runs
      }
      else {
         send_email($CONFIG{mailfrom}, $email, $username, "Port scan failed", "Dear $username,\n\nPort scan has failed. It may be due to the portscan timeout.");
      };
      $sql = qq{ UPDATE portscan SET scancomplete = "Y" WHERE id = '$id' }; 
      safe_db_write ( $dbhf,  $sql, 4 );

      #delete output file after scan
      unlink $outfile if -e $outfile;
      $pm->finish; # do the exit in the child process
   }

   # Wait for all spawned children to die(cruel Huh?)
   $pm->wait_all_children;
   disconn_db($dbh);
}

sub run_nmap {
   my ( $host, $ports, $portstart, $portend, $common, $outfile, $timeout, $dbhf, $id, $options) = @_;

   my ( $sql );
#   my $cmd = "nmap -O -P0";
   my $cmd = "nmap -n -O".$options;
   if ( $ports eq "T" ) {
      $cmd .= " -sS";
   } elsif ( $ports eq "U" ) {
      $cmd .= " -sU";
   }
   else {
      $cmd .= " -sS -sU";
   }
   if ( $common eq "N" ) {
      $cmd .= " -p ${portstart}-${portend}";
   }

   $cmd .= " --host_timeout 7200000 $host";

   print localtime(time)." port_scan: $cmd\n" if $debug;
   eval {
      local $SIG{ALRM} = sub { die "nmap_timeout\n" };
      alarm $timeout;      # Set alarm to $timeout

      open(PROC,"$cmd |") or die "failed to fork :$!\n";
      my ($version, $port, $type, $status, $description,$nhost, $trash, $os);
      while(<PROC>){

         if (/Interesting/) {
            if (/(\d+\.\d+\.\d+\.\d+)/) {
               $nhost=$1;
            }
         }
         elsif (m!(\d+)/(\w+)\s+(\w+)\s+(\w+)\s+(.*)!g) {
            $port=$1;
            $type=$2;
            $status=$3;
            $description=$4;
            $version=$5;

            my $sql = qq{ INSERT INTO portscan_results VALUES 
               ( $id, '$nhost', $port, '$type', '$status', '$description', '$version'); };
            safe_db_write ( $dbhf,  $sql, 4 );
          }
         elsif(/Running|guess/) {
            ($trash, $os) = split (/:/, $_);
#            $os=$1;
            my $sql = qq{ INSERT INTO portscan_os (id, host, os) VALUES ($id, '$nhost', '$os'); };
            safe_db_write ( $dbhf,  $sql, 4 );
         }
      }
      close PROC;



#       system($cmd) and die "Failed to run $cmd : ", $? >> 8 , "\n";
       alarm 0;    # Reset alarm
   };
   if ($@ eq "nmap_timeout\n") {
#      print localtime(time)." port_scan: -- $@\n";
      # For nmap timeout, write an extra message to STDERR
      warn "NMAP command timed out for $host\n" if $@ eq "nmap_timeout\n";
      #my $killprog="ps -C '$cmd' | perl -pe '(\$_)=/(\\d+)/;`kill -9 \$_`'";
      my $killprog="ps -auxww | grep '$cmd' | perl -pe '(\$_)=/root {3,9}(\\d+)/;`kill -9 \$_`'";
      #system($killprog);
      #system($killprog);
      return FALSE;
   }
   else {
      return TRUE;
   }
}

sub send_email {

   my ( $fromemail_addr, $email_addr, $username, $subject, $data) = @_;
   my ( $msg, $html_data );
   $msg = MIME::Lite->new(
       To      =>"$email_addr",
       From    =>"$fromemail_addr",
       Subject =>"$subject",
       Data    =>"$data"
       );
   $msg->send();
}

#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"     { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"         { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"         { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

sub read_file {
    # VER: 1.0 MODIFIED: 6/27/08 11:36
    my ( $myfile ) = @_;
    my ( $string ) = "";

    open FILE, "$myfile" or die "Couldn't open file: $!"; 
    while (<FILE>){
   $string .= $_;
    }
    close FILE;

    return $string;

}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $dbhf, $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbhf->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbhf->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
