#!/usr/bin/perl
#
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##
##    PRGM AUTHOR: Various
##   PROGRAM NAME: updatePlugin
##   PROGRAM DATE: 04/23/2008
##  PROGM VERSION: 1.0
##  REVISION HIST:
##        04/23/2008 - FIRST VERSION OF RECODE TO ADD DEBUG AND WORK OUT TYPICAL
##                     FLAWS THAT ARE AFFECTING PROFILES CONFIGS
##      06/29/2008 - Bug fix now was created/modified/deleted being populated by $now as was not defined
use DBI;
use Getopt::Std;

use DBI;
use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $updatepluginslog = "$CONFIG{'ROOTDIR'}/var/log/updateplugins.log";
$mysqlpath = "$CONFIG{'MYSQLPATH'}";                         #PATH TO MYSQL EXECUTABLE
$tempfile = "$CONFIG{'ROOTDIR'}/tmp/plugins.sql";        #Temp Nessus plugins file

my $updateplugins="$CONFIG{'UPDATEPLUGINS'}";

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

my $debug            = 0;
my $log_level        = 4;

my ($nessushost, $nessusport, $nessususer, $nessuspassword);
my ( $serverid );

my ( $dbh );        #DATABASE HANDLE TO BE USED THROUGHOUT PROGRAM

getopts("dh?",\%options);

main( );
exit;

sub main {

    my ( $sth_sel, $sql );

    if( $options{d} ) {                #ENABLE DEBUGGING
        print "Debugging mode\n";
        $debug = 1;
    }

    open(LOG,">>$updatepluginslog") or die "Failed to create $updatepluginslog: $!\n";
        *STDERR=*LOG;
        *STDOUT=*LOG;

    # CONNECT TO DATABASE
    $dbh = conn_db();
    #load_db_configs ( );

    $serverid = get_server_credentialsA( $CONFIG{'SERVERID'} );  #GET THE SERVER ID'S FOR WORK PROCESSING
    if ($serverid == 0 ) {  #CHECK FOR VALID SERVER ID)
        logwriter( "[$$]\tWARNING: ServerID is Invalid --CAN NOT CONTINUE", 1 );
        exit;
    }
    logwriter( "host=$nessushost, port=$nessusport, user=$nessususer, pass=$nessuspassword", 5 );

    #PROCEED WITH FORCE NESSUS TO UPDATE PLUGINS
    if ($updateplugins==1) {
        logwriter( "updateplugins: executing nessus-update-plugins", 4 );
        perform_update( );
    } else {
        logwriter( "updateplugins: inprotect.cfg configured to not updateplugins", 2 );
    }

    dump_plugins();
    import_plugins( );
    update_categories( );
    update_families( );
    update_nessus_plugins( );
    update_settings_plugins( );
    update_preferences();

    disconn_db($dbh);
    #end of main
    exit;

}

sub perform_update {

    logwriter( "BEGIN - PERFORM UPDATE", 4 );
    if ( -e $CONFIG{'NESSUSUPDATEPLUGINSPATH'} ) {
       system ("$CONFIG{'NESSUSUPDATEPLUGINSPATH'}") == 0 or die "". logwriter( "updateplugins: No new plugins installed", 4 );

       #If used patch for nessus-update-plugins, we may pass parameter on the command line.
       #system ("nessus-update-plugins.141 -u $inprotect_url/nessus") == 0 or die localtime(time)." updateplugins: No new plugins installed";

       logwriter( "updateplugins: sleeping for 120sec to allow nessus to restart", 4 );

       #Sleep 300 sec, allow nessusd to restart
       sleep 120;
   } else {
      logwriter( "INVALID PATH/FILE nessus-update-plugins named \"$CONFIG{'NESSUSUPDATEPLUGINSPATH'}\"", 1);
      die( "Invalid Path to nessus-update-plugins");
   }
   logwriter( "FINISH - PERFORM UPDATE", 4 );
}

sub dump_plugins {

    logwriter( "BEGIN - DUMP PLUGINS", 4 );

    #Delete existing temporary file
    unlink $tempfile if -e $tempfile;

    #Dump Nessus plugins info into a file
    my $cmd = "$CONFIG{'NESSUSPATH'} -xpS -q $nessushost $nessusport $nessususer $nessuspassword > $tempfile";
    logwriter( "$cmd", 5 );
    my $imp = system ( $cmd );

    if ( $imp != 0 ) { die "". logwriter( "updateplugins: Failed Dump Plugins", 2 ); }

    logwriter( "FINISH - DUMP PLUGINS", 4 );
    return 1;
}

sub import_plugins {

    logwriter( "BEGIN - IMPORT PLUGINS", 4 );

    #Dump Nessus plugins info into a file
    my $cmd = "$mysqlpath --force --user=$CONFIG{'DATABASEUSER'} --password=$CONFIG{'DATABASEPASSWORD'} --host=$CONFIG{'DATABASEHOST'} $CONFIG{'DATABASENAME'} < $tempfile";
    logwriter( "$cmd", 5 );
    my $imp = system ( $cmd );

    if ( $imp != 0 ) { die "". logwriter( "updateplugins: Failed Import Plugins", 2 ); }

    #Delete existing temporary file
    unlink $tempfile if -e $tempfile;
    logwriter( "FINISH - IMPORT PLUGINS", 4 );
    return 1;

}

sub update_categories {
    my ( $sth_sel, $sth_selc, $sth_ins, $sth_insc, $sql );

    logwriter( "BEGIN - UPDATE CATEGORIES", 4 );
    #Updating family and category tables
    $sql = qq{ select distinct plugins.category from plugins left join nessus_category on 
        plugins.category = nessus_category.name where nessus_category.id is null order by category };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while (my ($categoryname) = $sth_sel->fetchrow_array ) {
        $sql = qq{ insert into nessus_category (name) values('$categoryname') };
        safe_db_write( $sql, 5 );

        $sql = qq{ select id from nessus_category where name='$categoryname' };
        logwriter( "$sql", 5 );
        $sth_selc=$dbh->prepare( $sql );
        $sth_selc->execute;
        ($catid)=$sth_selc->fetchrow_array;

        $sql = qq{ select distinct sid from nessus_settings_category };
        logwriter( "$sql", 5 );
        $sth_selc=$dbh->prepare( $sql );
        $sth_selc->execute;
        while (($setid) =$sth_selc->fetchrow_array ) {
            $sql = qq{ insert into nessus_settings_category (sid, cid, status) values ($setid, $catid, 1) };
            safe_db_write( $sql, 5 );
        }
        $sth_selc->finish();
    }
    logwriter( "FINISH - UPDATE CATEGORIES", 4 );

}

sub update_families {
    my ( $sth_sel, $sth_self, $sth_ins, $sth_insf, $sql );

    logwriter( "BEGIN - UPDATE FAMILIES", 4 );

    #Updating family and category tables
    $sql = qq{ select distinct plugins.family from plugins left join nessus_family on 
            plugins.family = nessus_family.name where nessus_family.id is null order by family };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while (my ($familyname) = $sth_sel->fetchrow_array ) {
        $sql = qq{ insert into nessus_family (name) values('$familyname') };
        safe_db_write( $sql, 5 );

        $sql = qq{ select id from nessus_family where name='$familyname' };
        logwriter( "$sql", 5 );
        $sth_self=$dbh->prepare( $sql );
        $sth_self->execute;

        ($famid)=$sth_self->fetchrow_array;

        $sql = qq{ select distinct sid from nessus_settings_family };
        logwriter( "$sql", 5 );
        $sth_self=$dbh->prepare( $sql );
        $sth_self->execute;
        while (($setid) =$sth_self->fetchrow_array ) {
            $sql = qq{ insert into nessus_settings_family (sid, fid, status) values ($setid, $famid, 1) };
            safe_db_write( $sql, 5 );
        }
        $sth_self->finish();
    }

    logwriter( "FINISH - UPDATE FAMILIES", 4 );

}

sub test {

    my ( $sth_sel, $sth_sel2, $sth_sel3, $sth_ins, $sql );

    #Mark obsolete plugins deleted
    $sql = qq{ SELECT id FROM nessus_plugins WHERE deleted is null };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while ( ($id) = $sth_sel->fetchrow_array ) {
        $sql = qq{ SELECT count(*) FROM plugins WHERE id=$id };
        logwriter( "$sql", 5 );
        $sth_sel2 = $dbh->prepare( $sql );
        $sth_sel2->execute;
        ($plugincount)= $sth_sel2->fetchrow_array;
        if ($plugincount==0) {
            $sql = qq{ UPDATE nessus_plugins set enabled='N', deleted='$now' where id=$id };
            safe_db_write( $sql, 5 );
            $sql = qq{ DELETE FROM nessus_settings_plugins WHERE id=$id };
            safe_db_write( $sql, 5 );
        }
        $sth_sel2->finish();
    }


}

sub update_nessus_plugins {

    logwriter( "BEGIN - UPDATE NESSUS_PLUGINS", 4 );

    my ( $sth_sel, $sth_sel2, $sth_sel3, $sth_ins, $sql );
    my $now = genScanTime();

    #Select all plugin ids from the database
    $sql = qq{ SELECT id FROM plugins };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ($id) = $sth_sel->fetchrow_array ) {

        $sql = qq{ SELECT count(*) FROM nessus_plugins WHERE id=$id };
        logwriter( "$sql", 5 );
        $sth_sel2 = $dbh->prepare( $sql );
        $sth_sel2->execute;
        ($plugincount)= $sth_sel2->fetchrow_array;

        $sql = qq{ SELECT t1.id, t1.name, t2.id, t3.id, t1.copyright, t1.summary, t1.description, 
            t1.version, t1.cve_id, t1.bugtraq_id 
            FROM plugins t1
            LEFT JOIN nessus_family t2 ON t1.family = t2.name
            LEFT JOIN nessus_category t3 ON t1.category = t3.name
            WHERE t1.id=$id };
        logwriter( "$sql", 5 );
        $sth_sel3 = $dbh->prepare( $sql );
        $sth_sel3->execute;

        ($pid, $pname, $pfamily, $pcategory, $pcopyright, $psummary, $pdescription, $pversion, $pcve_id, 
            $pbugtraq_id) = $sth_sel3->fetchrow_array;

        if ($plugincount==0) {
            #INSERT new plugins into db

            $risk=7;
            $risk=1 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Serious/s);
            $risk=1 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Critical/s);
            $risk=2 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*High/s);
            $risk=3 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium/s);
            $risk=4 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low/s);
            $risk=5 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium/s);
            $risk=6 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Low/s);
            $risk=7 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Info/s);
            $risk=7 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*[nN]one/s);

            $sql = qq{ INSERT into nessus_plugins VALUES ($pid, 'Y','$pversion', '$now', null, null, $pcategory, 
                $pfamily, $risk, NULL) };
            safe_db_write( $sql, 5 );

        } else {
            #Update changed plugins info in the db
            $sql = qq{ SELECT version FROM nessus_plugins WHERE id=$id };
            logwriter( "$sql", 5 );
            $sth_sel2 = $dbh->prepare( $sql );
            $sth_sel2->execute;
            ($pluginversion) = $sth_sel2->fetchrow_array;
            if ($pluginversion ne $pversion) {
                $sql = qq{ UPDATE nessus_plugins SET version='$pversion', modified='$now' WHERE id=$id };
                safe_db_write( $sql, 5 );
            }
        }

        $sth_sel2->finish();
        $sth_sel3->finish();
    }

    #Updating risk level info
    $sql = qq{ select id from nessus_plugins where risk=0 };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare ( $sql );
    $sth_sel->execute;

    while (my ($pluginid) = $sth_sel->fetchrow_array ) {
        $sql = qq{ select description from plugins where id=$pluginid };
        logwriter( "$sql", 5 );
        $sth_sel2=$dbh->prepare( $sql );
        $sth_sel2->execute;
        $pdescription=$sth_sel2->fetchrow_array;
        $risk=7;
        $risk=1 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Serious/s);
        $risk=1 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Critical/s);
        $risk=2 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*High/s);
        $risk=3 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium/s);
        $risk=4 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low/s);
        $risk=5 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium/s);
        $risk=6 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Low/s);
        $risk=7 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*Info/s);
        $risk=7 if ($pdescription =~ m/Risk [fF]actor\s*:\s*(\\n)*[nN]one/s);
        $sql = qq{ update nessus_plugins set risk=$risk where id=$pluginid };
        safe_db_write( $sql, 5 );
        $sth_sel2->finish();
    }
    $sth_sel->finish();

    #UPDATE RISK WITH CUSTOM VALUE AS NEEDED
    $sql = qq{ UPDATE nessus_plugins SET risk=custom_risk WHERE custom_risk IS NOT NULL AND custom_risk > 0 }; 
    safe_db_write( $sql, 3 );

    logwriter( "FINISH - UPDATE NESSUS_PLUGINS", 4 );

}

sub update_preferences {

    logwriter( "BEGIN - UPDATE NESSUS_PREFERENCES", 4 );

    my ( $sql, $sth_sel, $sth_upd );

    logwriter( "updateprefs: Connecting to DB and updating preferences", 4 );

    my $now = genScanTime();

    # Create a table the first time we run this program if needed
    $sql = qq{show tables like "nessus_preferences_defaults"};
    logwriter( $sql, 4 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    $foo=$sth_sel->fetchrow_array;
    if (!$foo) {
        $sql = qq{ 
CREATE TABLE `nessus_preferences_defaults` (
  `nessus_id` varchar(255) NOT NULL default '',
  `nessusgroup` varchar(255) default NULL,
  `type` varchar(255) default NULL,
  `field` varchar(255) default NULL,
  `value` varchar(255) default NULL,
  `category` varchar(255) default NULL,
  `flag` char(1) default NULL,
  PRIMARY KEY  (`nessus_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

};
        safe_db_write( $sql, 3 );
    }

    $sql = qq{ update nessus_preferences_defaults set flag=null };
    safe_db_write( $sql, 5 );

    my ($cmd);
    my ($f0, $f1, $f2, $f3, $f4, $rhs, $rhs2, $sql);

    logwriter( "updateprefs: Getting plugin preferences from nessusd", 4 );

    $cmd = qq{$CONFIG{'NESSUSPATH'} -qxP $nessushost $nessusport $nessususer $nessuspassword};
    logwriter( $cmd, 5 );
    open(PROC, "$cmd |") or die "failed to fork :$!\n";
    while (<PROC>){
        if (/\]:/) {
            # PLUGINS_PREFS
            $f5 = "PLUGINS_PREFS";
            ($f1,$rhs) = split(/\[/);
            ($f2,$rhs2) = split(/\]:/,$rhs);
            ($f3,$f4) = split(/=/, $rhs2);
             $f3 =~ s/\s+$//;    # Remove trailing whitespace 
            $f4 =~ s/^ //;        # Remove leading whitespace
            $f4 =~ s/\n$//;        # Remove trailing newline

            $f0 = $f1."[".$f2."]:".$f3;
            $f2 =~ s/entry/T/;        # Text box
            $f2 =~ s/radio/R/;        # Radio button
            $f2 =~ s/checkbox/C/;        # Checkbox
            $f2 =~ s/password/P/;        # Password
            $f2 =~ s/file/T/;        # File

        } else {
            # SERVER_PREFS
            $f5 = "SERVER_PREFS";

            $f1 = "ServerPrefs";
            ($f3,$f4) = split(/=/);
            $f3 =~ s/\s+$//;    # Remove trailing whitespace
            $f4 =~ s/\n$//;        # Remove trailing newline
            $f4 =~ s/^ //;        # Remove leading whitespace
            $f2 = "T";
            $f0 = $f3;
        }

        # Does the current record exist? If not
        $sql = qq{ SELECT count(*) from nessus_preferences_defaults WHERE nessus_id = "$f0" };
        logwriter( $cmd, 5 );
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        $foo=$sth_sel->fetchrow_array;
        if ($foo == 0) {
            $sql = qq{insert into nessus_preferences_defaults (nessus_id, nessusgroup, type, field, 
                value, category,flag) values ("$f0", "$f1", "$f2", "$f3", "$f4", "$f5","T" );};
        } else {
            $sql = qq{UPDATE nessus_preferences_defaults SET nessusgroup="$f1", type="$f2", field="$f3",
                value="$f4", category="$f5", flag="T" WHERE nessus_id = "$f0" };
        }
        safe_db_write( $sql, 5 );

    }
    $sql = "UPDATE nessus_preferences_defaults set type = 'C' WHERE nessusgroup = 'ServerPrefs' and value in ('yes', 'no')";
    safe_db_write( $sql, 5 );

    $sql = "DELETE FROM nessus_preferences_defaults where flag is null";
    safe_db_write( $sql, 5 );

    logwriter( "FINISH - UPDATE NESSUS_PREFERENCES", 4 );

}

sub update_settings_plugins {

    logwriter( "BEGIN - UPDATE SETTINGS_PLUGINS", 4 );

    my ( $sth_sel, $sth_sel2, $sth_sel3, $sth_ins, $sql );
    my $now = genScanTime();

    #CREATE ARRAY OF AUTOENABLE PER PROFILES
    my %autoenable;
    my %autofam;
    my %autocat;
    my $profile_count = 0;

    $sql = qq{ SELECT id, autoenable FROM nessus_settings };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while ( my ($sid, $vaule)=$sth_sel->fetchrow_array) { 
        $autoenable->{$sid} = $value;
        $profile_count = $profile_count + 1;
    }

    #CREATE AUTOENABLE CATEGORY ARRAY
    $sql = qq{ select sid, cid, status from nessus_settings_category };
    logwriter( "$sql", 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;
    while ( my ($sid, $cid, $status) = $sth_sel->fetchrow_array ) {
       $autocat->{$sid}->{$cid} = $status;
    }

    #CREATE AUTOENABLE FAMILY ARRAY
    $sql = qq{ select sid, fid, status from nessus_settings_family };
    logwriter( "$sql", 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;
    while ( my ($sid, $fid, $status) = $sth_sel->fetchrow_array ) {
       $autofam->{$sid}->{$fid} = $status;
    }

    #ENSURE NESSUS_SETTINGS_PLUGINS VALUES EXISTS ( CHECK THE FRONT END CODE FOR FAILURES )
    $sql = qq{ SELECT id, enabled, category, family FROM nessus_plugins WHERE deleted IS NULL };
    logwriter( "$sql", 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ($pid, $enabled, $pcategory, $pfamily ) = $sth_sel->fetchrow_array ) {
        $sql = qq{ SELECT count(id) FROM nessus_settings_plugins WHERE id='$pid' };
        logwriter( "$sql", 5 );
        $sth_sel2 = $dbh->prepare( $sql );
        $sth_sel2->execute;
        ($plugincount)= $sth_sel2->fetchrow_array;
        if ( $plugincount < $profile_count ) {
            $sql_insert = qq{ INSERT INTO nessus_settings_plugins (id, sid, enabled, category, family) VALUES };
            foreach my $sid (sort(keys(%{$autoenable}))) {
                $sql = qq{ SELECT id FROM nessus_settings_plugins WHERE id='$pid' AND sid='$sid' };
                $sth_sel2 = $dbh->prepare( $sql );
                $sth_sel2->execute;
                my ($settingcount)= $sth_sel2->fetchrow_array;
                print "settingscount=$settingcount\n";
                if ( $settingcount < 1 ) {
                    $sql_insert .= qq{ ('$pid', '$sid', 'N', '$pcategory', '$pfamily' ),};
                }
            }
            chop $sql_insert;
            $sql_insert .= ";";
            safe_db_write( $sql_insert, 3 );
        }
    }

    #IMPROVED TO TRIGGER ENABLE BY FAMILY / CATEGORY ( IGNORE INTELLIGENT OR NO AUTOENABLE )
    foreach my $sid (sort(keys(%{$autoenable}))) {
        print "sid=$sid\n";

        if ( $autoenable->{$sid} eq "C" ) {
            foreach my $cid (sort(keys(%{$autocat->{$sid}}))) {
                my $status = 1;
                my $statusvalue = "Y";
                $status = $autocat->{$sid}->{$cid};
                if ($status ==3 or $status==4) {
                    $statusvalue="N";
                } #INTELLIGENT IGNORE DEFAULT Y
                $sql = qq{ UPDATE nessus_settings_plugins SET enabled='$statusvalue' 
                    WHERE id='$pid' AND sid='$sid' AND category='$cid' };
                safe_db_write( $sql, 3 );
            }
        } elsif ( $autoenable->{$sid} eq "F"   )  {
            foreach my $fid (sort(keys(%{$autofam->{$sid}}))) {
                my $status = 1;
                my $statusvalue = "Y";
                $status = $autofam->{$sid}->{$fid};
                if ($status ==3 or $status==4) {
                    $statusvalue="N";
                }
                $sql = qq{ UPDATE nessus_settings_plugins SET enabled='$statusvalue' 
                    WHERE id='$pid' AND sid='$sid' AND family='$fid' };
                safe_db_write( $sql, 3 );
            } #INTELLIGENT IGNORE DEFAULT Y
        } #ELSE { NO AUTOENABLE }
    }
    logwriter( "FINISH - UPDATE SETTINGS_PLUGINS", 4 );
}



sub get_server_credentialsA {
    # VER: 1.1 MODIFIED: 4/23/08 12:33
    my ( $select_id ) = @_;
  
    my ($sql, $sth_sel, $tmpserverid);

    $sql = qq{ SELECT id, hostname, port, user, password FROM nessus_servers WHERE id=$select_id };
    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    ($tmp_serverid, $nessushost, $nessusport, $nessususer, $nessuspassword)=$sth_sel->fetchrow_array;
    $sth_sel->finish;
    return $tmp_serverid;

}

sub genScanTime {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
}

sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );

    eval {
        $dbh->do( $sql_insert );
    };
    warn "FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }

}

sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;

    logwriter( $sql, $specified_level );

    my ( $sth_sel );

    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;

    return @data;

}

sub trim {
    my $string = @_;
    $string =~ s/^\s+//;
    $string =~ s/\s+$//;
    return $string;
}


sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };

    warn "FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }

    return 1;

}

sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
