#!/usr/bin/perl -w
#
###############################################################################
#                          Last update: 07.5.2008                             #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##
##    PRGM AUTHOR: Kenneth Kline
##   PROGRAM NAME: import_auditsettings
##   PROGRAM DATE: 06/24/2008
##  PROGM VERSION: 0.1
##  REVISION HIST:
##        06/24/2008 - FIRST VERSION OF AUDIT IMPORTS
use strict;

#BEGIN { push( @INC, "/var/www/Inprotect/usr/share/Inprotect/perl" ); };

#use MIME::Lite::HTML;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use DBI;
use Switch;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
#use Net::Nessus::ScanLite;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/update_audit_table.log";

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

my $debug           = 0;
my $log_level       = 3;
#use constant MAX_FORKS => 20;


#Declare Public Variables to use
my $outdir = "$CONFIG{'ROOTDIR'}/tmp";

my ( $dbh );        #DATABASE HANDLE TO BE USED THROUGHOUT PROGRAM

main( );
exit;


sub main {

    my ($sth_sel, $sth_upd, $sth_upd1);
    my ($host_count, $sql, $sel_id );


    open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
        *STDERR=*LOG;
        *STDOUT=*LOG;

    # CONNECT TO DATABASE
    $dbh = conn_db();

    my @checks = ("unixAuditDir", "winAuditDir", "winFileAudits" );


    foreach my $check ( @checks )  {
	$sql = qq{ SELECT settingValue FROM inprotect_settings WHERE settingName='$check' LIMIT 1 };
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );
        my ( $check_dir ) = $sth_sel->fetchrow_array(  );
	$sth_sel->finish;

	my @files = <$check_dir/*.audit>; 
	foreach my $file (@files) {
	    print $file . "\n";
	    process_audit_file( $file, $check_dir, $check );
	} 
    }

    disconn_db($dbh);
    #end of main

}

sub process_audit_file {
    my ( $audit_file, $check_dir, $check ) = @_;

    my ( $sql, $sth_sel );

    open(DAT, $audit_file) || die("Could not open file!");
	my @raw_data=<DAT>;
    close(DAT);

    $audit_file =~ s/$check_dir\///g;

    $sql = qq{ SELECT id FROM nessus_audits WHERE name='$audit_file' LIMIT 1 };
    logwriter( $sql, 5 );
    ### LOOKUP NESSUS CHECK BY NAME
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    my $match_id = $sth_sel->fetchrow_array(  );

    if ( ! $match_id ) {
	$sql = qq{ INSERT INTO nessus_audits ( name, description, check_type, type, owner
	    ) VALUES (  '$audit_file', "Imported by Backend Script", "$check", "P", "Admin" ); };
	safe_db_write ( $sql, 4);

    }


    my $now = genScanTime();

    my $isRecord = FALSE;

    my ( $temp_name, $temp_value ) = "";

    foreach my $line (@raw_data) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
	
	my ( $name, $value )=split(/:/,$line,2);
        $name =~ s/\s+$//;
        $name =~ s/^\s+//;

	if ( defined( $value ) && $value ne "" ) {
	    $value =~ s/\"//g;
	    $value =~ s/\\/\\\\/g;
	    $value =~ s/\n//g;
            $value =~ s/\s+$//;
            $value =~ s/^\s+//;
	}
	
	if ( $name =~ /<item>/ || $name =~ /<custom_item>/  ) {
	    $isRecord = TRUE;
	} elsif ( $name =~ /<\/item>/ ) {
	    $isRecord = FALSE;

    	    print "checking setting $temp_value\n";

	    $sql = qq{ SELECT id FROM nessus_checks WHERE name='$temp_name' LIMIT 1 };
            logwriter( $sql, 5 );
            ### LOOKUP NESSUS CHECK BY NAME
            $sth_sel = $dbh->prepare( $sql );
       	    $sth_sel->execute(  );
            my $match_id = $sth_sel->fetchrow_array(  );

	    if ( ! $match_id ) {
	        my $sql2 = qq{ INSERT INTO nessus_checks ( enabled, name, created
		    ) VALUES (  'Y', '$temp_name', '$now' ); };
		safe_db_write ( $sql2, 4);
		### LOOKUP NESSUS CHECK BY NAME AFTER INSERT
		$sth_sel = $dbh->prepare( $sql );
		$sth_sel->execute(  );
        	$match_id = $sth_sel->fetchrow_array(  );
	    }
	    
	    if ( ! $match_id ) {
		logwriter( "FAILED LOOKUP check name \"$temp_name\" AFTER AN INSERT", 1 );
		exit;
	    }

	    $sql = qq{ SELECT id FROM nessus_checks_audits WHERE cid='$match_id' AND auditfile='$audit_file' LIMIT 1 };
            logwriter( $sql, 5 );
            $sth_sel = $dbh->prepare( $sql );
       	    $sth_sel->execute(  );
            ### Retrieve the returned rows of data
            my $entry_id = $sth_sel->fetchrow_array(  );

	    if ( ! $entry_id ) {
	        $sql = qq{ INSERT INTO nessus_checks_audits ( cid, enabled, created, value, auditfile
		    ) VALUES (  '$match_id', 'Y', '$now', '$temp_value','$audit_file' ); };
		safe_db_write ( $sql, 4);

	    }

	    $temp_name = "";
	    $temp_value = "";

	} elsif ( $isRecord && ( $name =~ /^name$/ || $name =~ /description/ ) ) {
	    $temp_name =  $value;
	} elsif ( $isRecord && ( $name =~ /^value$/ || $name =~ /value_data/ ) ) {
	    $temp_value= $value;
	}
    }


}

sub genScanTime {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
