#!/usr/bin/perl -w
#
###############################################################################
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                      #
###############################################################################
##
##    PRGM AUTHOR: Kenneth Kline \ Clutch-INC
##   PROGRAM NAME: nessusCron TOOL
##   PROGRAM DATE: 01/13/2007
##  PROGM VERSION: 0.99
##  REVISION HIST:
##        02/14/2007 - FIRST VERSION OF NESSUS SUBNET SCANNER
##        02/15/2007 - ADD CRON JOB CONTROL
##        04/09/2005 - PRE 04/09 - CHANGES FOR STAT REPORTING / EMAIL / LOGGING
##        04/11/2007 - 0.5 Prep for rescanning network
##        05/30/2007 - 0.6 Add Host List Scanning
##        06/01/2007 - Scheduler Integration
##        06/04/2007 - Fix Major bug for duplicate Inserts
##        06/18/2007 - Added option to queue all subnets approved for scanning
##        07/03/2007 - Fix Cleanup of Current Scans for Scheduled Scan ( always subtracting 5 )
##        07/18/2007 - ADDED DNS Resolutions by NAMESERVERS
##        07/27/2007 - Additional DNS Resolution CHECK Fails back to nessus plugin test
##        08/02/2007 - 0.9 beta3 Net::Netmask used to split CIDR to hostlist to ensure exluding exceptions
##        08/31/2007 - 0.92 alpha - Implementing NNOC management of Subnets. I3_subnets < NNOC DB
##        10/14/2007 - 0.93 beta - Reschedule Failed Scans Fix due to SSL error connect to nessusd node
##        12/05/2007 - 0.94 beta - Coded Scan Request and restric scan hours.
##        12/27/2007 - 0.94 beta2 - Added Day restriction.
##        01/14/2008 - 0.95    Recode of the backend ( around a new jobs table to handle all job submissions).
##        02/05/2008 - 0.99   First Build to use a single routine to schedual all jobs while obeying
##                            priority, server selection, assigned sites ( nessus scanners ), 
##                            and job type priority
##        02/15/2008 - 0.99B1 Added code to track progress out to nessus_PID.work for job monitoring.
##        02/20/2008 - 0.99B2 Fixed some issues with email, and usage of timestamps
##        04/01/2008 - 0.99B2R1 Loading DB Results Additionally to Config Hash.  Allows storing more config in DB
##        04/05/2008 - 0.99a_alpha Adding Options to scan via Net::Nessus::Scanlite
##        04/21/2008 - 0.99a_B1 Improve Logging Levels and Some Minor Bug Fixes
##        04/30/2008 - 1.00   Recode HostHash routines, added MAC address, full debug. Declared 1.0
##        05/01/2008 - 1.01   Slick Change Track FeedType/PluginDate on Servers Table (Great to confirm current feeds)
##        05/02/2008 - 1.05   Cleanup old methods and document routines purpose
##	  05/07/2008 - 1.10   Recoded Load Results to properly identify hostip/hostname, scanlite returns only one
##                            Signif improvement ip/name resolution w/ (hosts/dns/namerserver pool fallback)
##                            Other Improvements around host tracker for host based statistics

use strict;
use warnings;

BEGIN { push @INC, "/var/www/Eprotect/usr/share/perl"; }

use Crypt::CBC;
use MIME::Base64;
use Data::Dumper::Simple;
use Net::IP;
use Net::Nslookup;
use Net::Netmask;
use MIME::Lite;
#use MIME::Lite::HTML;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use DBI;
use Getopt::Std;
use Switch;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
use Net::Nessus::ScanLite;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

&load_configs("/var/www/Eprotect/etc/inprotect.cfg");

#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/nessus_cron2.log";

my $debug           = 0;
my $log_level       = 4;
my $track_progress  = 0;
my $time_to_die     = 0;
my $use_scanlite    = 0;
my $notify_by_email = 1;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
#use constant MAX_FORKS => 20;


#Declare Public Variables to use
my $outdir = "$CONFIG{'ROOTDIR'}/tmp";
my $messages_dir = "$CONFIG{'ROOTDIR'}/usr/share/Eprotect/email";

my $dbk = "$CONFIG{'DBK'}";

my $outfile = "";
my ($nessushost, $nessusport, $nessususer, $nessuspassword);
my ( $serverid );
my $cred_name = "";
my $no_results = FALSE;
my $exclude_hosts = "";
my @nessus_plugins;


my ( $dbh );        #DATABASE HANDLE TO BE USED THROUGHOUT PROGRAM

# READ ARGUMENTS FROM COMMAND-LINE
my %options=();
getopts("cdh:i:j:k:o:qr:st:h?",\%options);

main( );
exit;

sub main {
    my ( $work, $outfile, $targets, $qlan, $qtime );

    if( $options{d} ) {                #ENABLE DEBUGGING
        print "Debugging mode\n";
        $debug = 1;
    }

    if( $options{s} ) {         #ENABLE DEBUGGING
        print "SCANLITE mode\n";

        $use_scanlite = 1;

        open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
            *STDERR=*LOG;
            *STDOUT=*LOG;

        #GENERATE RANDOM SEED
        srand(time() ^($$ + ($$ <<15))) ;

        # CONNECT TO DATABASE
        $dbh = conn_db();

        check_connect();

        check_schedule();

        #USE Front End Variables
        load_db_configs ( );

        #PROCEED WITH JOB SELECTION
        select_job();

        disconn_db($dbh);
        #end of main
        exit;
    } elsif( $options{q} && ( $options{o} || $options{t} ) ) {        #QUEUE SUBNETS BY ORG OR DT LAST SCANNED
        $qlan = "";
        $qtime = "";
        if ( is_number( $options{o} ) ) {
            $qlan = $options{o};
        }

        if ( $options{t} =~ /now/ ) {
            $qtime = getCurrentDateTime("datetime");
        } elsif ( is_number( $options{t}) ) {
            $qtime = $options{t}
        }

        if ( $qlan ne "" || $qtime ne "" ) {        
            # CONNECT TO DATABASE
            $dbh = conn_db();

            queueWork ( $qlan, $qtime );

            disconn_db($dbh);

        } else {
            print "Invalid queue argument $options{t} for nessusCron.pl\n\n";
            exit;
        }

    } elsif( $options{r} ) {            #ISSUE AN INPROTECT RESET
        if ( $options{r} =~ /yes/ ) {
            $dbh = conn_db();

            # CHECK CONNECT
            resetBackend ();

            disconn_db($dbh);
        } else {
            print "Inprotect Reset:: usage nessusCron.pl -r yes\n\n";
            exit;
        }

    } elsif( $options{i} && $options{h} ) {        #IMPORT A REPORT
        #IMPORT REPORT
        $outfile = $options{i};
        $targets = $options{h};

        my $job_type = "C";
        my $job_name = "";
        if ( $options{j} ) {
            if ( $options{k} eq "") {
                print "" . localtime(time)." - no job name specified\n";
            } else {
                $job_name = $options{k};
            }
            $job_type = uc( $options{j} );
        }

        if ( $outfile eq "") {
            print "" . localtime(time)." - no file specified to import\n";
            exit;
        } elsif ( ! -r $outfile ) {
            print "" . localtime(time)." - load_results: Failed to read $outfile\n";
            return 0;
        }
        if ( $targets eq "" ) {
            print "" . localtime(time)." - subnet name was not specified\n";
            exit;
        }

        # CONNECT TO DATABASE
        $dbh = conn_db();

        $targets  =~ s/'//g;
        $targets  =~ s/"//g;

        print "SUBNET=[$targets]\nFILE.OUT=[$outfile]\n";
        manually_import_report ( $outfile, $targets, $job_type, $job_name );

        disconn_db($dbh);

    } else {        #DISPLAY USAGE
        print "USAGE: nessusCron.pl [-dh:i:j:k:o:qr:st:h?].\n\n";
        print "\tnessusCron.pl -d \t - Process Work w/ Additional Logging\n";
        print "\tnessusCron.pl -s \t - Process Work in Queue via ScanLite.\n";
        print "\tnessusCron.pl -h [HOSTS] -i [FILE.out] -j [JOBTYPE] -k [JOB NAME ]\t - Import Report\n";
        print "\tnessusCron.pl -r yes \t - Reset Kill all jobs\n";
        print "\tnessusCron.pl -q -o [ORG] -t TIME:[now|20070101000000]\n";
        print "\t\tEx: nessusCron.pl -q -o Finance -t now \n";
        print "\tnessusCron.pl -h \t - Get This Help\n";
        print "\n\n\tNessus Cron is the backend interface to managing scans. nessus cron will provide a number of"
            ." inprotect functions.\n\tBy default, this should be implemented in scanlite in the following format:\n\t"
            ."*/1 * * * * /usr/bin/perl /var/www/Eprotect/sbin/nessusCron_1.00.pl -s > /dev/null 2>&1\n\n"
            ."\tAdditional Tasks include:\n\t\tQueueing all or individual subnet zones based on dt last scanned\n"
            ."\t\tInprotect Reset\n\t\tImport Reports (Typically due to exceeding long scans \ failed jobs \ etc )\n\n";

        exit;
    }

    exit;        #PROGRAM NORMAL END
}


# returns an array of the plugins enabled for this scan id
sub get_plugins {
    my ($sid ) = @_;
    my (@plugins);

    my ($sth_sel, $sth_sel2, $sql, $nessus_id, $nessus_value);

    #LOOKUP AUTOENABLE
    $sql = qq{ SELECT autoenable FROM nessus_settings WHERE id=$sid };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    my ($autoenable)=$sth_sel->fetchrow_array;

    #SCANNER SET
    $sql = qq{ SELECT t1.id, t1.enabled from nessus_settings_plugins t1
        LEFT JOIN plugins t2 ON t1.id=t2.id
        where t2.category ='scanner' and t1.sid=$sid order by id };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
        if ($nessus_value eq "N") {
        } else {
            push(@plugins,"$nessus_id");
        }
    }

    if ($autoenable eq "C") {
        $sql = qq{ SELECT t1.cid, t1.status FROM nessus_settings_category t1
            LEFT JOIN nessus_category t2 ON t2.id=t1.cid
         WHERE t2.name<>"scanner" AND t1.sid=$sid ORDER BY t2.id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while ( my ($catid, $catstatus)=$sth_sel->fetchrow_array ) {
            $sql = qq{ SELECT t1.id, t1.enabled FROM nessus_settings_plugins t1 
                LEFT JOIN nessus_plugins t2 ON t1.id=t2.id
               WHERE t1.sid=$sid AND t2.category =$catid ORDER BY t1.id };
            $sth_sel2=$dbh->prepare( $sql );
            $sth_sel2->execute;

            while (($nessus_id, $nessus_value)=$sth_sel2->fetchrow_array ) {
                if ($nessus_value eq "N") {
                } else {
                    push(@plugins,$nessus_id);
                }
            } #end loop
        } #end if
    } elsif ($autoenable eq "F") {

        $sql = qq{ SELECT t1.fid, t1.status FROM nessus_settings_family t1
            LEFT JOIN nessus_family t2 ON t2.id=t1.fid
         WHERE t2.name<>"Port scanners" AND t1.sid=$sid ORDER BY t2.id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while ( my ($famid, $famstatus)=$sth_sel->fetchrow_array ) {

            $sql = qq{ SELECT t1.id, t1.enabled FROM nessus_settings_plugins t1
                LEFT JOIN nessus_plugins t2 ON t1.id=t2.id
               WHERE t2.family =$famid AND t1.sid=$sid ORDER BY id };
            $sth_sel2=$dbh->prepare( $sql );
            $sth_sel2->execute;

            while (($nessus_id, $nessus_value)=$sth_sel2->fetchrow_array ) {
               if ($nessus_value eq "N") {
               } else {
                  push(@plugins,$nessus_id);
               }
            } #end loop
         } #end if

    } else {
        $sql = qq{ SELECT t1.id, t1.enabled from nessus_settings_plugins t1
            LEFT JOIN plugins t2 on t1.id=t2.id
            WHERE t2.category <>'scanner' AND t1.sid=$sid ORDER BY id };

        $sth_sel=$dbh->prepare( $sql );
        $sth_sel->execute;

        while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
            if ($nessus_value eq "N") {
            } else {
                push(@plugins,$nessus_id);
            }
        }
    }
    $sth_sel->finish;
    return(@plugins);
}

# get the prefs, both server and plugin
sub get_prefs {
    my ( $sid, $job_id ) = @_;

    my ($sth_sel, $sql, $nessus_id, $nessus_value);

    my $prefs = {};

    $sql = qq{ SELECT nessus_id, value FROM nessus_settings_preferences
        WHERE category IS NULL AND sid=$sid };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
        # Exclude null nessus_value records
        if ($nessus_value) {
            $prefs->{$nessus_id} = $nessus_value;
        }
    }

   # get SERVER_PREFS
    $sql = qq{ SELECT nessus_id, value FROM nessus_settings_preferences
        WHERE category='SERVER_PREFS' AND sid=$sid };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
      $prefs->{$nessus_id} = $nessus_value;
   }


   #now get the plugin preferences

   ## modified 2/17/07 - hsh - enable storage of sensitive passwords
   ## in an encrypted form (Blowfish)

   ## initialize the encryption routines

   my $cipher = Crypt::CBC->new( {
        'key'                => $dbk,
        'cipher'        => 'Blowfish',
        'iv'                => substr($dbk,12,8),
        'regenerate_key'=> 0,
        'padding'        => 'null',
        'prepend_iv'    => 0 });


    $sql = qq{ SELECT nessus_id, value FROM nessus_settings_preferences
        WHERE category='PLUGINS_PREFS' AND sid=$sid };

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
      ## check for encrypted values and decrypt accordingly
      if($nessus_value =~ /^ENC\{(.*)\}/) {
         $nessus_value =~ s/^ENC\{//;
         chop($nessus_value );
         my $ciphertext = decode_base64($nessus_value);
         $nessus_value = $cipher->decrypt($ciphertext);

      }
      $prefs->{$nessus_id} = $nessus_value;
   }

    $sql = qq{ SELECT meth_CRED from I3_jobs WHERE id= '$job_id' LIMIT 1 };

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

   my ( $cred_id ) = $sth_sel->fetchrow_array(  );

    if ( $cred_id ) {
        $sql = qq{ SELECT smbaccount, smbpassword, smbdomain, password_type, TYPE 
            FROM I3_credentials WHERE id= '$cred_id' LIMIT 1 };
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );
        my ( $smbacc, $smbpass, $smbdomain, $passType, $storMeth ) = $sth_sel->fetchrow_array(  );

        if ( defined ( $storMeth ) && $storMeth eq "O" ) {
            $sql = qq{ DELETE FROM I3_credentials WHERE id='$cred_id' LIMIT 1 };
            safe_db_write ( $sql, 4 );            #REMOVE Pass
        }

        if ( defined ( $smbacc ) ) {
            $cred_name = $smbacc;        #UPDATE GLOBAL VARIABLE FOR USE IN LOAD_RESULTS
            my $cipher = Crypt::CBC->new( {
                'key'            => $dbk,
                'cipher'            => 'Blowfish',
                'iv'            => substr($dbk,12,8),
                'regenerate_key'=> 0,
                'padding'            => 'null',
                'prepend_iv'    => 0 });

            if($smbpass =~ /^ENC\{(.*)\}/) {
               $smbpass =~ s/^ENC\{//;
               chop($smbpass);
               my $ciphertext = decode_base64($smbpass);
               $smbpass = $cipher->decrypt($ciphertext);
            }

            $prefs->{'Login configurations[entry]:SMB account :'} = $smbacc;
            $prefs->{'Login configurations[password]:SMB password :'} = $smbpass;
            $prefs->{'Login configurations[entry]:SMB domain (optional) :'} = $smbdomain;
            $prefs->{'Login configurations[radio]:SMB password type :'} = $passType;
        }
    }

   $sth_sel->finish;
   return($prefs);
}

# get the job schedule table to setup reoccuring jobs
sub check_schedule {

    my ( $sql, $sth_sel );

    my ( $now );

    $now = getCurrentDateTime("datetime");

    $sql = qq{ SELECT id, name, username, schedule_type, day_of_week, day_of_month, time, email,
        meth_TARGET, meth_CRED, meth_VSET, meth_TIMEOUT
        FROM I3_job_schedule WHERE enabled = '1' and next_check <= '$now' };

    logwriter( $sql, 5 );

    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ($jid, $name, $username, $schedule_type, $day_of_week, $day_of_month, $time_run, $email,
        $meth_TARGET, $meth_CRED, $meth_VSET, $meth_TIMEOUT ) = $sth_sel->fetchrow_array ) {

        $time_run =~ s/://g;

        my $nextscan = gen_sched_next_scan ( $jid, $schedule_type, $time_run, $day_of_week, $day_of_month );

        $sql = qq{ INSERT INTO I3_jobs ( name, username, job_TYPE, meth_TARGET, meth_CRED, meth_VSET,
            meth_TIMEOUT, notify, tracker_id, authorized, author_uname ) VALUES (
            '$name', '$username', 'S', '$meth_TARGET', '$meth_CRED', '$meth_VSET', '$meth_TIMEOUT', 
            '$email', '$jid', '1', 'SCHEDULER' )  };
        safe_db_write ( $sql, 4 );            #use insert/update routine
    }


}

#setup next scan field based on job schedule input
sub gen_sched_next_scan {
    my ( $schedid, $schedule_type, $stime, $day_of_week, $day_of_month ) = @_;

    my ( $sql, $sth_upd );
    my ($timenow,$wday,$year, $mon,$mday,$now );
    my ($dy, $dm, $dd);
    $dy=0;
    $dm=0;
    $dd=0;


    #Getting system time
    my ($sec,$min,$hour,$yday,$isdst);
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    #my $now;
    #my $timenow;
    $mon++;
    $now = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    $timenow = sprintf("%02d%02d%02d", $hour, $min, $sec);


    if ($schedule_type eq "N") {

    } elsif ($schedule_type eq "D") {
        if ($stime<$timenow) {
                #Update next scan datetime
            $dd=1;
        };
    } elsif ($schedule_type eq "W") {
        if ($day_of_week<$wday) {
            $dd=7-$wday+$day_of_week;
        } elsif ($day_of_week==$wday && $stime<$timenow) {
            $dd=7;
        } elsif ($day_of_week>$wday) {
            $dd=$day_of_week-$wday;
        };
    } elsif ($schedule_type eq "M") {
        #Getting max numbers of days in the month
        my ($daysm, $mydom);
        $mydom=$day_of_month;
        $daysm=Days_in_Month($year, $mon);
        my ($nyear, $nmon);
        if ($mon<12) {
            $nmon=$mon+1;
            $nyear=$year;
        } else {
            $nmon=1;
            $nyear=$year+1;
        };

        my ($daysnm);
        $daysnm=Days_in_Month($nyear, $nmon);
        if ($daysm < $day_of_month) {
            $mydom=$daysm;
        };
        if ($mydom < $mday) {
            $dd=$daysm-$mday;
            if ($day_of_month gt $daysnm) {
                $dd=$dd+$daysnm;
            } else {
                $dd=$dd+$day_of_month;
            };
        } elsif ($mydom == $mday && $stime lt $timenow){
            $dd=$daysm-$mday;
            if ($day_of_month gt $daysnm) {
                $dd=$dd+$daysnm;
            } else {
                $dd=$dd+$day_of_month;
            };
        } elsif ($mydom > $mday) {
            $dd=$mydom-$mday;
        };
    };
    my $nextscan;

    if ($schedule_type eq "N") {
    $nextscan = "$now";
    } else {
    my ($syear, $smonth, $sday)=Add_Delta_YMD($year,$mon,$mday,$dy,$dm,$dd);
        if ($smonth<10) {
            $smonth="0".$smonth;
        };

        if ($sday<10) {
            $sday="0".$sday;
        };

        $nextscan = "$syear$smonth$sday$stime";
    }

    logwriter( "\tnextscan=$nextscan", 4 );

    $sql = qq{ UPDATE I3_job_schedule SET next_CHECK='$nextscan' WHERE id='$schedid' };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    return $nextscan;
}

#check job queue and select job based on criteria if free scan slots
sub select_job {

    my ( $sql, $sth_sel );


    my $now= getCurrentDateTime("datetime");
    my $curTime = getCurrentDateTime("time");
    my $today = getCurrentDateTime("today");

    # get the list of all scanners
    #Select server attributes
    $sql = qq{ SELECT id, hostname, TYPE, site_code FROM nessus_servers 
        WHERE enabled='1' AND status='A' AND ( max_scans - current_scans > 5 )
        ORDER BY ( max_scans - current_scans ) DESC };
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my $nessus_servers = $sth_sel->fetchall_hashref('id');
    $sth_sel->finish;


    foreach my $serverid (sort(keys(%{$nessus_servers}))) {
        my ( $serv_name, $serv_type, $serv_code );  

        #Get server attributes
        $serv_name = $nessus_servers->{$serverid}->{'hostname'};
        $serv_type = $nessus_servers->{$serverid}->{'TYPE'};
        $serv_code = $nessus_servers->{$serverid}->{'site_code'};

        my $sql_filter = "AND ( t1.scan_ASSIGNED='$serverid' OR t1.scan_ASSIGNED IS Null ) ";

        # BUILD QUERY STRING FOR RESTRICTIONS

        if ( $serv_type eq "P" || $serv_code ne "" ) {
            $sql_filter .= "AND t2.site_code='$serv_code' ";
        }

        # CHECK FOR ASSIGNED JOBS TO SERVER / ZONE

        $sql = qq{  SELECT t1.id, t1.name, t1.job_TYPE, t1.meth_TARGET, t1.scan_ASSIGNED, 
            t1.scan_PRIORITY, t2.site_code, failed_attempts
            FROM I3_jobs t1
            LEFT JOIN I3_subnets t2 ON t1.meth_TARGET = t2.CIDR
            WHERE t1.status = 'S' $sql_filter
                    AND ( t2.restrict_start IS NULL OR t2.restrict_start < '$curTime')
                AND ( t2.restrict_cutoff IS NULL OR t2.restrict_cutoff > '$curTime')
                AND ( t2.restrict_day IS NULL OR t2.restrict_day = '$today' )
                AND ( t1.scan_NEXT IS NULL OR t1.scan_NEXT <= '$now' )
            ORDER BY t1.scan_PRIORITY DESC, t1.job_TYPE DESC, t1.scan_ASSIGNED DESC LIMIT 1};

        # SORTED BY PRIORITY->JOBTYPE->ASSIGNED
        # PRIORITY 1 FIRST
        # 1) REQUESTS 2) MANUAL 3) CRON
        # ASSIGNED JOBS OVER UNASSIGNED JOBS 

        logwriter( $sql, 5 );

        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );

        my ( $job_id, $job_name, $job_type, $job_targets, $job_assigned, $job_priority, $job_code,
            $times_failed ) = $sth_sel->fetchrow_array(  );
        $sth_sel->finish;

        if ( $job_id ) {

            logwriter( "id=$job_id\tname=$job_name\ttype=$job_type\ttarget=$job_targets\t"
                ."\tpriority=$job_priority", 5 );
            # FOUND SCHEDULED SCAN

            if ( $times_failed ge 3 ) {
                #FAIL JOB TOO MANY FAILURES
                $sql = qq{ UPDATE I3_jobs SET status='F' WHERE id='$job_id' };
                safe_db_write ( $sql, 3 );            #use insert/update routine
            }

            run_job ( $job_id, $serverid );        #RUN ONE JOB THEN QUIT
            return;
        }

    }
    logwriter( "No work in scan queues to process", 4 );
    exit;

}

#prep to to run job / call get server credentials, update dashboard, verify host_list, handle notification, setup the scan
sub run_job {
    my ( $job_id, $sel_servid ) = @_;

    my ( $sql, $sth_sel, $sth_upd );

    $serverid = get_server_credentials( $sel_servid );  #GET THE SERVER ID'S FOR WORK PROCESSING
    logwriter( "host=$nessushost, port=$nessusport, user=$nessususer, pass=$nessuspassword", 5 );
    if ($serverid == 0 ) {  #CHECK FOR AVAILABLE SCAN SLOTS (AN ID WOULD BE RETURNED)
        logwriter( "WARNING: Currently Not Enough Free scan slots to run cron job", 4 );
        return;
    }

    #UPDATE SERVER COUNT OF RUNNING SCANS
    $sql = qq{ UPDATE nessus_servers SET current_scans=current_scans+5 WHERE id=$serverid };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    my $startdate = getCurrentDateTime("datetime");

    $sql = qq{ SELECT name, username, job_TYPE, meth_TARGET, meth_VSET, meth_TIMEOUT
        FROM I3_jobs WHERE id='$job_id' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my ( $Jname, $juser, $Jtype, $host_list, $Jvset, $jtimout ) = $sth_sel->fetchrow_array(  );
    $sth_sel->finish;

    if ( $host_list eq "" ) {
        logwriter( "INVALID SCAN CONFIG\t\t( SCAN PUT ON HOLD )", 2 ); 

        $sql = qq{ UPDATE I3_jobs SET status='H' WHERE id='$job_id' };
        safe_db_write ( $sql, 2 );            #use insert/update routine
        return;
    }

    logwriter( "A SCHEDULED SCAN #$job_id WILL BEGIN SHORTLY FOR THE SELECTED HOSTS", 4 );
    logwriter( "\n-----------------\n<HOSTLIST>\n$host_list\n</HOSTLIST>\n", 4 );
    logwriter( "Available Server=$serverid", 4 );

    $sql = qq{ UPDATE I3_jobs SET status='R', scan_START='$startdate' WHERE id='$job_id' };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    if ( $notify_by_email ) {
        if ( $Jtype =~ /C/ ) {
            cron_start( $startdate, $host_list );
        } else {
            sched_start( $job_id )            #ie MANUAL/REQUESTS
        }
    }
    logwriter( "Begin Script Execution: nessusScan", 4 );

    my $enddate = setup_scan( $job_id, $Jname, $juser, $Jtype, $host_list, $Jvset, $jtimout );

    if ( $notify_by_email ) {
        if ( $Jtype =~ /C/ ) {
            cron_finish( $enddate, $host_list );
        } else {
            sched_finish( $job_id )
        }
    }
    logwriter( "End Script Execution", 4 );

}

#build hostlist, remove exceptions, call scanlite, load results, 
sub setup_scan {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $job_id, $Jname, $juser, $Jtype, $target, $Jvset, $timeout ) = @_;
    
    my ( $outfile, $targetfile, $targetinfo, $nessus_cfg, $workfile, $CIDR );
    my ( $sql, $sth_sel, $sth_upd, $sth_ins );
    my ( $job_title, $nessusok, $scantime );

    #INIT DEFAULT VALUES (CRON SCAN DEFAULTS)
    $CIDR="";

    #UPDATE JOB_ID with SERVER PROCESS INFO
    $sql = qq{ UPDATE I3_jobs SET scan_SERVER='$serverid', scan_PID=$$ WHERE id='$job_id' };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    $job_title = "$job_id - $Jname";

    if ( $Jtype =~ /c/i ) {
        $CIDR = $target;
        $targetinfo = build_hostlist( $target );            #CODE TO HANDLE SCAN EXCEPTIONS
        #LONG LIST OF HOSTS MAY BE CAUSING INCREASED NESSUS CLIENT LOAD
        if ( $exclude_hosts eq "" ) { $targetinfo = $target; }        #NO IPS FILTERED USE CIDR

        $sql = qq{ UPDATE I3_jobs SET fk_name='$CIDR' WHERE id='$job_id' };
        safe_db_write ( $sql, 4 );            #use insert/update routine

    } else {
        $target =~ s/\n/\r/g;
	$target =~ s/\r\r/\r/g;

	#ATTEMPT TO GET IP'S INCASE USERS SUPPLIED HOSTNAMES
	my @tmp_hostarr = split /\r/, $target;
	foreach my $line (@tmp_hostarr ) {
	    my $resolved_ip = resolve_name2ip ( $line );
	    if ( $resolved_ip eq "" ) { $resolved_ip = $line; }
	    print "translated NAME=[$line] to [$resolved_ip]\n";
	    $targetinfo .= "$resolved_ip\r";
        }
	#INCASE LOOKUP FAILS MISERABLY
	if ( $targetinfo eq "" ) { $targetinfo = "$target"; }

    }

    #MERGE/FILTER Potential \r\n as \r to ensure proper split
    $targetinfo =~ s/\n/\r/g;
    $targetinfo =~ s/\r\r/\r/g;

    my @hostarr = split /\r/, $targetinfo;
    my $nessus_pref = get_prefs( $Jvset, $job_id );
    #MAKE IT GLOBAL FOR USE WITH INCIDENT TRACKER
    @nessus_plugins = get_plugins( $Jvset );

    my @results = run_scanlite_nessus($nessus_pref, \@nessus_plugins, $timeout, $Jname,
        $juser, \@hostarr, $Jvset);
    $scantime = getCurrentDateTime("datetime");

    if ( check_dbOK() == "0" ) { $dbh = conn_db(); }

    if ( @results != FALSE ) {        #SUCCESSFUL NESSUS RUN
        logwriter( "[$job_title] Begin SQL Import", 4 );

        if ( load_scanlite_results( \@results, $job_id, $job_title, $Jtype, $juser, $Jvset, $scantime, $CIDR ) ){
            logwriter( "[$job_title] Completed SQL Import", 4 );

            $sql = qq{ UPDATE I3_jobs SET status='C' WHERE id='$job_id' };
            safe_db_write ( $sql, 4 );            #use insert/update routine   
            $nessusok = TRUE;
        } else {
            logwriter( "[$job_title] Failed SQL Import", 2 );
            $nessusok = FALSE;
        }
    }

    #UPDATE SERVER COUNT OF RUNNING SCANS
    $sql = qq{ UPDATE nessus_servers SET current_scans=current_scans-5 WHERE id=$serverid };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    if (!$nessusok) {

        if ( $CONFIG{'RESCAN'} eq "0" ) {;
            # MARK SCAN AS FAILED
            $sql = qq{ UPDATE I3_jobs SET status='F', scan_END=now() WHERE id='$job_id' };
            safe_db_write ( $sql, 4 );            #use insert/update routine

        } else {
            # RESET FOR A RESCAN

            $sql = qq{ SELECT failed_attempts FROM I3_jobs WHERE id='$job_id' LIMIT 1 };
            logwriter( $sql, 4 );
            $sth_sel = $dbh->prepare( $sql );
            $sth_sel->execute(  );
            my ( $failed_count ) = $sth_sel->fetchrow_array(  );
            my $tmpStatus = "S";
            if ( $failed_count ge 3 ) { $tmpStatus = "F"; }
            $failed_count += 1;

            $sql = qq{ UPDATE I3_jobs SET status='$tmpStatus', scan_SERVER='0', scan_START=NULL, scan_END=NULL,
            scan_PID='0', failed_attempts=failed_attempts+1 WHERE id='$job_id' };
            safe_db_write ( $sql, 4 );            #use insert/update routine

            print "ATTEMPT [ $failed_count ] TO HANDLE FAILED SCANS\n";

            if ( $tmpStatus eq "F" ) { logwriter( "Marked job [ $job_title ] as Failed", 2 ); }

            exit;
        }
    }
    return $scantime;
}

#run scan return issues to setup scan
sub run_scanlite_nessus {
    my (%nes_prefs) = %{$_[0]};
    my (@nes_plugins) = @{$_[1]};
    my ($timeout) = $_[2];
    my ($jobname) = $_[3];
    my ($username) = $_[4];
    my (@hosts) = @{$_[5]};
    my ($sid) = $_[6];

    my ($cmd, $retval);
    my ($sth_ins);

    #print "\nConfirmng Host List Join:\n";
    my $targets = join(',',@hosts);
    #my @targets = @hosts;
    #print "$targets\n";

    logwriter( "nessus_scan: server=$nessushost\tport=$nessusport\ntargets=$targets\n", 5 );

    my $nessus = Net::Nessus::ScanLite->new(
        host        => $nessushost,
        port        => $nessusport,
        ssl         => 1  # comment or set to 1 out if using ssl
    );

    # set the profile/preferences
    $nessus->preferences( { %nes_prefs } );

    # set the plugins to use
    $nessus->plugin_set( join(';',@nes_plugins) );

    logwriter( "nessus_scan: $jobname - launching attack.", 5 );

    # wrapped in an eval block so we can set up a timer to kill a scan
    # that is running too long.
    eval {
        local $SIG{ALRM} = sub { die "NessusTimeout\n" };
        #local $SIG{CHLD} = sub { if ( $? && $? > 0 ) { die "Program Exited Funny: $?\n"; } };
        alarm $timeout;
        # login to the scanner and fire up the scan
        if( $nessus->login($nessususer, $nessuspassword) ) {
            logwriter( "nessus_scan: $jobname - connected to " . $nessus->hostport, 5 );

            # FIRST FIELD IS DESIGNATED FOR LIVE STATS
            my $sth = "";

            #$nessus->attach( @targets);
            #$nessus->attack($sth, $targets);
            $nessus->attack( $targets );
            logwriter( "nessus_scan: $jobname - attack ended.", 5 );
            logwriter( "nessus_scan: $jobname - Holes found: " . $nessus->total_holes, 4 );
            logwriter( "nessus_scan: $jobname - Infos found: " . $nessus->total_info, 4 );
            logwriter( "nessus_scan: $jobname - Notes found: " . $nessus->total_note, 4 );
            logwriter( "nessus_scan: $jobname - Scan Duration: " . $nessus->duration . " secs", 4 );

            alarm 0;

            if( $debug ) {
               #warn Dumper($nessus->nessus2tmpl);
            }

            if ( $nessus->total_holes + $nessus->total_info + $nessus->total_note eq "0" ) {
                $no_results = TRUE;
                return TRUE;        #GOT TO FAKE IT AS WE HAVE NO RESULTS;
            }
   
            # holes and infos have the same structure, so merge them
            # and process as one array for simplicity
            my @issues = ($nessus->hole_list, $nessus->info_list,
                          $nessus->note_list);

            return @issues;
         } else {
             logwriter( "nessus_scan: $jobname - Error running scan: " . $nessus->error, 2 );
            return FALSE;
         } # fi nessus->login()
      }; # end eval
}

#process issues [ update_host_record, load_results, update_stats, update server feed_info ]
sub load_scanlite_results {
    my (@issues)    = @{$_[0]};
    my ($job_id)    = $_[1];
    my ($job_title) = $_[2];
    my ($scantype)  = $_[3];
    my ($username)  = $_[4];
    my ($sid)            = $_[5];
    my ($scantime)  = $_[6];
    my ($fk_name)   = $_[7];

    my ( $sth_sel, $sql, $domain, $hostname, $mac_address, $report_key );

    my %hostHash;
    # set up some error counters
    my $errCount = 0;
    my @errMsg;
    my $ctable = {};	#STORE NETBLOCKS FOR ORG LOOKUP

    #Build a report_key value to secure reports.
    my @arr = split(/\./, rand() );

    #print "$arr[0]|$arr[1]|";

    if ( $arr[1] && is_number($arr[1]) ) {
        $report_key = $arr[1];
    } else {
        logwriter( "Failed Report Key generation", 3 );
    }   
  
    if ( !defined ( $cred_name ) || $cred_name eq "" ) {
        $cred_name = "Null";
    } else {
        $cred_name = "'$cred_name'";
    }

    $sql = qq{ INSERT INTO nessus_reports 
        ( username, name, fk_name, sid, scantime, scantype, report_key, cred_used ) VALUES (
        '$username', '$job_title', '$fk_name', '$sid', '$scantime','$scantype', '$report_key', $cred_name ) };
    safe_db_write ( $sql, 4 );

    $sql = qq{ SELECT report_id FROM nessus_reports WHERE scantime='$scantime' AND report_key='$report_key'
        ORDER BY scantime DESC LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    my $report_id = $sth_sel->fetchrow_array( );
    $sth_sel->finish;

    if ( ! $report_id ) {
        logwriter( "failed to lookup report after insert for scan $scantime", 2 );
        return 0;
    } else {
        $sql = qq{ UPDATE I3_jobs SET report_id='$report_id' WHERE id="$job_id" };
        safe_db_write( $sql, 4 );
    }

    $sql = qq{ UPDATE I3_jobs SET report_id='$report_id', scan_END='$scantime' 
        WHERE id='$job_id' LIMIT 1};
    safe_db_write ( $sql, 4 );            #use insert/update routine

    if ( $no_results ) {
        logwriter( "NO Results to Import or Host offline", 2 );
        #return 1;   #RETURN A SUCCESS - Do it Later want to build stats table records too
    } else {
        logwriter( "Begin SQL Import", 4 );

        my $i = 0;        #COUNT AND INSERT NO MORE THAN 100 RECORDS AT A TIME.
        $sql = "";        #BE SURE TO REINITIALIZE SQL TO NULL

	#ORG LOOKUP POPULATE CIDRBLOCK TABLE
	if ( $scantype =~ /c/i ) {
	    my $net_table = new2  Net::Netmask($fk_name);   #$fk_name IS A CIDR AS SCANTYPE IS /C
	    $net_table->storeNetblock( $ctable );
	    logwriter( "LOADED CIDR $fk_name into Netblock", 5 ); 
	} else {
	    $sql = qq{ SELECT id, CIDR FROM I3_subnets WHERE 1 };
	    $sth_sel = $dbh->prepare( $sql );
	    $sth_sel->execute;
	    while( my ( $subID, $CIDR )=$sth_sel->fetchrow_array){
		my $net_table = new2  Net::Netmask($CIDR); 
		$net_table->storeNetblock( $ctable );
	    }
	    logwriter( "LOADED ALL Netblocks", 5 ); 
	}
        #GET POPULATE HOSTHASH WITH HOSTNAME /DOMAIN FROM PLUGIN 10150
        logwriter( "nessus_scan: Start Populating HostHash", 5 );
        foreach( @issues ) {
            my $issue = $_;
            my ($scanid, $port, $desc, $service, $proto, $host ) = "";

	    #THIS RETURNES EITHER HOSTNAME OR IP IF NESSUS CAN RESOLVE DNS
            $host = $issue->Host;	    
            $scanid = $issue->ScanID;

            my ( $hostip, $hostname, $domain, $mac_address) = "";
            if( $host eq "" ) { next; }

	    if ( !exists( $hostHash{$host}{'dns'}) ) {

		#DETERMINE IF IT IS AN IP (CRITICAL STEP AS SCANLITE RETURNS EITHER HOSTIP/HOSTNAME)
		my $isIP = FALSE;
		if ( $host =~ m/^(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)\.(\d\d?\d?)/ ){
		    if($1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255) {
			$hostip=$host;
			$isIP = TRUE;
		    }   
		}
		
                if ( $isIP == FALSE ) {	    #MUST BE A NAME ATTEMPT TO RESOLVE ELSE FAILED MISERABLY
                    $hostname=$host;	    #LETS AT LEAST SET NAME IN CASE ALL ELSE FAILS
		    my $tmp_hostip = resolve_name2ip( $host );
                    if ( defined( $tmp_hostip) && $tmp_hostip ne "" ) { $hostip = $tmp_hostip; }
		}

		if ( defined( $hostip ) && $hostip ne "" ) {
		    my $match = findNetblock($hostip, $ctable);
		    if ( $match ) {
			$sql = qq{ SELECT id, site_code, ORG FROM I3_subnets WHERE CIDR = "$match" LIMIT 1 };
			$sth_sel = $dbh->prepare( $sql );
			$sth_sel->execute;
			my ( $subID, $site_code, $ORG) = $sth_sel->fetchrow_array( );

			$hostHash{$host}{'org'} = "$ORG";	    #FOUND ORG
			$hostHash{$host}{'site'} = "$site_code";    #FOUND SITE
			logwriter( "FOUND ORG MATCH FOR hostip=$hostip ORG=$ORG site=$site_code", 5 );
	    	    }
		    #ATTEMPT TO CONSULT VARIOUS DNS SERVERS IN CASE SCANLITE RETURNTED IP
                    my $tmp_hostname = resolve_host( $hostip );
		    if ( defined( $tmp_hostname ) && $tmp_hostname ne "" ) { $hostname = $tmp_hostname; }
		}
	

    		$hostHash{$host}{'ip'} = $hostip; 
		if( defined( $hostname ) && $hostname ne "" ) {
		    $hostHash{$host}{'fqdn'} = $hostname;
		    $hostHash{$host}{'dns'} = "-1";    #INDICATE RESOLVED BY NAME FAILED
                    logwriter( "nessus_scan: successfully looked up name [$host]", 5 );
                } else {
        	    $hostHash{$host}{'dns'} = "-1";    #INDICATE RESOLVED BY NAME FAILED
                }
	    } 
            if ( $scanid eq "10150" ) {
                $mac_address = "unknown";
                my $txt_msg = $issue->Description;
                logwriter( "nessus_scan: plugin 10150 data: [[$txt_msg]]", 5 );
                my @arrMSG = split /\n/, $txt_msg;
                foreach my $line (@arrMSG) {
                    $line =~ s/\#.*$//;
                    chomp($line);
                    $line =~ s/\s+$//;
                    $line =~ s/^\s+//;
                    logwriter( "nessus_scan: LINE=[$line]", 5 ); 
                    if ($line =~ /computer\sname/i ) {
                        my @temp=split(/=/,$line,2);
                        $temp[0] =~ s/\s+$//;
                        $temp[0] =~ s/^\s+//;
                        $hostname = lc( $temp[0] );
                        logwriter( "nessus_scan: hostname=[$hostname]", 5 );
                    } elsif ($line =~ /workgroup/i ) {
                        my @temp=split(/=/,$line,2);
                        $temp[0] =~ s/\s+$//;
                        $temp[0] =~ s/^\s+//;
                        $domain = lc( $temp[0] );
                        logwriter( "nessus_scan: domain=[$domain]", 5 );
                        if ( $domain eq "fema" ) { 
                            $domain = "fema.net"; 
                            $hostname = "$hostname.$domain";
                        }
                    } elsif ($line =~ /^([0-9a-fA-F][0-9a-fA-F]:){5}([0-9a-fA-F][0-9a-fA-F])$/ ) {
                        $mac_address = uc( $line );
                    }

                    next;
                }
                if ( $hostHash{$host}{'dns'} eq "-1" ) { #ONLY UPDATE NAME FROM 10150 WHEN DNS FAILS
                    if ( defined( $hostname ) && $hostname ne "" && defined( $domain ) && $domain ne "" ) {
                        $hostHash{$host}{'fqdn'} = $hostname;
                        $hostHash{$host}{'domain'} = $domain;
                        $hostHash{$host}{'dns'} = '1'; 
                        logwriter( "nessus_scan: success plugin 10150 to look up name [$hostname]", 5 );  
                    }
                }
                $hostHash{$host}{'mac'} = $mac_address;   #ALWAYS GET MAC
            }
        }
        logwriter( "nessus_scan: Finished Populating HostHash", 5 );

        logwriter( "nessus_scan: Start Processing Results", 5 );
	$sql = "";
        foreach( @issues ) {
            my $issue = $_;
            my ($scanid, $port, $desc, $service, $proto, $host ) = "";
            $scanid = $issue->ScanID;
            $port = $issue->Port;
            $desc = $issue->Description;
            $service = $issue->Service();
            $proto = $issue->Proto;
            $host = $issue->Host;

            # service is actually stored in the DB as "service (port/proto)"
            my $app = $service;
            if(defined($service) && $service ne "") {
                if(defined($proto) && $proto ne "") {
                    $service = "$service ($port/$proto)";
                } else {
                    $app = "general";
                    $proto = $service;
                    $port = "";
                    $service = "general/$service";
                }
            }

            # get the hostname and domain
            my ( $hostip, $hostname, $mac_address ) = "";
            if($host ne "") {
                # check the %hostHash to see if it's cached already
                if(exists($hostHash{$host})) {
		    $hostip = $hostHash{$host}{'ip'};
                    $hostname = $hostHash{$host}{'fqdn'};
                } else {
		    $hostip = $host;
                    $hostname = $host;
                }
            } else {
		$hostip = $host;
                $hostname = $host;
            }

            # get the risk value from the text in the description
            my $risk='7';
            $risk='1' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Serious/s);
            $risk='1' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Critical/s);
            $risk='2' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*High/s);
            $risk='3' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium/s);
            $risk='4' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low/s);
            $risk='5' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium/s);
            $risk='6' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Low/s);
            $risk='7' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*Info/s);
            $risk='7' if ($desc =~ m/Risk [fF]actor\s*:\s*(\\n)*[nN]one/s);

            #remove the Risk Factor from the description
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Serious((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Critical((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*High((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Medium((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Medium\/Low((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Low\/Medium((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Low((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*Info((\\n)+| \/ |$)//s;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*[nN]one to High((\\n)+|(\s)+| \/ |$)//;
            $desc=~ s/Risk [fF]actor\s*:\s*(\\n)*[nN]one((\\n)+| \/ |$)//s;

            $service =~ s/(\\n)+$//;
            $desc =~ s/(\\n)+$//;
            $desc =~ s/\n+$//;

            $service = htmlspecialchars($service);
            $desc = htmlspecialchars($desc);

            #logwriter( "nessus_scan: $scanid:$ip:$host:$domain:$port:$service:$proto:$risk", 5 );

            #GET OS INFO
            if ( $scanid eq "11936" ) {
                my $os = extract_os( $desc );
                $hostHash{$host}{'os'} = $os;
            }



            if ( !defined( $sql ) || $sql eq "" ) {
                #FIRST ITERATION OR RESET VARIABLE AFTER IMPORTING 100 RECORDS
                $sql = "INSERT INTO nessus_results \n"
                    ."( report_id, scantime, hostip, hostname, service, port, protocol , app, scriptid, risk, msg )\nVALUES\n";
                $i=0;
            }

            $i += 1;
            $sql .= " ('$report_id', '$scantime', '$hostip', '$hostname', '$service', '$port', '$proto', '$app', '$scanid', "
                ."'$risk', '$desc' ),\n";

            if ( $i >= 100 ) {
                chop($sql);
                chop($sql);
                $sql .= ";";
                #CONNECT DB AND INSERT 100 RECORDS
                safe_db_write( $sql, 5 );
                logwriter( "[$job_title] - inserted $i results records", 4 );

                $sql = "";
                $i = 0;

            }
        }

        if ( defined( $sql ) && $sql ne "" ) {
            chop($sql);
            chop($sql);
            $sql .= ";";
            #CONNECT DB AND INSERT REMAINDER OF RECORDS
            safe_db_write( $sql, 5 );
            logwriter( "[$job_title] - inserted $i results records", 4 );

            $sql = "";
        }
    }

    if ( $scantype =~ /c/i ) {
        if ( defined( $fk_name ) && $fk_name ne "" ) {
            $sql = qq{ UPDATE I3_subnets SET report_id='$report_id', dtLastScanned='$scantime' 
                WHERE CIDR='$fk_name' LIMIT 1 };
            logwriter( $sql, 4 );
            safe_db_write ( $sql, 4 );            #use insert/update routine
        }
    } elsif ( $scantype =~ /s/i ) {
        if ( defined( $fk_name ) && $fk_name ne "" ) {
            $sql = qq{ UPDATE I3_systems SET report_id='$report_id', dtLastScanned='$scantime' 
                WHERE name='$fk_name' LIMIT 1 };
            logwriter( $sql, 4 );
            safe_db_write ( $sql, 4 );            #use insert/update routine
        }
    }

    #CHECK IF PROFILE IS SAFE/FULL AUDIT WHICH ALLOWED TO UPDATE HOST TRACKER STATS
    $sql = qq{ SELECT update_host_tracker FROM nessus_settings WHERE id='$sid' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    my $update_stats = $sth_sel->fetchrow_array( );

    #UPDATE HOST RECORD
    foreach my $host ( sort keys %hostHash ) {
	my $hostip	= $hostHash{$host}{'ip'};
        my $hostname	= $hostHash{$host}{'fqdn'};
        my $mac_address = $hostHash{$host}{'mac'};
        my $os		= " ";
	my $ip_org	= " ";
	my $ip_site	= " ";
        if ( defined ( $hostname ) ) {
            if ( $hostHash{$host}{'os'}   ) {     $os = $hostHash{$host}{'os'};     }
	    if ( $hostHash{$host}{'org'}  ) { $ip_org = $hostHash{$host}{'org'};    }
	    if ( $hostHash{$host}{'site'} ) { $ip_site = $hostHash{$host}{'site'};  }
            logwriter( "mac=$mac_address\tname=$hostname\tip=$hostip\treport=$report_id\tos=$os\torg=$ip_org", 4 );
            update_host_record ( $mac_address, $hostname, $hostip, $os, $ip_org, $ip_site, $report_id, $update_stats );
        }
    }
    logwriter( "Completed SQL Import", 4 );
    update_stats ( $job_id, $job_title, $report_id, $scantime );

    return 1;

}

#called by load_scanlite_results to populate stats for report
sub update_stats {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $job_id, $job_title, $report_id, $scantime ) = @_;
    my ( $sth_sel, $sql );

    my %SCAN = ();
    my %RISKS = ( 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0);
    my $hostcnt = 0;
    my $runtime = 0;
    my $trend = 0;

    if ( ! $report_id ) {
        logwriter( "UPDATE STATS: failed to lookup report for scan $scantime", 2 );
        return;
    }

    $sql = qq{ SELECT msg from nessus_results
        WHERE report_id = '$report_id' AND scriptid='19506' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    $SCAN{'Scanner IP'} = "";
    $SCAN{'Nessus version'} = "";
    $SCAN{'Type of plugin feed'} = "";
    $SCAN{'Plugin feed version'} = "";

    while ( my ( $txt_msg ) = $sth_sel->fetchrow_array ) {
        # Exclude null nessus_value records
        if ( $txt_msg && $txt_msg ne "" ) {
            my @arrMSG = split /\n/, $txt_msg;

            foreach my $line (@arrMSG) {
                $line =~ s/\#.*$//;
                chomp($line);
                $line =~ s/\s+$//;
                $line =~ s/^\s+//;
                if ($line eq "") { next; }
                my @temp=split(/:/,$line,2);
                $temp[0] =~ s/\s+$//;
                $temp[0] =~ s/^\s+//;
                $temp[1] =~ s/\s+$//;
                $temp[1] =~ s/^\s+//;
                if ($temp[0] ne "") { $SCAN{$temp[0]}=$temp[1]; }
                #print "0=$temp[0] 1=$temp[1]\n";
            }
        }
    }
    my $server_ip = "" . $SCAN{'Scanner IP'};
    my $server_nver = "" . $SCAN{'Nessus version'};
    my $server_feedtype= "" . $SCAN{'Type of plugin feed'};
    my $server_feedversion = "" . $SCAN{'Plugin feed version'};

    if ( defined ( $server_ip ) && $server_ip ne "" ) {
        $sql = qq{ UPDATE nessus_reports SET server_ip='$server_ip', 
            server_nversion='$server_nver',        server_feedtype='$server_feedtype',
            server_feedversion='$server_feedversion' WHERE report_id='$report_id' };
        safe_db_write( $sql, 4 );

        #UPDATE INFO IN THE NESSUS_SERVERS TABLE TOO IF FEED DATE CHANGED
        $sql = qq{ UPDATE nessus_servers SET server_nversion='$server_nver',
            server_feedtype='$server_feedtype', server_feedversion='$server_feedversion' 
            WHERE hostname='$server_ip' AND server_feedversion <= '$server_feedversion' };
        safe_db_write( $sql, 4 );

    }

    $sql = qq{ SELECT scan_START, scan_END FROM I3_jobs        WHERE id = '$job_id' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    my ( $start_dttm, $end_dttm ) = $sth_sel->fetchrow_array;

    $sql = qq{ SELECT count(risk) as count, risk FROM `nessus_results` WHERE report_id='$report_id'
        AND falsepositive<>'Y' AND scriptid <> 10180 GROUP BY risk };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ($count, $risk )=$sth_sel->fetchrow_array) {
        $RISKS{"$risk"} = $count;
    }

    $sql = qq{ SELECT count( distinct hostIP) as count FROM nessus_results 
        WHERE report_id='$report_id' AND falsepositive<>'Y' AND scriptid <> 10180 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    ( $hostcnt ) = $sth_sel->fetchrow_array; 
    $sth_sel->finish;

    $runtime = datediff( $start_dttm, $end_dttm, "M" );

    my $update_time = getCurrentDateTime("datetime");

    $sql = "INSERT INTO nessus_report_stats ( report_id, name, iHostCnt, dtLastScanned, iScantime,
        vSerious, vHigh, vMed, vMedLow, vLowMed, vLow, vInfo, trend, dtLastUpdated ) VALUES (
        '$report_id', '$job_title', '$hostcnt', '$scantime', '$runtime', '$RISKS{1}', '$RISKS{2}', 
        '$RISKS{3}', '$RISKS{4}', '$RISKS{5}', '$RISKS{6}', '$RISKS{7}', '$trend', '$update_time' ); ";

    safe_db_write ( $sql, 4 );

}

#called to update host tracker, most recent scan info for hosts
sub update_host_record {
    # VER: 1.1 MODIFIED: 5/22/08 07:31
    my ( $mac_address, $hostname, $hostip, $os, $ip_org, $ip_site, $report_id, $update_stats ) = @_;

    $os=~ s/^\s+//;  #MAY HAVE PASSED " " SO IT WOULD NOT BE NULL

    my ( $sql, $sql_select, $sth_sel, $scantime );
    my $scan_state = "Passed";  #SET INITAL STATE TO PASS
    my %RISKS = ( 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0);

    if ( defined( $report_id )) {
        $sql = qq{ SELECT count(risk) as count, risk, scantime FROM nessus_results 
            WHERE report_id='$report_id' AND hostip='$hostip'
            AND falsepositive<>'Y' AND scriptid <> 10180 and msg <> '' GROUP BY risk };

        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;
        while( my ($count, $risk, $tmp_scantime )=$sth_sel->fetchrow_array) {
            $RISKS{"$risk"} = $count;
            $scantime=$tmp_scantime;
        }
    }

    if ( $ip_org eq "" ) { $ip_org = "unknown"; }
    my $risk_value = $RISKS{1} + $RISKS{2};
    if ( $risk_value > 0 ) { $scan_state = "Pending Fixes"; }

    $sql = qq{ SELECT id FROM hosts WHERE hostname = '$hostname' LIMIT 1 };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    my ( $host_id ) = $sth_sel->fetchrow_array;

    if ( defined( $host_id ) && $host_id > 0 ) {
        my $os_sql = " ";  #ONLY UPDATE OS FIELD IF FOUND ( SOME PROFILES MAY NOT TEST )
	if ( defined( $os ) && $os ne "" ) { $os_sql = " os='$os',"; }
        $sql = "UPDATE hosts SET hostname='$hostname', hostip='$hostip',$os_sql
           scanstate='$scan_state', site_code='$ip_site', ORG='$ip_org', lastscandate='$scantime' WHERE id='$host_id';";
	safe_db_write ( $sql, 4 );
    } else {
        $sql = "INSERT INTO hosts ( hostip, hostname, status, os, site_code, ORG, scanstate, report_id, lastscandate ) VALUES (
            '$hostip', '$hostname', 'Production', '$os', '$ip_site', '$ip_org', '$scan_state', '$report_id', '$scantime' );";
        safe_db_write ( $sql, 4 );
    }

    #ENSURE HOST_ID IS LOOKED UP AFTER INSERT OR QUIT 
    #LOOKUP HOST_ID AGAIN FOR UPDATE OF MACS / INCIDENT TRACKER CODE
    $sql = qq{ SELECT id FROM hosts WHERE hostname = '$hostname' LIMIT 1 };
    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    ( $host_id ) = $sth_sel->fetchrow_array;
    if ( defined( $host_id ) && $host_id > 0 ) {
        #UPDATE ONE-TO-MANY MAC/IP ADD
        if ( defined( $mac_address ) && $mac_address ne "unknown" && $mac_address ne "" ) { 
            $sql = qq{ SELECT id FROM host_macs WHERE mac_address = '$mac_address' LIMIT 1 };
            logwriter( $sql, 5 );
            $sth_sel = $dbh->prepare( $sql );
            $sth_sel->execute;
            my ( $mac_id ) = $sth_sel->fetchrow_array;
            if ( defined( $mac_id ) && $mac_id > 0 ) {
               $sql = "UPDATE host_macs SET host_id='$host_id', hostip='$hostip' WHERE id='$mac_id'";
               safe_db_write ( $sql, 4 );
            } else {
                $sql = "INSERT INTO host_macs ( host_id, hostip, mac_address, createdate ) VALUES (
                    '$host_id', '$hostip', '$mac_address', '$scantime' );";
                safe_db_write ( $sql, 4 );
            }
        }

        #AN EFFORT TO TRACK PREVIOUSLY IDENTIFIED CRITICALS / HIGHS AGAINST THIS SCAN
        #WILL CODE IN A VALIDATION TO CHECK PLUGINS TESTED AGAINST KNOWN RISKS
        #USE OF CREDENTIALS WILL AFFECT ACCURACY (OF TRACKER)
        #update_incident( id_in_hosts_table, ip_at_time_of_scan, report_id this scan );
        update_incidents ( $host_id, $hostip, $report_id );
    } else {
	$host_id = 0;
    }

    #ADD ENTRY TO VULN_SUMMARY TABLE
    $sql = "INSERT INTO host_stats ( report_id, host_id, hostip, hostname, scantime, 
	vSerious, vHigh, vMed, vMedLow, vLowMed, vLow, vInfo ) VALUES (
        '$report_id', '$host_id', '$hostip', '$hostname', '$scantime', 
	'$RISKS{1}', '$RISKS{2}', '$RISKS{3}', '$RISKS{4}', '$RISKS{5}', '$RISKS{6}', '$RISKS{7}' );";

    safe_db_write ( $sql, 4 );

}



#called to update incident tracker, against most recent scan results
sub update_incidents {
    my ( $host_id, $hostip, $report_id ) = @_;

    my ( $sql, $sql2, $sth_sel, $sth_sel2, $scantime );

    #$nessus_plugins

    #INSERT /UPDATE EXISTING INCIDENTS
    my $now = getCurrentDateTime("datetime");

    $sql = qq{ SELECT result_id, scriptid, risk, service, msg FROM nessus_results 
        WHERE report_id='$report_id' AND hostip='$hostip' AND risk <='2' AND falsepositive<>'Y' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $result_id, $scriptid, $risk, $service, $msg )=$sth_sel->fetchrow_array) {
        $sql2 = qq{ SELECT id FROM I3_Incidents
	    WHERE host_id='$host_id' AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
        logwriter( $sql2, 5 );
        $sth_sel2 = $dbh->prepare( $sql2 );
        $sth_sel2->execute;
	my ( $incident_id ) = $sth_sel2->fetchrow_array;

	if ( defined($incident_id ) && $incident_id ne "" ) {
	    $sql2 = qq{ UPDATE I3_Incidents SET result_id='$result_id', msg='$msg', status='open' WHERE id="$incident_id" };
	} else {
	    $sql2 = qq{ INSERT INTO I3_Incidents ( host_id, result_id, scriptid, service, risk, msg, date_open ) VALUES (
	        '$host_id', '$result_id', '$scriptid', '$service', '$risk', '$msg', '$now' );};
	}
        safe_db_write ( $sql2, 4 );

    }


    $sql = qq{ SELECT id, scriptid, service FROM I3_Incidents
            WHERE host_id='$host_id' };
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;
    while(my ( $incident_id, $scriptid, $service )=$sth_sel->fetchrow_array) {
	#FAIL SAFE DO MARK ANY PLUGINS NOT TESTED AS RESOLVED
	if ( grep { $_ eq $scriptid } @nessus_plugins ) {
	    $sql2 = qq{ SELECT result_id FROM nessus_results
        	WHERE report_id='$report_id' AND hostip='$hostip' 
		    AND scriptid='$scriptid' AND service='$service' LIMIT 1 };
            logwriter( $sql2, 5 );
	    $sth_sel2 = $dbh->prepare( $sql2 );
            $sth_sel2->execute;
	    my ( $result_id ) = $sth_sel2->fetchrow_array;

	    if ( defined($result_id ) && $result_id ne "" ) {
		#CURRENTLY NOT RESOLVED
	    } else {
		#CURRENTLY CREDENTIALS VS NO /CREDENTIALS WILL CAUSE AN INVALID CLEANUP STATE TO BE SET
	        $sql2 = qq{ UPDATE I3_Incidents SET status='resolved', date_resolved='$now' WHERE id='$incident_id' };
	    }
            safe_db_write ( $sql2, 4 );

	} else {
	    logwriter( "PLUGIN $scriptid apparrently was not tested", 5 );
	}

    }

}

#build notification to be sent out on subnet scan start
sub cron_start {
    # VER: 1.0 MODIFIED: 4/02/07 13:48
    my ( $scantime, $subnet ) = @_;
    my ( $email_addr, $msg, $html_data, $sth_sel, $statistics );

    #READ FORM EMAIL MESSAGE
    my $body = "";

    open (MESSAGE, "$messages_dir/cron_start") || die "couldn't open the file!";
    while (my $line = <MESSAGE>) {
        $body .= $line;
    }

    close(MESSAGE);

  my $sql = qq{ SELECT t1.id, t1.CIDR,
        t1.cntHosts, t1.tiAutoEmail, t1.poc, t1.poc_alt,
        t2.scan_START, t2.scan_END, t2.scan_SUBMIT, t2.status, 
        t3.dtLastScanned, t3.iScantime, t3.vSerious, t3.vHigh, t4.hostname
        FROM I3_subnets t1
        LEFT JOIN I3_jobs t2 ON t2.meth_TARGET = t1.CIDR
        LEFT JOIN (
            SELECT name, dtLastScanned, iScantime, vSerious, vHigh
            FROM nessus_report_stats
            WHERE name = '$subnet'
            ORDER BY dtLastScanned DESC 
            LIMIT 1 
            )t3 ON t3.name = t1.CIDR
        LEFT JOIN nessus_servers t4 ON t4.id = t2.scan_SERVER 
        WHERE t1.CIDR = '$subnet' AND t2.status = 'R' };

    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my ( $sId, $sName, $sCnt, $bAutoEmail, $poc, $poc_alt,
        $sStart, $sEnd, $sSubmit, $sStatus, 
        $dtLastScan, $slruntime, $slSerious, $slHigh, $sServerName ) = $sth_sel->fetchrow_array(  );

    $sth_sel->finish;

    if ( !$sId ) {
        die ("Failed to Connect to Database / Invalid record for subnet \"$subnet\n");
    } else {
        if ( $sName ) { $statistics .= "\t Address Space: $sName\n"; }
        if ( $sServerName ) { $statistics .= "\t Nessus Server: $sServerName\n"; }
        if ( $sCnt ) { $statistics .= "\t   Known Hosts: $sCnt\n"; }

         if ( $sStatus ) {
            switch ($sStatus) {
                case "C"        { $statistics .= "\t   Scan Status: Completed\n" }
                case "F"        { $statistics .= "\t   Scan Status: Failed\n" }
                case "H"        { $statistics .= "\t   Scan Status: Hold\n" }
                case "R"        { $statistics .= "\t   Scan Status: Running\n" }
                case "S"        { $statistics .= "\t   Scan Status: Scheduled\n" }
                else                { $statistics .= "\t   Scan Status: Staring [$sStatus]\n" }
            }  
        }

        #if ( $sSubmit ) { $statistics .= "\tTime Submitted: $sSubmit\n"; }
        if ( $sStart ) { $statistics .= "\t  Scan Started: $sStart\n"; }
        if ( $dtLastScan ) {
            $statistics .= "\n"
                        ."\t  Scan Details: \n"
                        ."\tDate Last Scan: $dtLastScan\n"
                        ."   Elapsed Scan Time: $slruntime\n"
                        ." Last Scan Ser Vulns: $slSerious\n"
                        ."Last Scan High Vulns: $slHigh\n";
        } else {
            $statistics .= "\n\t Scan Details: This is the first recorded scan against address space \"$subnet\"\n";
        }
    }

    $sth_sel->finish;

    #UPDATE MESSAGE VARIABLES WITH APPROPRIATE DATA
    $body =~ s/\[ADDRESS_SPACE\]/\t$subnet/;
    $body =~ s/\[STATISTICS\]/$statistics/;
    $body =~ s/\[EXCLUSIONS\]/$exclude_hosts/;
    my $org = $CONFIG{'siteORG'};
    $body =~ s/\[ORG\]/$org/g;
    my $company = $CONFIG{'company'};
    $body =~ s/\[COMPANY\]/$company/g;
    my $sig = $CONFIG{'mailSignature'};  
    $body =~ s/\[SIGNATURE\]/$sig/;  

    if ( $bAutoEmail) {
        if ( $poc ne "" ) { $email_addr = "$poc"; }
        if ( $poc_alt ne "" ) {
            if ( $email_addr ne "" ) { $email_addr .= ", "; }
            $email_addr .= "$poc_alt"; 
        }
        if ( $email_addr eq "" ) { $email_addr = "$CONFIG{'replyto'}"; }
    } else {
        $email_addr = "$CONFIG{'replyto'}";
    }

    #SEND EMAIL RESULTS AS PLAIN TEXT
    if ( $sCnt > 0 ) {
        print "Host Count = $sCnt - SENDING START EMAIL!\n";
        send_email( $email_addr, $CONFIG{'mailcc'}, "SOC - Automated Nessus Scan start for $subnet", "text", $body, "" );
    } else {
        print "Host Count = $sCnt - NO EMAIL!\n";
    }
}

#build notification to be sent out on subnet scan finish
sub cron_finish {
    # VER: 1.0 MODIFIED: 4/02/07 13:48
    my ( $scantime, $subnet ) = @_;
    my ( $msg, $email_addr, $html_data, $sth_sel, $sth_upd, $statistics, $total_vulns );

    #READ FORM EMAIL MESSAGE
    my $body = "";

    open (MESSAGE, "$messages_dir/cron_finish") || die "couldn't open the file!";

    while (my $line = <MESSAGE>) {
        $body .= $line;
    }

    close(MESSAGE);

    my $sql = qq{ SELECT t1.id, t1.CIDR, t1.cntHosts, t1.tiAutoEmail, t1.poc, t1.poc_alt, 
        t4.scan_START, t4.scan_END, t4.scan_SUBMIT, t4.status, 
        t5.iHostCnt, t5.iScantime, t5.vSerious, t5.vHigh, t5.vMed, t5.vLow, t5.vInfo, 
        t3.report_id, t3.report_key, t3.scantype
        FROM I3_subnets t1
        LEFT JOIN (
            SELECT fk_name, max( scantime ) AS currentscan
            FROM nessus_reports GROUP BY fk_name )t2 ON t2.fk_name = t1.CIDR
        LEFT JOIN nessus_reports t3 ON t2.currentscan = t3.scantime
        LEFT JOIN I3_jobs t4 ON t3.report_id = t4.report_id
        LEFT JOIN nessus_report_stats t5 ON t4.report_id = t5.report_id
        WHERE t1.CIDR = '$subnet' AND t4.status = 'C'
        LIMIT 1  };

    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my ( $sId, $sName, 
        $cHosts, $bAutoEmail, $poc, $poc_alt,
         $start_date, $end_date, $submit_date, $nstatus, 
        $scntHosts, $siScantime, $svSerious, $svHigh, $svMed, $svLow, $svInfo, 
        $report_id, $report_key, $report_type
         ) = $sth_sel->fetchrow_array(  );

    $sth_sel->finish;

    if ( !$sId ) {
        die ("Failed to Connect to Database / Invalid record for subnet \"$subnet\n");
    } else {

        if ( $sName ) {
            $statistics .= "\tAddress Space: $sName\n";
        }
        if ( $scntHosts && $scntHosts > 0) { #USE SCAN RESULTS
            # do nothing $scntHost is fine
        } elsif ( $cHosts && $cHosts > 0) {
            $scntHosts = $cHosts;
        } else {
            $scntHosts = 0;
        }
        $statistics .= "\t  Known Hosts: $scntHosts \n";

        if ( !defined($svSerious))  { $svSerious = 0;    };
        if ( !defined($svHigh))     { $svHigh = 0;       };
        if ( !defined($svMed))      { $svMed = 0;        };
        if ( !defined($svLow))      { $svLow = 0;        };
        if ( !defined($svInfo))     { $svInfo = 0;       };

        $total_vulns = $svSerious + $svHigh + $svMed + $svLow + $svInfo;

        if ( $total_vulns > 0 ) {

            my $report_url = "$CONFIG{'inprotect_url'}/respdf.php?scantime=$scantime&scantype=$report_type&key=$report_key";
            $statistics .= "\n"
                ."\t\t  Found: [ $total_vulns ] vulnerabilities\n"
                ."\t\tSerious: $svSerious\n"
                ."\t\t   High: $svHigh\n"
                ."\t\t Medium: $svMed\n"
                ."\t\t    Low: $svLow\n"
                ."\t\t   Info: $svInfo\n\n"
                    ."\tA full report can be downloaded from the following URL: $report_url .  "
                ."Additionally, a shorter report (Limited to Critical/High Vulnerabilities only) can be downloaded from the following URL: $report_url&critical=2 .  "
                ."This report is dynamically generated PDF and should be saved to disk for off-line viewing and archival purposes.\n";

        } else {
            if ( $scntHosts < 1 ) {

            }
            $statistics .= "\n\t Scan Details: No known vulnerabilities for address space \"$subnet\"\n";
        }

    }

    $sth_sel->finish;

    #UPDATE MESSAGE VARIABLES WITH APPROPRIATE DATA
    $body =~ s/\[ADDRESS_SPACE\]/\t$subnet/;
    $body =~ s/\[STATISTICS\]/$statistics/;

    $body =~ s/\[EXCLUSIONS\]/$exclude_hosts/;
    my $org = $CONFIG{'siteORG'};
    $body =~ s/\[ORG\]/$org/g;
    my $company = $CONFIG{'company'};
    $body =~ s/\[COMPANY\]/$company/g;
    my $sig = $CONFIG{'mailSignature'};  
    $body =~ s/\[SIGNATURE\]/$sig/;  

    if ( $bAutoEmail) {
        if ( $poc ne "" ) { $email_addr = "$poc"; }
        if ( $poc_alt ne "" ) {
            if ( $email_addr ne "" ) { $email_addr .= ", "; }
            $email_addr .= "$poc_alt"; 
        }
        if ( $email_addr eq "" ) { $email_addr = "$CONFIG{'replyto'}"; }

        $sql = qq{ UPDATE nessus_reports SET
            results_sent='1' WHERE report_id='$report_id' };
        #$sth_upd = $dbh->prepare( $sql );
        #$sth_upd->execute;

    } else {
        $email_addr = "$CONFIG{'replyto'}";
    }
 
    #SEND EMAIL RESULTS AS PLAIN TEXT

    if ( $total_vulns > 0 ) {
        print "Vuln Count = $total_vulns - SENDING COMPLETION EMAIL!\n";
            send_email( $email_addr, $CONFIG{'mailcc'}, "SOC - Automated Nessus Scan completed for $subnet", "text", $body, "" );
    } else {
        print "Vuln Count = $total_vulns - NO EMAIL!\n";
    }
}

#build notification to be sent out on all other jobs start
sub sched_start {
    # VER: 1.0 MODIFIED: 4/02/07 13:48
    my ( $job_id ) = @_;
    my ( $email_addr, $msg, $html_data, $sth_sel, $statistics, $jobname );

    #READ FORM EMAIL MESSAGE
    my $body = "";

    open (MESSAGE, "$messages_dir/sched_start") || die "couldn't open the file!";
    while (my $line = <MESSAGE>) {
        $body .= $line;
    }
    close(MESSAGE);

    my $sql = qq{ SELECT id, name, username, meth_TARGET, scan_SERVER, scan_START, status, notify 
        FROM I3_jobs WHERE id="$job_id" };

    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my ( $sId, $sName, $susername, $hosts, $sServerName, $sStart, $sStatus, $poc ) = $sth_sel->fetchrow_array(  );

    $sth_sel->finish;

    if ( !$sId ) {
        die ("Failed to Connect to Database / Invalid record for job \"$sName\n");
    } else {
        if ( $sName ) { $jobname .= "\t JOB NAME: $sName\n"; }
        if ( $sServerName ) { $statistics .= "\t Nessus Server: $sServerName\n"; }

         if ( $sStatus ) {
            switch ($sStatus) {
                case "C"        { $statistics .= "\t   Scan Status: Completed\n" }
                case "F"        { $statistics .= "\t   Scan Status: Failed\n" }
                case "H"        { $statistics .= "\t   Scan Status: Hold\n" }
                case "R"        { $statistics .= "\t   Scan Status: Running\n" }
                case "S"        { $statistics .= "\t   Scan Status: Scheduled\n" }
                else            { $statistics .= "\t   Scan Status: Staring [$sStatus]\n" }
            }  
        }

        if ( $sStart ) { $statistics .= "\t  Scan Started: $sStart\n"; }

    }

    $sth_sel->finish;

    if ( $susername eq "" ) { $susername=$poc; } 

    #UPDATE MESSAGE VARIABLES WITH APPROPRIATE DATA
    $body =~ s/\[USERNAME\]/\t$susername/;
    $body =~ s/\[JOBNAME\]/\tJOB NAME: $sName/;
    $body =~ s/\[HOSTS\]/\tHOSTS:\n\n$hosts/;
    $body =~ s/\[STATISTICS\]/$statistics/;
    $body =~ s/\[EXCLUSIONS\]/$exclude_hosts/;
    my $org = $CONFIG{'siteORG'};
    $body =~ s/\[ORG\]/$org/g;
    my $company = $CONFIG{'company'};
    $body =~ s/\[COMPANY\]/$company/g;
    my $sig = $CONFIG{'mailSignature'};  
    $body =~ s/\[SIGNATURE\]/$sig/;  

    if ( $poc ne "" ) { 
        $email_addr = "$poc";
    } else {
        $email_addr = "$CONFIG{'replyto'}";
    }

    #SEND EMAIL RESULTS AS PLAIN TEXT
    send_email( $email_addr, $CONFIG{'mailcc'}, "SOC - Automated Nessus Scan start job [ $sName ]", "text", $body, "" );

}

#build notification to be sent out on all other jobs finish
sub sched_finish {
    # VER: 1.0 MODIFIED: 4/02/07 13:48
    my ( $job_id ) = @_;
    my ( $msg, $email_addr, $html_data, $sth_sel, $sth_upd, $statistics );


    #READ FORM EMAIL MESSAGE
    my $body = "";

    open (MESSAGE, "$messages_dir/sched_finish") || die "couldn't open the file!";

    while (my $line = <MESSAGE>) {
        $body .= $line;
    }

    close(MESSAGE);

    my $sql = qq{ SELECT t1.id, t1.name, t3.scantime, t1.job_TYPE, t1.notify, t1.meth_TARGET, t1.scan_START, 
        t1.scan_END, t1.scan_SUBMIT, t1.status, t2.iHostCnt, 
        t2.iScantime, t2.vSerious, t2.vHigh, t2.vMed, t2.vLow, t2.vInfo, 
        t3.report_id, t3.report_key
        FROM I3_jobs t1
        LEFT JOIN nessus_report_stats t2 ON t1.report_id = t2.report_id and t2.dtLastScanned >= t1.scan_END
        LEFT JOIN nessus_reports t3 ON t2.report_id = t3.report_id
        WHERE t1.id = '$job_id'
        LIMIT 1  };

    logwriter( $sql, 5 );

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );

    my ( $sId, $sName, $scantime, $cJobType, $poc, $hosts, 
         $start_date, $end_date, $submit_date, $nstatus, 
        $scntHosts, $siScantime, $svSerious, $svHigh, $svMed, $svLow, $svInfo, 
        $report_id, $report_key
         ) = $sth_sel->fetchrow_array(  );

    $sth_sel->finish;

    if ( !$sId ) {
        die ("Failed to Connect to Database / Invalid job \"$sName\n");
    } else {

        if ( $scntHosts && $scntHosts > 0) { #USE SCAN RESULTS
            # do nothing $scntHost is fine
        } else {
            $scntHosts = 0;
        }
        $statistics .= "\t  Known Hosts: $scntHosts \n";

        if ( !defined($svSerious))  { $svSerious = 0;        };
        if ( !defined($svHigh))            { $svHigh = 0;        };
        if ( !defined($svMed))            { $svMed = 0;        };
        if ( !defined($svLow))            { $svLow = 0;        };
        if ( !defined($svInfo))            { $svInfo = 0;        };

        my $total_vulns = $svSerious + $svHigh + $svMed + $svLow + $svInfo;

        if ( $total_vulns > 0 ) {

            my $report_url = "$CONFIG{'inprotect_url'}/respdf.php?scantime=$scantime&scantype=$cJobType&key=$report_key";
            $statistics .= "\n"
                ."\t\t  Found: [ $total_vulns ] vulnerabilities\n"
                ."\t\tSerious: $svSerious\n"
                ."\t\t   High: $svHigh\n"
                ."\t\t Medium: $svMed\n"
                ."\t\t    Low: $svLow\n"
                ."\t\t   Info: $svInfo\n\n"
                    ."\tA full report can be downloaded from the following URL: $report_url .  "
                ."Additionally, a shorter report (Limited to Critical/High Vulnerabilities only) can be downloaded from the following URL: $report_url&critical=2 .  "
                ."This report is dynamically generated PDF and should be saved to disk for off-line viewing and archival purposes.\n";

        } else {
            if ( $scntHosts < 1 ) {

            }
            $statistics .= "\n\t Scan Details: No known vulnerabilities for job \"$sName\"\n";
        }

    }

    $sth_sel->finish;

    #UPDATE MESSAGE VARIABLES WITH APPROPRIATE DATA
    $body =~ s/\[JOBNAME\]/\t$sName/;
    $body =~ s/\[HOSTS\]/\tHOSTS:\n\n$hosts/;
    $body =~ s/\[STATISTICS\]/$statistics/;
    $body =~ s/\[EXCLUSIONS\]/$exclude_hosts/;
    my $org = $CONFIG{'siteORG'};
    $body =~ s/\[ORG\]/$org/g;
    my $company = $CONFIG{'company'};
    $body =~ s/\[COMPANY\]/$company/g;
    my $sig = $CONFIG{'mailSignature'};  
    $body =~ s/\[SIGNATURE\]/$sig/;  

    if ( $poc ne "" ) { 
        $email_addr = "$poc";
    } else {
        $email_addr = "$CONFIG{'replyto'}";
    }
 
    #SEND EMAIL RESULTS AS PLAIN TEXT
    send_email( $email_addr, $CONFIG{'mailcc'}, "SOC - Automated Nessus Scan completed for $sName", "text", $body, "" );

}

#sent the notification per the build notification process
sub send_email {
    # VER: 1.0 MODIFIED: 7/18/07 8:35
    my ( $to, $cc, $subject, $type, $message, $url ) = @_;

    logwriter( "to=$to\ncc=$cc\nfrom=" . $CONFIG{'mailfrom'} . "\nsub=" . $subject . "\n\nmsg=\n" . $message, 5 );

    if ( $type =~ /html/i ) {
        my $mailHTML = new MIME::Lite::HTML
            To      => $to,
            CC      => $cc,
            From    => $CONFIG{'mailfrom'},
            Subject => $subject;

        my $MIMEmail = $mailHTML->parse( $url );
        $MIMEmail->send;
    } else {
        my $msg = MIME::Lite->new(
            To      => $to,
            CC      => $cc,
            From    => $CONFIG{'mailfrom'},
            Subject => $subject,
            Data    => $message
        );
        $msg->send();
    }
}

#handle failed scans according to settings
sub reschedule_scan {
    my ( $job_id ) = @_;

    my $sql = qq{ UPDATE I3_jobs SET status='S', server_id='0', scan_START=NULL, scan_end=NULL,
            process_id='0' WHERE id='$job_id' };
    safe_db_write ( $sql, 1 );

}

#get server credential / port info from the db to setup connect to nessus node
sub get_server_credentials {
    # VER: 1.1 MODIFIED: 2/01/08 12:33
    my ( $select_id, $weight ) = @_;
  

    if ( ! is_number( $weight ) ) {
        $weight = 5;                #CRON JOBS USE 5 SLOTS PER SCAN
    }
 
    my ($sql, $sql1, $sth_sel, $sth_upd, $tmpserverid);

    $sql = qq{ SELECT id, max_scans, current_scans FROM nessus_servers 
        WHERE enabled='1' AND status='A' AND id='$select_id' };

    logwriter( $sql, 5 );
    
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;
    my ($tmp_serverid, $tmp_max, $tmp_current, $remaining);
    while(($tmp_serverid, $tmp_max, $tmp_current)=$sth_sel->fetchrow_array) {
        if ( is_number($tmp_serverid) && $tmp_max >= $weight ) {
            $remaining = $tmp_max - $tmp_current - $weight;        #REQUIRE 5 SLOTS FOR CRON SCAN
            if ( $remaining >= 1 ) {
                $sql = qq{ SELECT hostname, port, user, password FROM nessus_servers WHERE id=$tmp_serverid };
                logwriter( $sql, 5 );
                $sth_sel = $dbh->prepare( $sql );
                $sth_sel->execute;
                ($nessushost, $nessusport, $nessususer, $nessuspassword)=$sth_sel->fetchrow_array;
                $sth_sel->finish;
                return $tmp_serverid;
            }
        }
    }
    $sth_sel->finish;
    return 0;

}

#handle failed server nodes, to keep database accurate
sub move_server_offline {
    my ( $tmp_server_id ) = @_;

    my $sql = qq{ UPDATE nessus_servers SET max_scans='0', current_scans='0', enabled='0'
        WHERE id = '$tmp_server_id' LIMIT 1 };

    safe_db_write ( $sql, 2 );

    logwriter( "TAKING SERVER serverid=$serverid name=$nessushost OFFLINE DUE TO SCAN ERROR", 2 );
    logwriter( "\t$nessushost: PLEASE RESTART NESSUSD", 2 );

}

#check nessus up
sub check_connect {

    #Check if the Nessus scanners are up
    my ( $sql, $sql1, $sth, $sth1 );
    my($serverid,$n_hostname,$n_port,$tmp_current);
    my($socket, $connect);

    $sql = qq{ SELECT id, hostname, port, current_scans FROM nessus_servers
        WHERE enabled='1' AND checkin_time+interval 300 second<now() };

    logwriter( $sql, 5 );

    $sth=$dbh->prepare( $sql );
    $sth->execute;

    #Check status of the Nessus servers loop
    while(($serverid,$n_hostname, $n_port, $tmp_current )=$sth->fetchrow_array) {

        $sql1 = qq{ UPDATE nessus_servers SET status="C"
            WHERE id="$serverid" };

        safe_db_write ( $sql1, 5 );

        $connect=1;
        #Check if the server is alive
        #print localtime(time)." sched: Checking Nessus server status - $n_hostname:$n_port\n";
        $socket = IO::Socket::INET->new("$n_hostname:$n_port") or $connect=0;

        #If server is dead
        if ($connect==0) {
            logwriter( "\tsched: Nessus server OFFLINE - $n_hostname:$n_port", 2 );
            $sql1 = qq{ UPDATE nessus_servers SET status="N", checkin_time=now(), current_scans=0
                WHERE id="$serverid" };
            safe_db_write ( $sql1, 3 );

            $sql1 = qq{ UPDATE I3_jobs SET scan_SERVER=0, status="S", scan_START=NULL, scan_END=NULL
                WHERE scan_SERVER="$serverid" AND status="R" OR scan_SERVER="$serverid" AND status="S" };
            safe_db_write ( $sql1, 2 );
        } else {
            #print localtime(time)." sched: Nessus server ONLINE - $n_hostname:$n_port\n";
            close $socket;
            $sql1 = qq{ UPDATE nessus_servers SET status="A", checkin_time=now() WHERE id="$serverid" };
            safe_db_write ( $sql1, 5 );
        }

        if ( $tmp_current < 0 ) {
            $sql1 = qq{ UPDATE nessus_servers SET current_scans="0" WHERE id="$serverid" };
            safe_db_write ( $sql1, 3 );
        }
    } #End of loop
    #$sth->finish;
    #print "check_complete\n";
}

#import a scan run manually
sub manually_import_report {
    my ( $outfile, $targets, $job_type, $job_title ) = @_;

    my ( $sql, $sth_sel, $sth_ins, $Jvset, $juser);

    my @results = ( );

    $Jvset = 1;
    $juser = "Admin";
    if ( !$job_title ) { 
        $job_title = "CRON - $targets";
    }

    # SELECT OLDEST RECORD FROM CRON QUEUE

    if ( $job_type eq "C" ) {
        $sql = qq{ SELECT id, CIDR, site_code, location
            FROM I3_subnets WHERE CIDR='$targets' LIMIT 1};
        logwriter( $sql, 5 );
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );

        my ( $Lid, $Lname, $Lsite, $Llocation ) = $sth_sel->fetchrow_array(  );
        $sth_sel->finish;
        if ( ! $Lid ) {
            logwriter( "Invalid subnet specified [$targets].", 1 );
            return 0;
        }
    }

    $sql = qq{ SELECT id FROM I3_jobs WHERE status="R" AND name='$job_title' LIMIT 1};
    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    my ( $job_id ) = $sth_sel->fetchrow_array(  );

    if ( !$job_id ) {
        $sql = qq{ INSERT INTO I3_jobs ( name, username, job_TYPE, meth_TARGET, 
            meth_VSET, status, authorized, author_uname ) VALUES ( 
                '$job_title', '$juser', '$job_type', '$targets', 
            '$Jvset', 'I', '1', 'ACL' ); };
        logwriter( $sql, 4 );
        $sth_ins = $dbh->prepare( $sql );
        $sth_ins->execute(  );

        $sql = qq{ SELECT id
            FROM I3_jobs WHERE status="I" AND name='$job_title' LIMIT 1};
        logwriter( $sql, 5 );
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute(  );
        $job_id = $sth_sel->fetchrow_array(  );

        if ( ! $job_id ) { 
            logwriter( "ERROR Creating Job.", 4 );
            return 0;
        }
    }

    if ( @results != FALSE ) {        #SUCCESSFUL NESSUS RUN
        logwriter( "[$job_title] Begin SQL Import", 4 );

        my $scantime = getCurrentDateTime("datetime");

        if ( load_scanlite_results( \@results, $job_id, $job_title, $job_type, $juser, $Jvset, $scantime, $targets ) ){
            logwriter( "[$job_title] Completed SQL Import", 4 );

            $sql = qq{ UPDATE I3_jobs SET status='C' WHERE id='$job_id' };
            safe_db_write ( $sql, 4 );            #use insert/update routine   
        } else {
            logwriter( "[$job_title] Failed SQL Import", 2 );
        }
    }

    $sth_sel->finish;

}

#queue subnets for scanning ( typically used for monthly network audit feeds subnet/site/org/executive summary reports
sub queueWork {
    my ( $LanID, $sql_time ) = @_;

    my ($sql, $sql1, $sql_filter, $sth_sel, $sth_upd );

    if ( defined ( $LanID ) && $LanID ne "" ) {
        #$sql_filter = "AND t2.LanID = '$LanID'";
    }

    if ( !defined ( $sql_time ) || $sql_time eq "" ) {
        $sql_time = getCurrentDateTime("datetime");
    }
    $sql_filter .= "AND t4.scan_END < '$sql_time'";

    $sql = qq{ SELECT t1.id, t1.CIDR 
        FROM I3_subnets t1
        LEFT JOIN ( SELECT fk_name, max( scantime ) AS currentscan
            FROM nessus_reports        GROUP BY fk_name ) t2 ON t2.fk_name = t1.CIDR
        LEFT JOIN nessus_reports t3 on t2.currentscan = t3.scantime
        LEFT JOIN I3_jobs t4 ON t3.report_id = t4.report_id

        WHERE t1.status != 'available' 
            AND t4.status = 'C' 
            AND t1.tiScanApproval = '1' 
            AND t1.serial_flag = 'N'
            $sql_filter
            ORDER BY t4.scan_END DESC };

    logwriter($sql, 4 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ( $subID, $subnetName ) = $sth_sel->fetchrow_array) {
        if ( is_number( $subID ) && defined($subnetName) ) {

            $sql1 = qq{ INSERT INTO I3_jobs ( name , username , fk_name, job_TYPE, meth_SCHED , 
                meth_TARGET , meth_VSET , scan_SUBMIT , scan_PRIORITY , status , authorized, author_uname  
                ) VALUES (
                'CRON - $subnetName', 'Admin', '$subnetName', 'C', 'C', 
                '$subnetName', '1', '$sql_time', '3' , 'S', '1', 'ACL' ); };
            safe_db_write ( $sql1, 5 );            #use insert/update routine
            logwriter( "\tQueued subID: $subID\tNAME: $subnetName\n", 1 );
        }
    }

    $sth_sel->finish;

}

#clean failed jobs from jobs table
sub resetBackend {
    my ( $sql );
    print "WARNING: Inprotect Reset Issued!\n";

    $sql = qq{ UPDATE I3_jobs SET status = 'C', scan_END=scan_START WHERE status='R' };
    safe_db_write ( $sql, 4 );            #use insert/update routine

    $sql = qq{ UPDATE nessus_servers SET checkin_time=now(), current_scans=0 };
    safe_db_write ( $sql, 4 );            #use insert/update routine

}

#prep host list move cidr based lists to ip list to check for exceptions
sub build_hostlist {
    my ( $CIDR ) = @_;

    my ( $sql, $sth_sel );

    my $block = new Net::Netmask ( $CIDR );

    my $host_list;
    foreach my $ip ( $block->enumerate( ) ){
        $host_list .= "$ip\n";
    }

    #FILTER BASE & BROADCASE ADDRESS
    my $base = $block->base();
    my $bcast = $block->broadcast();
    $host_list =~ s/$base\n//g;
    $host_list =~ s/$bcast\n//g;

    $sql = qq{ SELECT ip_range FROM I3_scan_exclusions
        WHERE active = '1' ORDER BY ip_range };

    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $ex_CIDR )=$sth_sel->fetchrow_array ) {
        my $exBlock = new Net::Netmask ( $ex_CIDR );
            foreach my $ex_ip ( $exBlock->enumerate( ) ){
            if ( $host_list =~ /$ex_ip\n/ ) {
                $exclude_hosts .= "$ex_ip\n";
            }
            $host_list =~ s/$ex_ip\n//g;
        }

    }

    $sth_sel->finish;

    return $host_list;
}

#needed for host updated
sub extract_os {
    # VER: 1.0 MODIFIED: 11/15/07 15:43
    my ( $txt_msg ) = @_;
    my $os = "";
    my @arrMSG = split /\n/, $txt_msg;

    foreach my $line (@arrMSG) {
        $line =~ s/\#.*$//;
               chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line =~ "Remote operating system") {
            my @temp=split(/:/,$line,2);
            $temp[1] =~ s/\s+$//;
            $temp[1] =~ s/^\s+//;
            $os = $temp[1];
        } else { next; }
    }

    if ( defined( $os ) ) {
        logwriter( $os, 5 );
        return $os;
    } else {
        return "";
    }

}

#needed for host updated / load results
sub resolve_host {
    # VER: 2.0 MODIFIED: 5/06/08 15:30
    my ( $hostip ) = @_;

    if ( ! defined ( $hostip) || $hostip eq "" ) { return ""; }

    # ATTEMPT GET HOST BY ADDRESS WILL CHECK FILES/DNS PRIMARY
    my $iaddr = inet_aton( $hostip ); # or whatever address
    my $namer  = gethostbyaddr($iaddr, AF_INET);

    if ( defined($namer ) ) {
        my $thost = lc ( $namer );
        #logwriter( $thost, 5 );
        return $thost;
    } else {

	if ( $CONFIG{'nameservers'} ne "" ) {
	    my @nameservers = split /,/, $CONFIG{'nameservers'};
	    foreach my $nameserver ( @nameservers ) {
		$nameserver =~ s/\s+//g;
		my $namer = nslookup(host => "$hostip", type => "PTR", server => "$nameserver" );
		if ( defined($namer ) && $namer ne "" ) {
	            my $thost = lc ( $namer );
	    	return $thost;
	        }
	    }
	} 
    }
    logwriter( "REVERSE IP [$hostip] TO NAME FAILED\n", 3 );
    return "";

}

#ENSURE IP'S ARE IN THE BUILDLIST Otherwise Hostnames are returned instead IP's
#THEY GET STUFFED IN THE HOSTIP Field as a result
sub resolve_name2ip {
    # VER: 2.0 MODIFIED: 5/06/08 15:30
    my ( $hostname ) = @_;
    if ( ! defined ( $hostname ) || $hostname eq "" ) { return ""; }

    # ATTEMPT GET HOST BY ADDRESS WILL CHECK FILES/DNS PRIMARY
    my $packed_ip = gethostbyname( $hostname );

    if ( defined( $packed_ip ) ) {
        my $c_ip = inet_ntoa($packed_ip);
        return $c_ip;
    } else {
        #TRY OTHER NAMES SERVERS
	if ( $CONFIG{'nameservers'} ne "" ) {
	    my @nameservers = split /,/, $CONFIG{'nameservers'};

	    foreach my $nameserver ( @nameservers ) {
		$nameserver =~ s/\s+//g;
		my $namer = nslookup(host => "$hostname", server => "$nameserver" );
		if ( defined($namer ) && $namer ne "" ) {
	            my $thost = lc ( $namer );
	    	    return $thost;
	        }
	    }
	} 
    }
    logwriter( "RESOLVE [$hostname] TO IP FAILED\n", 3 );
    return "";
}

#needed for report stats
sub datediff {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $start_date, $end_date, $unit ) = @_;
    my ( %start, %end );

    if ( ! defined( $start_date ) || ! defined( $end_date ) ) {
        return  -1;
    }

    $start_date =~ s/\///g;
    $start_date =~ s/\s//g;
    $start_date =~ s/-//g;
    $start_date =~ s/://g;

    $start{YEAR} = substr($start_date, 0,4 );
    $start{MO} = substr($start_date, 4,2 );
    $start{D} = substr($start_date, 6,2 );
    $start{H} = substr($start_date, 8,2 );
    $start{M} = substr($start_date, 10,2 );
    $start{S} = substr($start_date, 12,2 );

    $end_date =~ s/\///g;
    $end_date =~ s/\s//g;
    $end_date =~ s/-//g;
    $end_date =~ s/://g;

    $end{YEAR} = substr($end_date, 0,4 );
    $end{MO} = substr($end_date, 4,2 );
    $end{D} = substr($end_date, 6,2 );
    $end{H} = substr($end_date, 8,2 );
    $end{M} = substr($end_date, 10,2 );
    $end{S} = substr($end_date, 12,2 );

    my ($Dd,$Dh,$Dm,$Ds) = Delta_DHMS($start{YEAR},$start{MO},$start{D}, $start{H},$start{M},$start{S},
        $end{YEAR},$end{MO},$end{D}, $end{H},$end{M},$end{S});

    my $diff = 0;

    if ( $Dd ) { $diff += ( $Dd * 216000 ); }
    if ( $Dh ) { $diff += ( $Dh * 3600 ); }
    if ( $Dm ) { $diff += ( $Dm * 60 ); }
    if ( $Ds ) { $diff += $Ds }

    if ( $unit eq "D" ) {
        $diff = ( $diff / 216000 );
    } elsif ( $unit eq "H" ) {
        $diff = ( $diff / 3600 );
    } elsif ( $unit eq "M" ) {
        $diff = ( $diff / 60 );
    } else {
        # Already seconds do not convert; 
    }

    $diff = sprintf("%.2f", $diff);

    return $diff;
}

#reusable routine to get subnet id from CIDR
sub getSubnetID {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $subnet ) = @_;
    my ($sql, $sth_sel, $sub_id);

    ### CHECK THE DATABASE FOR A MATCHING SUBNET
    $sql = qq{ SELECT id FROM I3_subnets WHERE Name='$subnet' LIMIT 1 };

    logwriter( $sql, 5 );
    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute(  );
    ### Retrieve the returned rows of data
    while ( my ( $id ) = $sth_sel->fetchrow_array(  ) ) {
        $sth_sel->finish;
        return $id;
    }
}

#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
