#!/usr/bin/perl
#
################################################################################
#				     Inprotect				      #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2004 Inprotect.com				      #
#									      #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.					      #
#									      #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.								      #
#									      #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA			      #
#									      #
# Contact Information:							      #
# Alex Zimin (alex@inprotect.com)					      #
# http://www.inprotect.com						      #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.							      #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.							      #
#									      #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.				      #
###############################################################################

use strict;
use warnings;
use MIME::Lite;
#use Date::Calc qw( Days_in_Month Day_of_Week Add_Delta_YMD);
use DBI;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
use Data::Dumper::Simple;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 1;

#Declare vars
my ($timenow,$wday,$year, $mon,$mday,$now);
my($day_of_week, $day_of_month, $time);
my ($dbh, $sth, $mailfrom);
my ($sth_nohost, $sth_ip); 

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

my $toemail = $CONFIG{'SCANSTATETOEMAIL'};
my $nostatus = $CONFIG{'NOSTATUS'};

#Redirect output to the log file
my $nessuslog = "$CONFIG{'UPDEXCLOG'}";
open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
if(!DEBUG) {
   *STDERR=*LOG;
   *STDOUT=*LOG;
}
print localtime(time)." hostCheck: Starting\n";

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." hostCheck: You must be a superuser to start this application\n";
    exit 1;
}


# Check if the program is currently running
if ( CheckRunning("checkHosts.pl") ) {
    print localtime(time)." hostCheck: Another instance is currently running\n";
    exit 2;
}


#Connecting to the database
$dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                        PrintError => 0,
                        RaiseError => 1,
                        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");


#Read some more config data 
$sth=$dbh->prepare(qq{
                       SELECT settingValue
                       FROM inprotect_settings
                       WHERE settingName = 'mailfrom'
                   });
$sth->execute();
($mailfrom) = $sth->fetchrow_array;

#Getting system time
my ($sec,$min,$hour,$yday,$isdst);
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
#my $now;
#my $timenow;
$mon++;
$now = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
$timenow = sprintf("%02d%02d%02d", $hour, $min, $sec);

# find all IPs in nessus_results that don't have an entry in hosts
#$sth = $dbh->prepare(qq{
#                         SELECT distinct(nr.host), h.status, h.hostname
#                         FROM nessus_results as nr
#                         LEFT JOIN hosts as h ON nr.host = h.ip
#                         WHERE ( h.ip is null
#                                 OR ( h.ip IS NOT NULL AND h.status 
#                                      IN ( $nostatus ) ) )
#                           AND ( nr.scriptid = '10180' 
#                                 AND nr.msg = 'The remote host is up' )
#                         ORDER BY nr.host
#                         });
# get all IPs from nessus_results that are "active"
$sth = $dbh->prepare(qq{
                        SELECT distinct(host)
                        FROM nessus_results
                        WHERE (scriptid = 10180 
                               AND msg = 'The remote host is up')
                           OR (scriptid = 12053)
                        ORDER BY host
                       });

my (%ipdata, $ip);
$sth->execute();
$sth->bind_columns(\$ip);
if(DEBUG) { 
   print "Checking for new hosts with scan results\n\n";
   print "IP           \tHostname\n";
   print "-------------\t---------------\n";
}
# prepare sql statements to get data from hosts table
$sth_nohost = $dbh->prepare(qq{
                               SELECT count(id)
                               FROM hosts
                               WHERE ip = ?
                                  AND status NOT IN ( $nostatus )
                              });

$sth_ip = $dbh->prepare(qq{
                           SELECT msg
                           FROM nessus_results
                           WHERE scantime = 
                              (SELECT max(scantime)
                               FROM nessus_results
                               WHERE host = ?)
                           AND host = ?
                           AND scriptid = 12053
                          });
while($sth->fetch()) {
   # check for an entry in hosts
   $sth_nohost->execute($ip);
   my($ipcount);
   $sth_nohost->bind_columns(\$ipcount);
   $sth_nohost->fetch();
   if($ipcount == 0) {
      # check for an IP in the msg of plugin 12053 otherwise get it from DNS
      $sth_ip->execute($ip, $ip);
      my($msg, $name);
      $sth_ip->bind_columns(\$msg);
      $sth_ip->fetch();
      $_ = $msg;
      if($msg && m/(.*) resolves as (.*)./) {
         $name = $2;
      } else {
         $name = gethostbyaddr(inet_aton($ip), AF_INET);
      }
      if(defined($name)) {
         my ($hostname,$domain,$tld) = split(/\./,$name);
         if(DEBUG) { print "$ip\t$hostname\n"; }
         $ipdata{$ip} = uc($hostname);
      }
   }
}
$sth_nohost->finish;
if(DEBUG) { warn Dumper(%ipdata);}

# set up statement to update the DB with the scan state
$sth=$dbh->prepare(qq{
                   INSERT INTO hosts
                      ( ip, hostname, status, createdate )
                   VALUES
                      ( ?, ?, ?, ? )
                   });

my $updatesExist = 0;
foreach $ip (keys %ipdata) {
   $updatesExist++;
   if(!DEBUG) {
      $sth->execute($ip,$ipdata{$ip},'AutoGenerated', $now) or die(localtime(time) . 
               " hostCheck: Unable to execute insert statement: " . $sth->errstr);
   }
}

if ($updatesExist >= 1 && !(DEBUG)) {
   my($mailSubject, $mailBody, $mailAttach);
   $mailSubject = "Host update status report for " . localtime();
   $mailBody = qq~
<html>
<head>
<style type="text/css">
table, td, th {
   border-width: 1px;
   border-style: solid;
   border-collapse: collapse;
}

th {
   font-weight: bold;
   background-color: #6797BF;
   color: #FFFFFF;
}
</style>
</head>
<body>
<p>The following new hosts were auto-created in the hosts table:</p>
<hr>
<table>
<tr>
   <th>IP</th>
   <th>Hostname</th>
</tr>
                 ~;

   foreach my $ip (keys %ipdata) {
      $mailBody .= qq{
<tr>
   <td>$ip</td>
   <td>$ipdata{$ip}</td>
</tr>
                     }; 
   }
   $mailBody .= qq{
   </table>
   </body>
   </html>
   };
   email_results($toemail, $mailfrom, $mailSubject, $mailBody);
}

print localtime(time)." hostCheck: $updatesExist update(s) made to hosts table.\n";

$sth->finish;

#Close DB connection
$dbh->disconnect or die(localtime(time) . " hostCheck: Failed to disconnect : $DBI::errstr\n");

print localtime(time)." hostCheck: Finish\n";
close LOG;

exit 0;

sub load_configs
{
    my $configfile="/usr/local/etc/inprotect.cfg";
    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." hostCheck: No inprotect.cfg file found.\n"; }
    foreach my $line (@CONFILE)
    {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}


sub email_results {

    my ( $mailto, $mailfrom, $msgSubject, $msgBody, ) = @_;
    my ( $msg );


    $msg = MIME::Lite->new(
               To      =>"$mailto",
               From    =>"$mailfrom",
               Subject =>"$msgSubject",
               Type    =>'text/html',
               Data    =>"$msgBody");

    $msg->send();
}


## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
        next if /grep|vi|vim|tail|$pid/;
        if ( /$pgname/ ) {
                $exists = 1;
                last;
            }
        }
    close PROC;
    return $exists;
}
