#!/usr/bin/perl
#
################################################################################
#				     Inprotect				      #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2004 Inprotect.com				      #
#									      #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.					      #
#									      #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.								      #
#									      #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA			      #
#									      #
# Contact Information:							      #
# http://www.inprotect.com						      #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.							      #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.							      #
#									      #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.				      #
###############################################################################

use strict;
use warnings;
use MIME::Lite;
#use Date::Calc qw( Days_in_Month Day_of_Week Add_Delta_YMD);
use DBI;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
use Data::Dumper::Simple;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 1;
use constant UPDATE => 1;

#Declare vars
my ($timenow,$wday,$year, $mon,$mday,$now);
my($day_of_week, $day_of_month, $time);
my ($dbh, $sth, $mailfrom);
my ($sth_nohost); 
my($i);

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

my $toemail = $CONFIG{'SCANSTATETOEMAIL'};
my $nostatus = $CONFIG{'NOSTATUS'};

#Redirect output to the log file
my $nessuslog = "$CONFIG{'UPDEXCLOG'}";
open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
if(!DEBUG) {
   *STDERR=*LOG;
   *STDOUT=*LOG;
}
print localtime(time)." genVulSummary: Starting\n";

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." genVulnSummary: You must be a superuser to start this application\n";
    exit 1;
}


# Check if the program is currently running
if ( CheckRunning("checkHosts.pl") ) {
    print localtime(time)." genVulnSummary: Another instance is currently running\n";
    exit 2;
}


#Connecting to the database
$dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                        PrintError => 0,
                        RaiseError => 1,
                        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");


#Read some more config data 
$sth=$dbh->prepare(qq{
                       SELECT settingValue
                       FROM inprotect_settings
                       WHERE settingName = 'mailfrom'
                   });
$sth->execute();
($mailfrom) = $sth->fetchrow_array;

#Getting system time
my ($sec,$min,$hour,$yday,$isdst);
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
#my $now;
#my $timenow;
$mon++;
$now = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
$timenow = sprintf("%02d%02d%02d", $hour, $min, $sec);

# find all IPs in nessus_results that don't have an entry in hosts
#$sth = $dbh->prepare(qq{
#                         SELECT distinct(nr.host), h.status, h.hostname
#                         FROM nessus_results as nr
#                         LEFT JOIN hosts as h ON nr.host = h.ip
#                         WHERE ( h.ip is null
#                                 OR ( h.ip IS NOT NULL AND h.status 
#                                      IN ( $nostatus ) ) )
#                           AND ( nr.scriptid = '10180' 
#                                 AND nr.msg = 'The remote host is up' )
#                         ORDER BY nr.host
#                         });
# get all IPs from nessus_results that are "active"
$sth = $dbh->prepare(qq{
                        SELECT distinct(host)
                        FROM nessus_results
                        ORDER BY host
                       });

#$sth = $dbh->prepare(qq{
#                        SELECT distinct(host)
#                        FROM nessus_results
#                        WHERE scriptid = 10180 
#                           AND msg = 'The remote host is up'
#                        ORDER BY host
#                       });

my (%ipdata, $ip);
$sth->execute();
$sth->bind_columns(\$ip);
if(DEBUG) { 
   print "Checking for new hosts with scan results\n\n";
   print "IP           \tHostname\n";
   print "-------------\t---------------\n";
}
# prepare sql statements to get data from nessus_results
my($sth_hostname);
$sth_hostname = $dbh->prepare(qq{
                               SELECT msg
                               FROM nessus_results
                               WHERE host = ?
                                  AND scantime = ?
                                  AND scriptid = 12053
                                  AND msg <> ""
                               LIMIT 1
                              });
# sql query to get the scantimes for an IP
my($sth_scantime);
$sth_scantime = $dbh->prepare(qq{
                                 SELECT distinct scantime
                                 FROM nessus_results
                                 WHERE host = ?
                                 });
# sql query to get the vuln summary for an IP and scantime
my($sth_scanresults);
$sth_scanresults = $dbh->prepare(qq{
                                   SELECT risk, count(risk) as vcount
                                   FROM nessus_results
                                   WHERE host = ? AND scantime = ?
                                   GROUP BY risk
                                   });
while($sth->fetch()) {
   if(DEBUG) { print "Processing $ip - "; }
   my ($scancount) = 0; # keep track of how many times this IP was scanned
   # now get the scantimes for the IP
   $sth_scantime->execute($ip);
   my($scantime);
   $sth_scantime->bind_columns(\$scantime);
   while($sth_scantime->fetch()) {
      $scancount++;
      # get the hostname
      $sth_hostname->execute($ip,$scantime);
      my($msg, $fqdn);
      $sth_hostname->bind_columns(\$msg);
      $sth_hostname->fetch();
      # look for it in the scan results first, then do a DNS lookup 
      if(!defined($msg)) {
         $fqdn = gethostbyaddr(inet_aton($ip), AF_INET);
         if(defined($fqdn)) {
            my ($hostname,$domain,$tld) = split(/\./,$fqdn);
            if($domain ne "putnaminv" && $domain ne "putnam") { $hostname = $fqdn; }
            $ipdata{$ip}{$scantime}{'hostname'} = uc($hostname);
         } else {
            # hmm, not hostname, set to 'unknown'
            $ipdata{$ip}{$scantime}{'hostname'} = 'unknown';
         }
      } else {
         $_ = $msg;
         ($fqdn) = /\d+\.\d+\.\d+\.\d+ resolves as (.*)\./;
         # split it up into hostname and domain
         my($hostname,$domain,$tld) = split(/\./,$fqdn);
         # some of the IPs may be for web sites, and not server names
         if($domain ne "putnaminv" && $domain ne "putnam") { $hostname = $fqdn; }
         $ipdata{$ip}{$scantime}{'hostname'} = uc($hostname);
      }
      # initialize the risk counts to 0
      for($i=1;$i<=7;$i++) {
         $ipdata{$ip}{$scantime}{$i} = 0;
      }
      # now get the vuln scan results for the scan
      $sth_scanresults->execute($ip,$scantime);
      my($risk, $count);
      $sth_scanresults->bind_columns(\$risk,\$count);
      while($sth_scanresults->fetch()) {
         $ipdata{$ip}{$scantime}{$risk} = $count;
      }
   }
   if(DEBUG) { print " found $scancount result sets\n"; }
}
# finish with the statement handles to free up the memory
$sth_hostname->finish;
$sth_scantime->finish;
$sth_scanresults->finish;

if(DEBUG) { warn Dumper(%ipdata);}

# set up statement to update the DB with the scan state
$sth=$dbh->prepare(qq{
                   INSERT INTO vuln_summary 
                      ( `ip`, `scantime`, `hostname`, `1`, `2`, `3`, 
                        `4`, `5`, `6`, `7` )
                   VALUES
                      ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )
                   });

if(UPDATE) {
   foreach $ip (keys %ipdata) {
      my($scantime);
      foreach $scantime (keys %{$ipdata{$ip}}) {
         $sth->execute($ip, 
                       $scantime,
                       $ipdata{$ip}{$scantime}{'hostname'},
                       $ipdata{$ip}{$scantime}{'1'},
                       $ipdata{$ip}{$scantime}{'2'},
                       $ipdata{$ip}{$scantime}{'3'},
                       $ipdata{$ip}{$scantime}{'4'},
                       $ipdata{$ip}{$scantime}{'5'},
                       $ipdata{$ip}{$scantime}{'6'},
                       $ipdata{$ip}{$scantime}{'7'}) 
            or die(localtime(time) . 
                   " genVulnSummary: Unable to execute insert statement: " . 
                   $sth->errstr);
      }
   }
}

if (UPDATE) {
   my($mailSubject, $mailBody, $mailAttach);
   $mailSubject = "Vuln Summary update status report for " . localtime();
   $mailBody = "
<html>
<head>
<style type='text/css'>
table, td, th {
   border-width: 1px;
   border-style: solid;
   border-collapse: collapse;
}

th {
   font-weight: bold;
   background-color: #6797BF;
   color: #FFFFFF;
}
</style>
</head>
<body>
<table>
<tr>
   <th>IP</th>
   <th>Hostname</th>
   <th>Vuln Counts</th>
</tr>
                 ";

   my($risktab);
   foreach my $ip (keys %ipdata) {
      for ($i = 0; $i <=7; $i++) {
         if(defined($ipdata{$ip}{$i})) {
            $risktab .= "
$i = $ipdata{$ip}{$i}<br>
                         ";
         }
      }
      $mailBody .= "
<tr>
   <td>$ip</td>
   <td>$ipdata{$ip}{'hostname'}</td>
   <td>$risktab</td>
</tr>
                     "; 
   }
   $mailBody .= "
   </table>
   </body>
   </html>
   ";
   if(DEBUG) { $toemail = 'hhendrickson@putnam.com'; }
   email_results($toemail, $mailfrom, $mailSubject, $mailBody);
}

if(UPDATE) {
   print localtime(time)." genVulnSummary: vuln_summary table updated.\n";
}

$sth->finish;

#Close DB connection
$dbh->disconnect or die(localtime(time) . " genVulnSummary: Failed to disconnect : $DBI::errstr\n");

print localtime(time)." genVulnSummary: Finish\n";
close LOG;

exit 0;

sub load_configs
{
    my $configfile="/usr/local/etc/inprotect.cfg";
    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." genVulnSummary: No inprotect.cfg file found.\n"; }
    foreach my $line (@CONFILE)
    {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}


sub email_results {

    my ( $mailto, $mailfrom, $msgSubject, $msgBody, ) = @_;
    my ( $msg );


    $msg = MIME::Lite->new(
               To      =>"$mailto",
               From    =>"$mailfrom",
               Subject =>"$msgSubject",
               Type    =>'text/html',
               Data    =>"$msgBody");

    $msg->send();
}


## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
        next if /grep|vi|vim|tail|$pid/;
        if ( /$pgname/ ) {
                $exists = 1;
                last;
            }
        }
    close PROC;
    return $exists;
}
