#!/usr/bin/perl
#

use strict;
use warnings;

 #Use Perl modules
use Net::IP;
use Net::DNS;
use DBI;


my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

#Redirect output to the log file
my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/fix1.log";

my $debug           = 0;
my $log_level       = 4;
my $track_progress  = 0;
my $time_to_die     = 0;
my $use_scanlite    = 0;
my $notify_by_email = 1;

#Declare variables to use

my ( $dbh );

main( );
		
sub main {
    my ( $dbh, $sql,$sql2, $sth_sel, $sth_sel2, $sth_upd );

    $dbh = conn_db();

        open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
            *STDERR=*LOG;
            *STDOUT=*LOG;

    print "starting..\n";

    $sql = qq{ SELECT id, hostname FROM hosts };

    $sth_sel = $dbh->prepare( $sql );
    $sth_sel->execute;

    while( my ($host_id, $hostname )=$sth_sel->fetchrow_array) {

	if ( $hostname =~ /\./ ) {

	    print "valid fqdn [$hostname]\n";


	} else {
      
		$sql = qq{ SELECT count(hostname) FROM hosts WHERE hostname LIKE '%$hostname%' };
	        print "query=[$sql]\n";
	        $sth_sel2 = $dbh->prepare( $sql );
	        $sth_sel2->execute;
	        my ( $count ) = $sth_sel2->fetchrow_array ( );

	        if ( $hostname ne "" && $count > 1 ) {		
		    print "REMOVING THIS DUPLICATE HOST: [ $hostname ]\n";

		    $sql = qq{ DELETE FROM hosts WHERE id='$host_id' };		
		    safe_db_write ( $sql, 4 );

			
    		    $sql = qq{ SELECT id FROM hosts WHERE hostname LIKE '%$hostname%' AND id !='$host_id' LIMIT 1 };
	            $sth_sel2 = $dbh->prepare( $sql );
	            $sth_sel2->execute;
		    my ( $replace_id  ) = $sth_sel2->fetchrow_array ( );
    	
       		    if ( $replace_id ) {
       		
      			$sql = qq{ UPDATE host_macs SET host_id='$replace_id' WHERE host_id='$host_id'};
			safe_db_write ( $sql, 4 );
			
    	  		$sql = qq{ UPDATE host_stats SET host_id='$replace_id' WHERE host_id='$host_id'};
			safe_db_write ( $sql, 4 );
			
      			$sql = qq{ UPDATE I3_Incidents SET host_id='$replace_id' WHERE host_id='$host_id'};
			safe_db_write ( $sql, 4 );
       		    }	
		}

	}
    }
}


#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}


#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
