#!/usr/bin/perl
#
################################################################################
#				     Inprotect				      #
#-----------------------------------------------------------------------------#
# Copyright (C) 2002 - 2004 Inprotect.com				      #
#									      #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.					      #
#									      #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.								      #
#									      #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA			      #
#									      #
# Contact Information:							      #
# http://www.inprotect.com						      #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.							      #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.							      #
#									      #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.				      #
###############################################################################

use strict;
use warnings;
use MIME::Lite;
use Date::Calc qw( Days_in_Month Day_of_Week Add_Delta_YMD);
use DBI;
use Socket;
use POSIX qw(setsid);
use IO::Socket;
#use Data::Dumper::Simple;

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;
use constant DEBUG => 0;

#Declare vars
my ($timenow,$wday,$year, $mon,$mday,$now);
my($day_of_week, $day_of_month, $time);
my ($dbh, $sth, $mailfrom);

#Read settings from the inprotect.cfg file
use vars qw/%CONFIG/;
load_configs();

my $maxScanAge = $CONFIG{'MAXSCANAGE'};
my $toemail = $CONFIG{'SCANSTATETOEMAIL'};
#my @nostatus = split(/,/,$CONFIG{'NOSTATUS'});
my $nostatus = $CONFIG{'NOSTATUS'};

#Redirect output to the log file
my $nessuslog = "$CONFIG{'UPDEXCLOG'}";
open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
if(!DEBUG) {
   *STDERR=*LOG;
   *STDOUT=*LOG;
}
print localtime(time)." updexc: Starting\n";

# Only user root can run this program 
if ( $> != 0 ) {
    print localtime(time)." updexc: You must be a superuser to start this application\n";
    exit 1;
}


# Check if the program is currently running
if ( CheckRunning("updateExceptions.pl") ) {
    print localtime(time)." updexc: Another instance is currently running\n";
    exit 2;
}


#Connecting to the database
$dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}","$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
                        PrintError => 0,
                        RaiseError => 1,
                        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");


#Read some more config data 
$sth=$dbh->prepare(qq{
                       SELECT settingValue
                       FROM inprotect_settings
                       WHERE settingName = 'mailfrom'
                   });
$sth->execute();
($mailfrom)=$sth->fetchrow_array;

#Getting system time
my ($sec,$min,$hour,$yday,$isdst);
($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
#my $now;
#my $timenow;
$mon++;
$now = sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
$timenow = sprintf("%02d%02d%02d", $hour, $min, $sec);

# get the exceptions and put into a hash for later reference
my $sth4 = $dbh->prepare(qq{
                         SELECT eid, ip, plugin, status, port
                         FROM exceptions
                         });

my (@ehash, %e, %ed);
my (%edata);
my ($eid, $eip, $eplugin, $estatus, $eport);
$sth4->execute();
$sth4->bind_columns(\$eid, \$eip, \$eplugin, \$estatus, \$eport);
while($sth4->fetch()) {
   if(!defined($eport)) {
      $eport = "";
   }
   # handle case of multiple IPs
   my(@mips) = split(/,/,$eip);
   foreach(@mips) {
      $edata{$_}{$eplugin} = { 'eid' => $eid,
                               'status' => $estatus,
                               'eport' => $eport };
   }
}
#warn Dumper(%edata);

# get all hosts and the current scanstate
my $nocount = 0;
my $nswc = " h.status NOT IN ($nostatus) ";
$sth = $dbh->prepare(qq{
                         SELECT distinct h.ip, h.hostname, h.scanstate,
                                nr.scantime
                         FROM hosts as h
                         LEFT JOIN nessus_results as nr ON h.ip = nr.host
                         WHERE $nswc AND ( nr.scantime = 
                            (SELECT max(scantime) FROM nessus_results
                             WHERE host = nr.host) OR h.scanstate = 'Pending Scan' )
                         ORDER BY h.hostname
                     } );
$sth->execute();
#my (@hdata) = $sth->fetchall_hashref('ip');
my ($ip, $hostname, $scanstate, $mst);
$sth->bind_columns(\$ip, \$hostname, \$scanstate, \$mst );

# get the date of the latest scan (max(scantime)) for a given host
my $sth2 = $dbh->prepare(qq{
                         SELECT max(scantime)
                         FROM nessus_results
                         WHERE host = ?
                         } );

#my $sth3 = $dbh->prepare(qq{
#                         SELECT nr.scriptid, e.status
#                         FROM nessus_results as nr
#                         LEFT JOIN exceptions as e
#                         ON nr.scriptid=e.plugin
#                         WHERE nr.scantime = ? AND
#                            (nr.host = ? OR e.ip = 'ALL')
#                         });

# query to get the vuln results for a given host and scantime
# excludes any Info (risk = 7) results
my $sth3 = $dbh->prepare(qq{
                         SELECT scriptid, service, msg
                         FROM nessus_results
                         WHERE scantime = ? 
                            AND host = ? 
                         });
#                            AND ( risk <> 7 OR scriptid = 10180)

# set up hash for final results
my (%finalhash);

while ($sth->fetch() ) {
   # see if there are any scan results for this host
#   $sth2->execute($ip);
   $finalhash{$ip}{'hostname'} = $hostname;
#   my ($mst);
#   $sth2->bind_columns(\$mst);
#   $sth2->fetch();
   if(defined($mst)) {
      $finalhash{$ip}{'mst'} = $mst;
      my $recentScan = check_scan_date($mst,$maxScanAge);
      my($openIssues) = 0;
      my($excPend) = 0;
      # check the current scanstate
      if ($scanstate eq "Passed") {
        # IP has state of passed, but is it recent enough to still be passed?
        if (!$recentScan) {
           $finalhash{$ip}{'scanstate'} = "Scan Out of Date";
        } else {
           $finalhash{$ip}{'scanstate'} = $scanstate;
        }
      } elsif($scanstate eq "Fixes Applied - Pending ReScan") {
#      if($scanstate eq "Fixes Applied - Pending ReScan") {
         # if it's indicated that Fixes are applied, leave it be
         # unless the last scan was more than MST days ago
         if(!$recentScan) {
            $finalhash{$ip}{'scanstate'} = "Scan Out of Date";
         }
      } else {
         my ($deadhost) = 0;
         if($recentScan) {
            # see if we have open issues (sid = plugin)
            my ($sid, $service, $port, $msg);
            $sth3->execute($mst,$ip);
            $sth3->bind_columns(\$sid, \$service, \$msg);
            while( $sth3->fetch() ) {
               # split out the port
               if(defined($service) && $service =~ m/\((.*)\/(.*)\)/) {
                  $port = $1;
               #} else {
               #   $port = "";
               }
               if(defined($sid)) {
                  # check for the dead-host plugin result
                  if($sid == "10180" && (index($msg,"dead") > 0)) {
                     $deadhost = 1;
                  } else {
                     # scenarios to check for
                     #   ip + port + sid
                     #   ip + _blank + sid
                     #   ALL + port + sid
                     #   ALL + _blank + sid
                     # exception for specific IP?
                     if (defined($edata{$ip}{$sid}{'status'}) &&
                         defined($edata{$ip}{$sid}{'port'})) { 
                        if($edata{$ip}{$sid}{'port'} eq $port ||
                           $edata{$ip}{$sid}{'port'} == '') { 
                           if($edata{$ip}{$sid}{'status'} eq "Pending") { 
                              $excPend++; 
                           }
                           $finalhash{$ip}{$sid} = $edata{$ip}{$sid}{'status'};
                        } else {
                           $finalhash{$ip}{$sid} = "Open";
                           $openIssues++;
                        }
                     # how about for IP = ALL?
                     } elsif (defined($edata{'ALL'}{$sid}{'status'}) &&
                              defined($edata{'ALL'}{$sid}{'port'})) {
                        if($edata{'ALL'}{$sid}{'port'} == $port ||
                           $edata{'ALL'}{$sid}{'port'} == '') {
                           if($edata{'ALL'}{$sid}{'status'} eq "Pending") { 
                              $excPend++; 
                           }
                           $finalhash{$ip}{$sid} = $edata{'ANY'}{$sid}{'status'};
                        } else {
                           $finalhash{$ip}{$sid} = "Open";
                           $openIssues++;
                        }
                     # nope, flag as open issue
                     } else {
                        $finalhash{$ip}{$sid} = "Open";
                        $openIssues++;
                     }
                  }
               }
            }
            if($deadhost == 1) {
               # host was marked dead on last scan, set it to Pending Scan
               $finalhash{$ip}{'scanstate'} = "Dead Host";
            } else {
               # any open issues, and the IP is flagged "Pending Fixes"
               if($openIssues >= 1) {
                  $finalhash{$ip}{'scanstate'} = "Pending Fixes";
               # no open issues, but issues with pending exceptions?
               } elsif($excPend >= 1) {
                  $finalhash{$ip}{'scanstate'} = "Pending Exceptions";
               # otherwise IP passed
               } else {
                  # changed this to be manually assigned, can't construct
                  # the logic to handle cases where scans are run without
                  # the full vuln profile, causing host to not have full
                  # vuln results.  So for now, just leave the current
                  # scanstate as is.
                  #$finalhash{$ip}{'scanstate'} = "Passed";
                  $finalhash{$ip}{'scanstate'} = $scanstate;
               }
            }
         } else {
            $finalhash{$ip}{'scanstate'} = "Scan Out of Date";
         }
      }
   } else {
      $finalhash{$ip}{'mst'} = "Not Scanned";
      $finalhash{$ip}{'scanstate'} = "Pending Scan";
   }
   if(DEBUG) {
      print "$ip - $finalhash{$ip}{'mst'} - $finalhash{$ip}{'scanstate'}\n";
   }
}

#warn Dumper(%finalhash);
# set up statement to update the DB with the scan state
$sth=$dbh->prepare(qq{
                   UPDATE hosts
                   SET scanstate = ?,
                       lastscandate = ?
                   WHERE ip = ? AND hostname = ?
                   });

if(!DEBUG) {
   foreach $ip (keys %finalhash) {
      $sth->execute($finalhash{$ip}{'scanstate'}, $finalhash{$ip}{'mst'},
                    $ip, $finalhash{$ip}{'hostname'})
         or die(localtime(time) . " updexc: Unable to execute update statement: " . $sth->errstr);
   }
}

my($mailSubject, $mailBody, $mailAttach);
$mailSubject = "Host status report for " . localtime();
#$mailBody .= "==============================================\n\n";
$mailBody = qq~
<html>
<head>
<style type="text/css">
table, td, th {
   border-width: 1px;
   border-style: solid;
   border-collapse: collapse;
}

th {
   font-weight: bold;
   background-color: #6797BF;
   color: #FFFFFF;
}
</style>
</head>
<body>
~;

#$mailAttach = qq{
#<table border=1>
#<tr>
#   <th>IP</th>
#   <th>Hostname</th>
#   <th>Scan State</th>
#   <th>Last Scan</th>
#</tr>
#};
my $filename = "hostScanState_$now.xls";
##$mailAttach = "Hostname,IP,Scan State, Last Scan\n";
#
my (%stateArr); # for creating the summary based on state
foreach my $ip(keys %finalhash) {
   $stateArr{$finalhash{$ip}{'scanstate'}}++;
   my $tmpmst = $finalhash{$ip}{'mst'};
   my $lsd = $tmpmst;
   if($tmpmst =~ m/\d+/) { # if mst is a number
      $lsd = substr($tmpmst,4,2) . "/" . substr($tmpmst,6,2) . "/" . substr($tmpmst,0,4) . " " . substr($tmpmst,8,6);
   }
#   $mailAttach .= qq{
#<tr>
#   <td>$ip</td>
#   <td>$finalhash{$ip}{'hostname'}</td>
#   <td>$finalhash{$ip}{'scanstate'}</td>
#   <td>$lsd</td>
#</tr>
#   }; 
##   $mailAttach .= "$hostname,$finalhash{$hostname}{'ip'},$finalhash{$hostname}{'scanstate'},\"$finalhash{$hostname}{'mst'}\"\n";
}
$mailBody .= qq{
<table>
<tr>
   <th>Scan State</th>
   <th># Unique IPs</th>
</tr>
};
my ($hostTotal) = 0;
foreach my $state (keys %stateArr) {
   my $sst = $stateArr{$state};
   $hostTotal += $sst;
   if($state eq "Scan Out of Date") { 
      $state = "Scan Out of Date (>$maxScanAge days)";
   }
   $mailBody .= "<tr><td>$state</td><td>$sst</td></tr>";
}
$mailBody .= "<tr><td>Total Hosts</td><td>$hostTotal</td></tr>";
$mailBody .= qq{
</table>
</body>
</html>
};
#<p>See <a href="cid:$filename">$filename</a> for details.</p>
if(!DEBUG) {
   email_results($toemail, $mailfrom, $mailSubject, $mailBody, $mailAttach, $filename);
}

$sth->finish;
$sth2->finish;
$sth3->finish;
$sth4->finish;

#Close DB connection
$dbh->disconnect or die(localtime(time) . " updexc: Failed to disconnect : $DBI::errstr\n");

print localtime(time)." updexc: Finish\n";
close LOG;

exit 0;

sub check_scan_date {
   my($mst,$maxScanAge) = @_;
   # see if the latest scantime wasn't more than $maxScanAge days old
   my ($scanYear,$scanMon,$scanDay,$scanTime);
   $scanYear = substr($mst,0,4);
   my $scanYr = substr($scanYear,2,2);
   $scanMon = substr($mst,4,2);
   $scanDay = substr($mst,6,2);
   $scanTime = substr($mst,8,6); #HHMMSS
   my $MSAsecEpoch = `date -d '-$maxScanAge days' +%s`;
   chomp $MSAsecEpoch;
   my $scanSecEpoch = `date -d $scanYr$scanMon$scanDay +%s`;
   chomp $scanSecEpoch;
#   print " $mst - $MSAsecEpoch - $scanSecEpoch ";
   if($scanSecEpoch >= $MSAsecEpoch) {
      # scan is recent enough
      return 1;
   } else {
      # scan is older than the maxScanAge
      return 0;
   }
}

sub load_configs
{
    my $configfile="/usr/local/etc/inprotect.cfg";
    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." updexc: No inprotect.cfg file found.\n"; }
    foreach my $line (@CONFILE)
    {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}


sub email_results {

    my ( $mailto, $mailfrom, $msgSubject, $msgBody, 
         $msgAttach, $filename ) = @_;
    my ( $msg );


    $msg = MIME::Lite->new(
               To      =>"$mailto",
               From    =>"$mailfrom",
               Subject =>"$msgSubject",
               Type    =>"multipart/mixed");
    $msg->attach(Type    =>'text/html',
                 Data    =>"$msgBody");
#    $msg->attach(Type         =>'application/vnd.ms-excel',
#                 Id           =>"$filename",
#                 Filename     =>"$filename",
#                 Disposition  =>'attachment',
#                 Data         =>"$msgAttach");

    $msg->send();
}

## Checks entries in the process table for the program 
## Returns true if entry is found 
sub CheckRunning {
    my ($pgname) = @_;
    my ($exists, $pid);
    $pid = $$;
    open(PROC,"ps -eo '%p %a'|") or die "Failed to fork : $!\n";

    $exists = 0;
    while(<PROC>) {
        next if /grep|vi|vim|tail|$pid/;
        if ( /$pgname/ ) {
                $exists = 1;
                last;
            }
        }
    close PROC;
    return $exists;
}
