#!/usr/bin/perl
#
###############################################################################
#                          Last update: 08.15.2008                            #
#-----------------------------------------------------------------------------#
#                                 Inprotect                                   #
#-----------------------------------------------------------------------------#
# Copyright (C) 2008 Inprotect.net                                            #
#                                                                             #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of version 2 of the GNU General Public License as published #
# by the Free Software Foundation.                                            #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc.,     #
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                       #
#                                                                             #
# Contact Information:                                                        #
# inprotect-devel@lists.sourceforge.net                                       #
# http://www.inprotect.net                                                    #
###############################################################################
# See the README.txt and/or help files for more information on how to use &   #
# configuration.                                                              #
# See the LICENSE.txt file for more information on the License this software  #
# is distributed under.                                                       #
#                                                                             #
# This program is intended for use in an authorized manner only, and the      #
# author can not be held liable for anything done with this program, code, or #
# items discovered with this program's use.                                   #
###############################################################################
##
##    PRGM AUTHOR: kkline
##   PROGRAM NAME: archive_report
##   PROGRAM DATE: 08/15/2008
##  PROGM VERSION: 0.5
##  REVISION HIST:
##       08/01/2008 - SCRIPT TO GEN .nessus FILES COMPAT WITH SECURITY CENTER
##
use strict; 

# PERL MODULES WE WILL BE USING
use DBI;
use DBD::mysql;
use Date::Calc qw( Delta_DHMS Add_Delta_YMD );
use Getopt::Std;
use Switch;

my %loginfo;         # plot information hash
   $loginfo{'1'} = "FATAL";
   $loginfo{'2'} = "ERROR";
   $loginfo{'3'} = "WARN";
   $loginfo{'4'} = "INFO";
   $loginfo{'5'} = "DEBUG";

use vars qw/%CONFIG/;

&load_configs("/etc/inprotect.cfg");

my $nessuslog = "$CONFIG{'ROOTDIR'}/var/log/archive.log";
my $datadir = "$CONFIG{'ROOTDIR'}/var/lib/Inprotect/nessus";

#Declare constants
use constant TRUE => 1;
use constant FALSE => 0;

my $debug	    = 0;
my $log_level	    = 4;
my @nessus_plugins;

my ( $dbh );    	    #DATABASE HANDLE TO BE USED THROUGHOUT PROGRAM

my %options=();
getopts("r:h?",\%options);

main( );
exit;

sub main {

   my $report_id = "";

   if( $options{d} || $debug eq "1" ) {         		    #ENABLE DEBUGGING
      use warnings;
      print "Debugging mode\n";
      $debug = 1;
      $log_level = 5;
   }

   if ( $options{r} ) { $report_id = $options{r}; } else { die("Missing required argument!\n"); }


#  open(LOG,">>$nessuslog") or die "Failed to create $nessuslog: $!\n";
#        *STDERR=*LOG;
#        *STDOUT=*LOG;

   $dbh = conn_db();
   #USE Front End Variables
   load_db_configs ( );

   #UPDATE INCIDENT TRACKER WITH EXCEPTIONS STATUS
   archive_report ( $report_id );


   $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");

}

sub  archive_report {
   my ( $rep_id ) = @_;

   my $scandata = "";

   $scandata = "<NessusClientData>\n";
   $scandata .= build_target_section( $rep_id );
   $scandata .= build_policy_section( $rep_id );
   $scandata .= build_report_section( $rep_id );
   #print "$scandata";

   my $sql = qq{ SELECT t1.report_id, t2.name, DATE_FORMAT(t1.scantime, '%Y%m%d')
      FROM nessus_reports t1
        LEFT JOIN nessus_report_stats t2 on t1.report_id=t2.report_id
        WHERE t1.report_id = '$rep_id' };
   my $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   my ( $report_id, $rname, $scantime  ) = $sth_sel->fetchrow_array(  );
   my $subTemp = $scantime."_".$rname;

   $subTemp =~ s/\//_/g;
   $subTemp =~ s/\s-\s/_/g;
   $subTemp =~ s/\s+/_/g;
   $subTemp =~ s/\./_/g;

   my $year = substr($scantime,0,4);
   my $month= substr($scantime,4,2);

   if( ! -e "${datadir}/$year" ) { mkdir ( "${datadir}/$year" ); }
   if( ! -e "${datadir}/$year/$month" ) { mkdir ( "${datadir}/$year/$month" ); }

   my $datafile = "${datadir}/$year/$month/$subTemp.nessus";

   unlink $datafile if -e $datafile;            # REMOVE ANY PRE-EXISTING Results File

   open(NESSUS,">>$datafile") or die "Failed to create $datafile: $!\n";
      *STDERR=*NESSUS;
      *STDOUT=*NESSUS;

   print NESSUS $scandata;

   close NESSUS;




}

sub build_target_section {
   my ( $rep_id ) = @_;

   my $section = "";

   my $sql = qq{ SELECT distinct(hostip) FROM nessus_results WHERE report_id='$rep_id' };
   logwriter( $sql, 5 );
   my $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );

   $section = "<Targets>\n";

   while ( my ( $hostip ) = $sth_sel->fetchrow_array ) {
      $section .= "<Target>
<selected>yes</selected>
<type>hostname</type>
<value>$hostip</value>
</Target>\n";
   }

   $section .= "</Targets>\n";
   return $section;
}

sub build_policy_section {
   my ( $rep_id ) = @_;

   my $section = "";

   my $sql = qq{ SELECT t2.id, t2.name, t2.description FROM nessus_reports t1
      LEFT JOIN nessus_settings t2 on t1.sid=t2.id WHERE t1.report_id='$rep_id' };
   my $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   my ( $sid, $policy_name, $policy_description )= $sth_sel->fetchrow_array ( );

   $section = "<Policies>
<Policy>
<policyName>$policy_name</policyName>
<policyComments>$policy_description</policyComments>
<uuid></uuid>

<Preferences>
";
   $section .= build_server_preferences ( $sid );
   $section .= build_plugin_preferences ( $sid );
   $section .= "</Preferences>\n<PluginSelection>\n";
   $section .= build_plugin_families ( $sid );
   $section .= build_plugin_selection ( $sid );
   $section .= 
"</PluginSelection>
</Policy>
</Policies>\n";

   return $section;
}

sub build_report_section {
   my ( $rep_id ) = @_;

   my $section = "";

   my $hostname = qx/hostname/;
   my $datetime = localtime();

   my $sql = qq{ SELECT t1.report_id, t2.name, sid, DATE_FORMAT(t1.scantime, '%Y%m%d'),
         DATE_FORMAT(t1.scantime, '%a %b %d %H:%i:%s %Y'),
         DATE_FORMAT(DATE_SUB(t1.scantime, INTERVAL t2.iScantime minute), '%a %b %d %H:%i:%s %Y')
      FROM nessus_reports t1
        LEFT JOIN nessus_report_stats t2 on t1.report_id=t2.report_id
        WHERE t1.report_id = '$rep_id' };
   my $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );
   my ( $report_id, $rname, $sid, $scantime, $rep_end, $rep_start ) = $sth_sel->fetchrow_array(  );
   my $subTemp = $scantime."_".$rname;

   $subTemp =~ s/\//_/g;
   $subTemp =~ s/\s-\s/_/g;
   $subTemp =~ s/\s+/_/g;
   $subTemp =~ s/\./_/g;

   my $year = substr($scantime,0,4);
   my $month= substr($scantime,4,2);

   if( ! -e "${datadir}/$year" ) { mkdir ( "${datadir}/$year" ); }
   if( ! -e "${datadir}/$year/$month" ) { mkdir ( "${datadir}/$year/$month" ); }

   my $datafile = "${datadir}/$year/$month/$subTemp.nessus";

   unlink $datafile if -e $datafile;            # REMOVE ANY PRE-EXISTING Results File

   $section = "<Report>
<ReportName>$rname</ReportName>
<StartTime>$rep_start</StartTime>
<StopTime>$rep_end</StopTime>
<PolicyUUID>31323135-3731-3139-3832-756e77673030</PolicyUUID>
<Policy>
<policyName>$rname</policyName>
<policyComments>This policy was generated from the database on the system $hostname running Inprotect on $datetime.</policyComments>
<uuid>31323135-3731-3139-3832-756e77673030</uuid>
<Preferences>
";
   $section .= build_server_preferences ( $sid );
   $section .= build_plugin_preferences ( $sid );
   $section .= "</Preferences>\n<PluginSelection>\n";
   $section .= build_plugin_families ( $sid );
   $section .= build_plugin_selection ( $sid );
   $section .= 
"</PluginSelection>
</Policy>\n";
   $section .= build_target_section ( $rep_id );
   $section .= build_pluginlist_selection ( $rep_id );
   $section .= build_report_hosts ( $rep_id );
   $section .= "</Report>
</NessusClientData>\n";
   return $section;
}

sub build_server_preferences {
   my ( $sid ) = @_;

   my ($sth_sel, $sql, $nessus_id, $nessus_value);

   my $section = "<ServerPreferences>\n";

   # get SERVER_PREFS
   $sql = qq{ SELECT nessus_id, value FROM nessus_settings_preferences
       WHERE category='SERVER_PREFS' AND sid=$sid };
   $sth_sel=$dbh->prepare( $sql );
   $sth_sel->execute;

   while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
      $section .= 
"<preference>
<name>$nessus_id</name>
<value>$nessus_value</value>
</preference>\n";

   }

   $section .= "</ServerPreferences>\n";

   return $section;

}

sub build_plugin_preferences {
   my ( $sid ) = @_;

   my ($sth_sel, $sql, $nessus_id, $nessus_value);

   my $section = "<PluginsPreferences>\n";

   # get SERVER_PREFS
   $sql = qq{ SELECT nessus_id, value FROM nessus_settings_preferences WHERE category='PLUGINS_PREFS' AND sid=$sid };
   $sth_sel=$dbh->prepare( $sql );
   $sth_sel->execute;

   while (($nessus_id, $nessus_value)=$sth_sel->fetchrow_array ) {
#      print "nessus_id=$nessus_id\n";
      my @temp=split(/:/,$nessus_id);
#      print "tmp1=$temp[0]\ttmp2=$temp[1]\n";
      my $prefName = $temp[1];
      my @temp2=split(/\[/,$temp[0]);
#      print "tmp1=$temp2[0]\ttmp2=$temp2[1]\n";
      my $pluginName = $temp2[0];
      my $prefType = $temp2[1];
      $prefType =~ s/\]//;


      $section .= 
"<item>
<fullName>$nessus_id = $nessus_value</fullName>
<preferenceName>$prefName</preferenceName>
<pluginName>$pluginName</pluginName>
<preferenceType>$prefType</preferenceType>
<preferenceValue>$nessus_value</preferenceValue>
<selectedValue>$nessus_value</selectedValue>
</item>\n";
   }

   $section .= "</PluginsPreferences>\n";
   return $section;

}

sub build_plugin_families {
   my ( $sid ) = @_;

   my ($sth_sel, $sql, $nessus_id, $nessus_value);

   my $section = "<FamilySelection>\n";

   # get FAMILY STATUS
   $sql = qq{ SELECT t2.name, t1.status FROM nessus_settings_family t1
      LEFT JOIN nessus_family t2 ON t2.id=t1.fid
      WHERE t2.name<>"Port scanners" AND t1.sid=$sid ORDER BY t2.id };

   $sth_sel=$dbh->prepare( $sql );
   $sth_sel->execute;

   while ( my ($famName, $famStatus)=$sth_sel->fetchrow_array ) {
      $section .= 
"<FamilyItem>
<FamilyName>$famName</FamilyName>
<Status>partial</Status>
</FamilyItem>\n";

   }

   $section .= "</FamilySelection>\n";

   return $section;

}

sub build_plugin_selection {
   my ( $sid ) = @_;

   my ( $sth_sel, $sql );

   my $section = "<IndividualPluginSelection>\n";

   # get PLUGIN SET
   $sql = qq{ SELECT t1.id, t2.name, t3.name from nessus_settings_plugins t1
      LEFT JOIN plugins t2 ON t1.id=t2.id
      LEFT JOIN nessus_family t3 ON t3.id=t1.family
        WHERE t1.sid='$sid' AND t2.category <>'scanner' AND t1.enabled ='Y' ORDER BY id };
   $sth_sel=$dbh->prepare( $sql );
   $sth_sel->execute;
   #print PROFILE "\n\nbegin(PLUGIN_SET)\n";

   while ( my ( $plug_id, $plugName, $plugFamily ) = $sth_sel->fetchrow_array ) {
      $section .= 
"<PluginItem>
<PluginId>$plug_id</PluginId>
<PluginName>$plugName</PluginName>
<Family>$plugFamily</Family>
<Status>enabled</Status>
</PluginItem>\n";

   }

   $section .= "</IndividualPluginSelection>\n";

   return $section;

}

sub build_pluginlist_selection {
   my ( $sid ) = @_;

   my ( $sth_sel, $sql );

   my $section = "<PluginSelection>\n";

   # get PLUGIN SET
   $sql = qq{ SELECT t1.id FROM nessus_settings_plugins t1
      LEFT JOIN plugins t2 ON t1.id=t2.id
        WHERE t1.sid='$sid' AND t2.category <>'scanner' AND t1.enabled ='Y' ORDER BY id };
   $sth_sel=$dbh->prepare( $sql );
   $sth_sel->execute;
   #print PROFILE "\n\nbegin(PLUGIN_SET)\n";

   while ( my ( $plug_id  ) = $sth_sel->fetchrow_array ) {
      $section .= "$plug_id;";
   }

   $section .= "</PluginSelection>\n";

   return $section;

}

sub build_report_hosts {
   my ( $rep_id ) = @_;

   my ( $sql, $sth_sel, $sth_sel2 );

   my $section = "";

   $sql = qq{ SELECT distinct(hostip), hostname FROM nessus_results WHERE report_id='$rep_id' };
   logwriter( $sql, 5 );
   $sth_sel = $dbh->prepare( $sql );
   $sth_sel->execute(  );

   while ( my ( $hostip, $hostname )= $sth_sel->fetchrow_array ) {

      my ( $netb_name, $dns_name, $os_name, $mac_name ) = "";

      if ( $hostname =~ /\./ ) {
         $dns_name = $hostname;
         my @temp=split(/\./,$hostname);
         $netb_name = $temp[0];
      } else {
         if ( defined( $hostname ) && $hostname ne "" ) {
            $netb_name = $hostname;
         } else { $netb_name = "(unknown)"; }
         $dns_name = "(unknown)";
      }

      $sql = qq{ SELECT t1.id, t1.os, t2.mac_address
         FROM hosts t1
            LEFT JOIN host_macs t2 on t1.id=t2.host_id
         WHERE t1.hostname='$hostname' };
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );
      my ( $hostid, $host_os, $host_mac ) = $sth_sel2->fetchrow_array( );

      if ( defined ( $host_mac ) && $host_mac ne "" ) { 
         $mac_name = $host_mac; 
      } else {
         $mac_name = "(unknown)";
      }

      if ( defined ( $host_os ) && $host_os ne "" ) { 
         $os_name = $host_os; 
      } else {
         $os_name = "(unknown)";
      }

      $sql = qq{ SELECT COUNT(DISTINCT(port)) FROM nessus_results
         WHERE report_id='$rep_id' AND hostip='$hostip' };
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );
      my ( $open_ports ) = $sth_sel2->fetchrow_array( );

      $sql = qq{ SELECT count(risk), risk FROM nessus_results
         WHERE report_id='$rep_id' AND hostip='$hostip' group by risk order by risk };
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );

      my ( $num_hi, $num_med, $num_lo ) = 0;

      while ( my ( $count, $risk )= $sth_sel2->fetchrow_array ) {
         if ( $risk <=2 ) {
            $num_hi += $count;
         } elsif ( $risk <=5 ) {
            $num_med += $count;
         } else {
            $num_lo += $count;
         }
      }

      if ( ! is_number( $num_lo  )) { $num_lo = 0;  }
      if ( ! is_number( $num_med )) { $num_med = 0; }
      if ( ! is_number( $num_hi  )) { $num_hi = 0;  }

      $section .=
"<ReportHost>
<HostName>$hostip</HostName>
<startTime />
<stopTime />
<netbios_name>$netb_name</netbios_name>
<mac_addr>$mac_name</mac_addr>
<dns_name>$dns_name</dns_name>
<os_name>$os_name</os_name>
<num_ports>$open_ports</num_ports>
<num_lo>$num_lo</num_lo>
<num_med>$num_med</num_med>
<num_hi>$num_hi</num_hi>\n";

      $sql = qq{ SELECT t1.service, t1.risk, t1.scriptid, t1.msg, t2.name 
         FROM nessus_results t1
            LEFT JOIN plugins t2 on t1.scriptid=t2.id
         WHERE t1.report_id='$rep_id' AND t1.hostip='$hostip' };
      logwriter( $sql, 5 );
      $sth_sel2 = $dbh->prepare( $sql );
      $sth_sel2->execute(  );
      while ( my ( $service, $risk, $scriptid, $msg, $pname )= $sth_sel2->fetchrow_array ) {

         $msg =~ s/\n/\\n/g;

         $section .=
"<ReportItem>
<port>$service</port>
<severity>$risk</severity>
<pluginID>$scriptid</pluginID>
<pluginName>$pname</pluginName>
<data>$msg</data>
</ReportItem>\n";
      }
$section .=
"</ReportHost>\n";
   }
   return $section;


}

#get current date/time
sub getCurrentDateTime {
    # VER: 2.0 MODIFIED: 4/03/08 20:35
    my ( $format ) = @_;

    my @days = qw(Su Mo Tu We Th Fr Sa);
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;

    if ( $format ) {
        switch (lc($format)) {
            case "datetime"        { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
            case "date"                { return sprintf("%04d%02d%02d",$year, $mon, $mday); }
            case "time"                { return sprintf("%02d%02d%02d", $hour, $min, $sec); }
            case "today"        { return $days[$wday]; }
            else                { return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec); }
        }
    } else {
        return sprintf("%04d%02d%02d%02d%02d%02d",$year, $mon, $mday, $hour, $min, $sec);
    }

}

#is this a num
sub is_number{
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my($n)=@_;

    if ( $n ) { 
        return ($n=~/^\d+$/);
    } else {
        return;
    }
}

#filter html special characters
sub htmlspecialchars {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my $tmpSTRmsg = $_[0];
    $tmpSTRmsg =~ s/&/&amp;/g;
    $tmpSTRmsg =~ s/\'/&#039;/g;
    $tmpSTRmsg =~ s/\"/&quot;/g;
    $tmpSTRmsg =~ s/</&lt;/g;
    $tmpSTRmsg =~ s/>/&gt;/g;
    return $tmpSTRmsg;
}

#read settings from db (overrides settings in file)
sub load_db_configs {
    # VER: 1.0 MODIFIED: 4/1/08 12:39
    my ($sth_sel);

    my $sql = qq{ SELECT settingName, settingValue FROM inprotect_settings };
    logwriter( $sql, 5 );
    $sth_sel=$dbh->prepare( $sql );
    $sth_sel->execute;

    while ( my ( $name,$value) = $sth_sel->fetchrow_array ) {
       if ( $name eq "mailSignature" ) { $value =~ s/&lt;br&gt;/\n/g; }
       if ( $name ne "") { $CONFIG{$name}=$value; }
    }

    $sth_sel->finish;
    return;
}

sub read_file {
    # VER: 1.0 MODIFIED: 6/27/08 11:36
    my ( $myfile ) = @_;
    my ( $string ) = "";

    open FILE, "$myfile" or die "Couldn't open file: $!"; 
    while (<FILE>){
	$string .= $_;
    }
    close FILE;

    return $string;

}

#read settings from file
sub load_configs {
    # VER: 1.1 MODIFIED: 4/12/07 9:17
    my ( $configfile ) = @_;

    my $noconfig=0;
    open(CONF,"<$configfile") || $noconfig++;
    my @CONFILE=<CONF>;
    close(CONF);
    if ($noconfig) { print localtime(time)." port_scan: No config.txt file found.\n"; }
    foreach my $line (@CONFILE) {
        $line =~ s/\#.*$//;
        chomp($line);
        $line =~ s/\s+$//;
        $line =~ s/^\s+//;
        if ($line eq "") { next; }
        my @temp=split(/=/,$line,2);
        if ($temp[0] ne "") { $CONFIG{$temp[0]}=$temp[1]; }
    }
    return;
}

#safe write code to help prevent complete job failure
sub safe_db_write {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql_insert, $specified_level ) = @_;

    logwriter( $sql_insert, $specified_level );
    eval {
        $dbh->do( $sql_insert );
    };
    warn "[$$] FAILED - $sql_insert\n" . $dbh->errstr . "\n\n" if ($@);

    if ( $@ ) { return 0; }
}

#safe to use when table structure may not be accurate, one in a while a select query is problematic
sub safe_db_query {
    # VER: 1.0 MODIFIED: 1/16/08 7:36
    my ( $sql, $specified_level ) = @_;
    logwriter( $sql, $specified_level );
    my ( $sth_sel );
    my @data = ();

    eval {
        $sth_sel = $dbh->prepare( $sql );
        $sth_sel->execute;

        @data= $sth_sel->fetchrow_array;
    };

    $sth_sel->finish;
    return @data;
}

#check db is up
sub check_dbOK {
    # VER: 1.1 MODIFIED: 11/26/07 10:08
    my $sql = "SELECT count( hostname ) FROM nessus_servers WHERE 1";

    eval {
            $dbh->do( $sql );
    };
    warn "[$$] FAILED - Connection Test\n" . $dbh->errstr . "\n\n" if ($@);
    if ( $@ ) { return 0; }
    return 1;
}

#routine to do log writing
sub logwriter {
   # VER: 1.0 MODIFIED: 4/21/08 20:19
    my ( $message, $specified_level ) = @_;

    if ( !defined($specified_level) || $specified_level eq "" ) { $specified_level = 5; }

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year+=1900;
    $mon++;
    my $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d",$year, $mon, $mday, $hour, $min, $sec);

    $message = "$now [$$] $loginfo{$specified_level} $message";

    if ( $debug || $log_level ge $specified_level )  { print $message ."\n"; }

}

#connect to db
sub conn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    $dbh = DBI->connect("$CONFIG{'DATABASEDSN'}:$CONFIG{'DATABASENAME'}:$CONFIG{'DATABASEHOST'}",
        "$CONFIG{'DATABASEUSER'}","$CONFIG{'DATABASEPASSWORD'}", {
        PrintError => 0,
        RaiseError => 1,
        AutoCommit => 1 } ) or die("Failed to connect : $DBI::errstr\n");
    return $dbh;
}

#disconnect from db
sub disconn_db {
    # VER: 1.0 MODIFIED: 3/29/07 13:03
    my ( $dbh ) = @_;
    $dbh->disconnect or die("Failed to disconnect : $DBI::errstr\n");
}
